



// ==================================================================
// Variable
var server_ip="10.42.0.1";
var image_prefix = "http://10.42.0.1:8080/stream?topic=";
var image_prefix_2 = ":8080/stream?topic=";
var image_surfix = "&type=ros_compressed";

var frames = "/basic_vision_node/image_out"
var vision_basic_parameter = {

    filter_h_min : 350,
    filter_h_max : 20,
    filter_s_min : 222,
    filter_s_max : 255,
    filter_v_min : 30,
    filter_v_max : 255,

}


// ==================================================================
// Connecting to ROS
// ==================================================================
var full_add = location.href;
var add_array = full_add.split("/");
console.log("address : " + add_array[2]);

change_server_ip(add_array[2]);

var ros = new ROSLIB.Ros({
    //url: 'ws://10.42.0.1:9090'
    url: 'ws://' + server_ip + ':9090'
});

ros.on('connection', function () {
    console.log('Connected to websocket server.');
});

ros.on('error', function (error) {
    console.log('Error connecting to websocket server: ', error);
});

ros.on('close', function () {
    console.log('Connection to websocket server closed.');
});

// ==================================================================
// Publishing a Topic
// ==================================================================

var demoModeCommand = new ROSLIB.Topic({
    ros: ros,
    name: '/robotis/mode_command',
    messageType: 'std_msgs/String'
});

function publishModeCommand(mode_command) {
    var message = new ROSLIB.Message({
        data: mode_command
    });
    demoModeCommand.publish(message);
    console.log('publish Mode Command : ' + mode_command);
}

var demoCommand = new ROSLIB.Topic({
    ros: ros,
    name: '/robotis/demo_command',
    messageType: 'std_msgs/String'
});

function publishDemoCommand(demo_command) {
    var message = new ROSLIB.Message({
        data: demo_command
    });
    demoCommand.publish(message);
    console.log('publish Demo Command : ' + demo_command);
}
var paramCommand = new ROSLIB.Topic({
    ros: ros,
    name: '/basic_vision_node/param_command',
    messageType: 'std_msgs/String'
});

function publishParamCommand(param_command) {
    var message = new ROSLIB.Message({
        data: param_command
    });
    paramCommand.publish(message);
    console.log('publish Parameter Command : ' + param_command);
}






// ==================================================================
// Subscribing to a Topic
// ==================================================================
var param_listener = new ROSLIB.Topic({
    ros: ros,
    name: '/basic_vision_node/current_params',
    messageType: 'basic_vision/BallDetectorParams'
});

param_listener.subscribe(function (message) {

    vision_basic_parameter.filter_h_min             = message.filter_h_min;
    vision_basic_parameter.filter_h_max             = message.filter_h_max;
    vision_basic_parameter.filter_s_min             = message.filter_s_min;
    vision_basic_parameter.filter_s_max             = message.filter_s_max;
    vision_basic_parameter.filter_v_min             = message.filter_v_min;
    vision_basic_parameter.filter_v_max             = message.filter_v_max;

    refresh_basic_vision_parameters();
    console.log('ball detector parameter is refreshed.');
});



// ==================================================================
// Calling the service
// ==================================================================
var setBallDetectorParamClient = new ROSLIB.Service({
    ros : ros,
    name : '/basic_vision_node/set_param',
    serviceType : 'basic_vision/SetParameters'
});

var getBallDetectorParamClient = new ROSLIB.Service({
    ros : ros,
    name : '/basic_vision_node/get_param',
    serviceType : 'basic_vision/GetParameters'
});

function callServiceSetBallDetectorParams(new_params) {
    var request = new ROSLIB.ServiceRequest({
        params : new_params
    });
    setBallDetectorParamClient.callService(request, function(result) {
        console.log('Result for service call : set ball detector params ' + result.returns.gaussian_blur_size);
    });
}

function callServiceGetBallDetectorParams() {
    var request = new ROSLIB.ServiceRequest();
    getBallDetectorParamClient.callService(request, function(result) {

        vision_basic_parameter.filter_h_min             = result.returns.filter_h_min;
        vision_basic_parameter.filter_h_max             = result.returns.filter_h_max;
        vision_basic_parameter.filter_s_min             = result.returns.filter_s_min;
        vision_basic_parameter.filter_s_max             = result.returns.filter_s_max;
        vision_basic_parameter.filter_v_min             = result.returns.filter_v_min;
        vision_basic_parameter.filter_v_max             = result.returns.filter_v_max;

        refresh_basic_vision_parameters();
    });
}




// ==================================================================
// Getting and setting a param value
// ==================================================================
/**
 * Setup all visualization elements when the page is loaded.
 */

function init() {
    console.log('called init function');
    console.log("server ip : " + server_ip);

}

function change_mode(value) {
    console.log('Mode: ' + value);
    window.document.getElementById("mode_text").innerText = "Mode : " + value;
    publishModeCommand(value);
}

function send_demo_command(value) {
    console.log('send demo command : ' + value);
    publishDemoCommand(value);
}

function id(e){return document.getElementById(e)}

// ==================================================================
// UI - Tab
// ==================================================================


$(document).ready(function () {
    window.document.getElementById("camera_feed_back").src = "http://" + server_ip + image_prefix_2 + frames + image_surfix;
})


$( function() {
    $("#input_server_ip").val(server_ip);
});

// ==================================================================
// UI - ball detector / color filter
// ==================================================================
$( function() {
    $( "#h_min_slider" ).slider({
        value: 350,
        min: 0,
        max: 359,
        step : 1,
        slide: function (event, ui) {
            $("#h_min").val(ui.value);
            vision_basic_parameter['filter_h_min'] = ui.value;
        },
        stop: function (event, ui) {
            change_basic_vision_parameters();
        }
    });
    $("#h_min").val($("#h_min_slider").slider("value"));
    $( "#h_min" ).on( "change", function() {
        $( "#h_min_slider" ).slider( "value", this.value );

        var target = $("#h_min_slider").slider("value");
        vision_basic_parameter['filter_h_min'] = target;
        change_basic_vision_parameters();
    });
});

$( function() {
    $( "#h_max_slider" ).slider({
        value: 20,
        min: 0,
        max: 359,
        step : 1,
        slide: function (event, ui) {
            $("#h_max").val(ui.value);
            vision_basic_parameter['filter_h_max'] = ui.value;
        },
        stop: function (event, ui) {
            change_basic_vision_parameters();
        }
    });
    $("#h_max").val($("#h_max_slider").slider("value"));
    $( "#h_max" ).on( "change", function() {
        $( "#h_max_slider" ).slider( "value", this.value );

        var target = $("#h_max_slider").slider("value");
        vision_basic_parameter['filter_h_max'] = target;
        change_basic_vision_parameters();
    });
});

$( function() {
    $( "#s_min_slider" ).slider({
        value: 222,
        min: 0,
        max: 255,
        step : 1,
        slide: function (event, ui) {
            $("#s_min").val(ui.value);
            vision_basic_parameter['filter_s_min'] = ui.value;
        },
        stop: function (event, ui) {
            change_basic_vision_parameters();
        }
    });
    $("#s_min").val($("#s_min_slider").slider("value"));
    $( "#s_min" ).on( "change", function() {
        $( "#s_min_slider" ).slider( "value", this.value );

        var target = $("#s_min_slider").slider("value");
        vision_basic_parameter['filter_s_min'] = target;
        change_basic_vision_parameters();
    });
});

$( function() {
    $( "#s_max_slider" ).slider({
        value: 255,
        min: 0,
        max: 255,
        step : 1,
        slide: function (event, ui) {
            $("#s_max").val(ui.value);
            vision_basic_parameter['filter_s_max'] = ui.value;
        },
        stop: function (event, ui) {
            change_basic_vision_parameters();
        }
    });
    $("#s_max").val($("#s_max_slider").slider("value"));
    $( "#s_max" ).on( "change", function() {
        $( "#s_max_slider" ).slider( "value", this.value );

        var target = $("#s_max_slider").slider("value");
        vision_basic_parameter['filter_s_max'] = target;
        change_basic_vision_parameters();
    });
});

$( function() {
    $( "#v_min_slider" ).slider({
        value: 30,
        min: 0,
        max: 255,
        step : 1,
        slide: function (event, ui) {
            $("#v_min").val(ui.value);
            vision_basic_parameter['filter_v_min'] = ui.value;
        },
        stop: function (event, ui) {
            change_basic_vision_parameters();
        }
    });
    $("#v_min").val($("#v_min_slider").slider("value"));
    $( "#v_min" ).on( "change", function() {
        $( "#v_min_slider" ).slider( "value", this.value );

        var target = $("#v_min_slider").slider("value");
        vision_basic_parameter['filter_v_min'] = target;
        change_basic_vision_parameters();
    });
});

$( function() {
    $( "#v_max_filter" ).slider({
        value: 255,
        min: 0,
        max: 255,
        step : 1,
        slide: function (event, ui) {
            $("#v_max").val(ui.value);
            vision_basic_parameter['filter_v_max'] = ui.value;
        },
        stop: function (event, ui) {
            change_basic_vision_parameters();
        }
    });
    $("#v_max").val($("#v_max_filter").slider("value"));
    $( "#v_max" ).on( "change", function() {
        $( "#v_max_filter" ).slider( "value", this.value );

        var target = $("#v_max_filter").slider("value");
        vision_basic_parameter['filter_v_max'] = target;
        change_basic_vision_parameters();
    });
});

function change_debug_image(value) {
    if(value == "true")
    {
        console.log('change image to debug');
        publishParamCommand('debug');
    }
    else
    {
        console.log('change image to normal');
        publishParamCommand('normal');
    }
}

function send_basic_vision_reset() {
    console.log('send ball detector parameter reset command');
    publishParamCommand('reset');
}

function change_server_ip() {
    server_ip = $("#input_server_ip").val();
    console.log("ip changed - " + server_ip);
}

function change_server_ip(ip_address) {
    server_ip = ip_address;
    $("#input_server_ip").val(server_ip);
    console.log("ip changed : " + server_ip);
}

function change_basic_vision_parameters() {
    for(p in vision_basic_parameter)
        console.log(p + ", " + vision_basic_parameter[p]);

    callServiceSetBallDetectorParams(vision_basic_parameter);
}

function refresh_basic_vision_parameters() {



    $("#h_min").val(vision_basic_parameter.filter_h_min );
    $("#h_max").val(vision_basic_parameter.filter_h_max );
    $("#s_min").val(vision_basic_parameter.filter_s_min );
    $("#s_max").val(vision_basic_parameter.filter_s_max );
    $("#v_min").val(vision_basic_parameter.filter_v_min );
    $("#v_max").val(vision_basic_parameter.filter_v_max );



    $( "#h_min_slider" ).slider( "value", vision_basic_parameter.filter_h_min );
    $( "#h_max_slider" ).slider( "value", vision_basic_parameter.filter_h_max );
    $( "#s_min_slider" ).slider( "value", vision_basic_parameter.filter_s_min );
    $( "#s_max_slider" ).slider( "value", vision_basic_parameter.filter_s_max );
    $( "#v_min_slider" ).slider( "value", vision_basic_parameter.filter_v_min );
    $( "#v_max_filter" ).slider( "value", vision_basic_parameter.filter_v_max );


    for(p in vision_basic_parameter)
        console.log(p + ", " + vision_basic_parameter[p]);
    console.log('refresh ball detector params');
}



