// ==================================================================
// Variable
var server_ip = "10.42.0.1";
var image_prefix = "http://10.42.0.1:8080/stream?topic=";
var image_prefix_2 = ":8080/stream?topic=";
var image_surfix = "&type=ros_compressed";
// var images = new Array("/ball_detector_node/image_out", "/facerec/image_raw", "/usb_cam_node/image_raw");
var frame_stream = "/ball_detector_node/image_out";
var ball_parameters = {
    gaussian_blur_size: 7,
    gaussian_blur_sigma: 2.0,
    canny_edge_th: 100,
    hough_accum_resolution: 1.0,
    hough_accum_th: 28.0,
    min_circle_dist: 100,
    min_radius: 20,
    max_radius: 300,
    filter_h_min: 350,
    filter_h_max: 20,
    filter_s_min: 222,
    filter_s_max: 255,
    filter_v_min: 30,
    filter_v_max: 255,
    ellipse_size: 2
};
var robotStates = {
    voltage: 10.0
};


// ==================================================================
// Connecting to ROS
// ==================================================================
var full_add = location.href;
var add_array = full_add.split("/");
console.log("address : " + add_array[2]);

change_server_ip(add_array[2]);

var ros = new ROSLIB.Ros({
    //url: 'ws://10.42.0.1:9090'
    url: 'ws://' + server_ip + ':9090'
});

ros.on('connection', function () {
    console.log('Connected to websocket server.');
});

ros.on('error', function (error) {
    console.log('Error connecting to websocket server: ', error);
});

ros.on('close', function () {
    console.log('Connection to websocket server closed.');
});

// ==================================================================
// Publishing a Topic
// ==================================================================

var paramCommand = new ROSLIB.Topic({
    ros: ros,
    name: '/ball_detector_node/param_command',
    messageType: 'std_msgs/String'
});

function publishParamCommand(param_command) {
    var message = new ROSLIB.Message({
        data: param_command
    });
    paramCommand.publish(message);
    console.log('publish Parameter Command : ' + param_command);
}


// ==================================================================
// Subscribing to a Topic
// ==================================================================
var param_listener = new ROSLIB.Topic({
    ros: ros,
    name: '/ball_detector_node/current_params',
    messageType: 'ball_detector/BallDetectorParams'
});

param_listener.subscribe(function (message) {

    ball_parameters.filter_h_min = message.filter_h_min;
    ball_parameters.filter_h_max = message.filter_h_max;
    ball_parameters.filter_s_min = message.filter_s_min;
    ball_parameters.filter_s_max = message.filter_s_max;
    ball_parameters.filter_v_min = message.filter_v_min;
    ball_parameters.filter_v_max = message.filter_v_max;

    refresh_ball_detector_parameters();
    console.log('ball detector parameter is refreshed.');
});

var voltage_listener = new ROSLIB.Topic({
    ros: ros,
    name: "/robotis/open_cr/voltage",
    messageType: 'robotis_controller_msgs/VoltageMsg'
});

voltage_listener.subscribe(function (message) {
    robotStates.voltage = message.voltage;
    window.document.getElementById("voltage").innerHTML = "battery: " + robotStates.voltage;
});



// ==================================================================
// Calling the service
// ==================================================================
var setBallDetectorParamClient = new ROSLIB.Service({
    ros: ros,
    name: '/ball_detector_node/set_param',
    serviceType: 'ball_detector/SetParameters'
});

var getBallDetectorParamClient = new ROSLIB.Service({
    ros: ros,
    name: '/ball_detector_node/get_param',
    serviceType: 'ball_detector/GetParameters'
});

function callServiceSetBallDetectorParams(new_params) {
    var request = new ROSLIB.ServiceRequest({
        params: new_params
    });
    setBallDetectorParamClient.callService(request, function (result) {
        console.log('Result for service call : set ball detector params ' + result.returns.gaussian_blur_size);
    });
}

function callServiceGetBallDetectorParams() {
    var request = new ROSLIB.ServiceRequest();
    getBallDetectorParamClient.callService(request, function (result) {

        ball_parameters.filter_h_min = result.returns.filter_h_min;
        ball_parameters.filter_h_max = result.returns.filter_h_max;
        ball_parameters.filter_s_min = result.returns.filter_s_min;
        ball_parameters.filter_s_max = result.returns.filter_s_max;
        ball_parameters.filter_v_min = result.returns.filter_v_min;
        ball_parameters.filter_v_max = result.returns.filter_v_max;

        refresh_ball_detector_parameters();

    });
}


// ==================================================================
// Getting and setting a param value
// ==================================================================
/**
 * Setup all visualization elements when the page is loaded.
 */

function init() {
    console.log('called init function');
    console.log("server ip : " + server_ip);
}


function id(e) {
    return document.getElementById(e)
}

$(document).ready(function () {

    window.document.getElementById("tab-2-image").src = "http://" + server_ip + image_prefix_2 + frame_stream + image_surfix;
    // window.document.getElementById("voltage").innerHTML = "battery: "+robotStates.voltage;

    callServiceGetBallDetectorParams();
});


// ==================================================================
// UI - ball detector / color filter
// ==================================================================
$(function () {
    $("#filter_h_min_slider").slider({
        value: 350,
        min: 0,
        max: 359,
        step: 1,
        slide: function (event, ui) {
            $("#filter_h_min_value").val(ui.value);
            ball_parameters['filter_h_min'] = ui.value;
        },
        stop: function (event, ui) {
            change_ball_detector_parameters();
        }
    });
    $("#filter_h_min_value").val($("#filter_h_min_slider").slider("value"));
    $("#filter_h_min_value").on("change", function () {
        $("#filter_h_min_slider").slider("value", this.value);

        var target = $("#h_min_slider").slider("value");
        ball_parameters['filter_h_min'] = target;
        change_ball_detector_parameters();
    });
});

$(function () {
    $("#filter_h_max_slider").slider({
        value: 20,
        min: 0,
        max: 359,
        step: 1,
        slide: function (event, ui) {
            $("#filter_h_max_value").val(ui.value);
            ball_parameters['filter_h_max'] = ui.value;
        },
        stop: function (event, ui) {
            change_ball_detector_parameters();
        }
    });
    $("#filter_h_max_value").val($("#filter_h_max_slider").slider("value"));
    $("#filter_h_max_value").on("change", function () {
        $("#filter_h_max_slider").slider("value", this.value);

        var target = $("#filter_h_max_slider").slider("value");
        ball_parameters['filter_h_max'] = target;
        change_ball_detector_parameters();
    });
});

$(function () {
    $("#filter_s_min_slider").slider({
        value: 222,
        min: 0,
        max: 255,
        step: 1,
        slide: function (event, ui) {
            $("#filter_s_min_value").val(ui.value);
            ball_parameters['filter_s_min'] = ui.value;
        },
        stop: function (event, ui) {
            change_ball_detector_parameters();
        }
    });
    $("#filter_s_min_value").val($("#filter_s_min_slider").slider("value"));
    $("#filter_s_min_value").on("change", function () {
        $("#filter_s_min_slider").slider("value", this.value);

        var target = $("#filter_s_min_slider").slider("value");
        ball_parameters['filter_s_min'] = target;
        change_ball_detector_parameters();
    });
});

$(function () {
    $("#filter_s_max_slider").slider({
        value: 255,
        min: 0,
        max: 255,
        step: 1,
        slide: function (event, ui) {
            $("#filter_s_max_value").val(ui.value);
            ball_parameters['filter_s_max'] = ui.value;
        },
        stop: function (event, ui) {
            change_ball_detector_parameters();
        }
    });
    $("#filter_s_max_value").val($("#filter_s_max_slider").slider("value"));
    $("#filter_s_max_value").on("change", function () {
        $("#filter_s_max_slider").slider("value", this.value);

        var target = $("#filter_s_max_slider").slider("value");
        ball_parameters['filter_s_max'] = target;
        change_ball_detector_parameters();
    });
});

$(function () {
    $("#filter_v_min_slider").slider({
        value: 30,
        min: 0,
        max: 255,
        step: 1,
        slide: function (event, ui) {
            $("#filter_v_min_value").val(ui.value);
            ball_parameters['filter_v_min'] = ui.value;
        },
        stop: function (event, ui) {
            change_ball_detector_parameters();
        }
    });
    $("#filter_v_min_value").val($("#filter_v_min_slider").slider("value"));
    $("#filter_v_min_value").on("change", function () {
        $("#filter_v_min_slider").slider("value", this.value);

        var target = $("#filter_v_min_slider").slider("value");
        ball_parameters['filter_v_min'] = target;
        change_ball_detector_parameters();
    });
});

$(function () {
    $("#filter_v_max_slider").slider({
        value: 255,
        min: 0,
        max: 255,
        step: 1,
        slide: function (event, ui) {
            $("#filter_v_max_value").val(ui.value);
            ball_parameters['filter_v_max'] = ui.value;
        },
        stop: function (event, ui) {
            change_ball_detector_parameters();
        }
    });
    $("#filter_v_max_value").val($("#filter_v_max_slider").slider("value"));
    $("#filter_v_max_value").on("change", function () {
        $("#filter_v_max_slider").slider("value", this.value);

        var target = $("#filter_v_max_slider").slider("value");
        ball_parameters['filter_v_max'] = target;
        change_ball_detector_parameters();
    });
});

function change_debug_image(value) {
    if (value == "true") {
        console.log('change image to debug');
        publishParamCommand('debug');
    } else {
        console.log('change image to normal');
        publishParamCommand('normal');
    }
}

function send_ball_detector_reset() {
    console.log('send ball detector parameter reset command');
    publishParamCommand('reset');
}

function change_server_ip() {
    server_ip = $("#input_server_ip").val();
    console.log("ip changed - " + server_ip);
}

function change_server_ip(ip_address) {
    server_ip = ip_address;
    $("#input_server_ip").val(server_ip);
    console.log("ip changed : " + server_ip);
}


function change_ball_detector_parameters() {
    for (p in ball_parameters)
        console.log(p + ", " + ball_parameters[p]);

    callServiceSetBallDetectorParams(ball_parameters);
}

function refresh_ball_detector_parameters() {


    $("#filter_h_min_value").val(ball_parameters.filter_h_min);
    $("#filter_h_max_value").val(ball_parameters.filter_h_max);
    $("#filter_s_min_value").val(ball_parameters.filter_s_min);
    $("#filter_s_max_value").val(ball_parameters.filter_s_max);
    $("#filter_v_min_value").val(ball_parameters.filter_v_min);
    $("#filter_v_max_value").val(ball_parameters.filter_v_max);


    $("#filter_h_min_slider").slider("value", ball_parameters.filter_h_min);
    $("#filter_h_max_slider").slider("value", ball_parameters.filter_h_max);
    $("#filter_s_min_slider").slider("value", ball_parameters.filter_s_min);
    $("#filter_s_max_slider").slider("value", ball_parameters.filter_s_max);
    $("#filter_v_min_slider").slider("value", ball_parameters.filter_v_min);
    $("#filter_v_max_slider").slider("value", ball_parameters.filter_v_max);


    for (p in ball_parameters)
        console.log(p + ", " + ball_parameters[p]);
    console.log('refresh ball detector params');
}


//possible use in future
function initialTrackbars(name, min, max, value, step) {
    $("#" + name + "_slider").slider({
        value: value,
        min: min,
        max: max,
        step: step,
        slide: function (event, ui) {
            $("#" + name + "_value").val(ui.value);
            ball_parameters['filter_v_max'] = ui.value;
        },
        stop: function (event, ui) {
            change_ball_detector_parameters();
        }
    });
    $("#" + name + "_value").val($("#" + name + "_slider").slider("value"));
    $("#" + name + "_value").on("change", function () {
        $("#" + name + "_slider").slider("value", this.value);

        var target = $("#" + name + "_slider").slider("value");
        ball_parameters['filter_v_max'] = target;
        change_ball_detector_parameters();
    });
}
