#!/usr/bin/env python

from enum import Enum
from time import sleep

import time
import math
import rosnode
import rospy
from std_msgs.msg import String

from control_modules.control_module import ControlModule
from vision.msg import PositionArea
from archery_target import Archery_Target

import roslib.packages as rospkg
import os
import json
import numpy as np

'''
Logic description

    step 1
        I. compare colour area sizes, if within an acceptible error use to determine centre

    step 2
        I. enter timing loop.
       II. record centre of target and start timing
      III. sleep 1 to avoid false timing
       IV. watch until centre is within del+-error of start
        V. record end time
    
    step 3
        I. wait until target centre is within del+-error of fireing position
       II. once at fireing position wait one full cycle - fireing time
      III. fire
'''
class RobotMode(Enum):
    READY = 0
    RUNNING = 1


class Movements(Enum):
    MARCH = 0
    FORWARD = 1
    BACKWARD = 2
    TURN_LEFT = 3
    TURN_RIGHT = 4
    ACTION = 5

class State(Enum):
    WAIT_ANALYZE = 0
    ANALYZE = 1
    WAIT_FIRE = 2
    FIRE = 3


def button_callack(msg):
    global robot_mode, mode_change

    if msg.data == 'start':
        # Start button pressed
        # Send robot to RUNNING mode
        if robot_mode == RobotMode.READY:
            robot_mode = RobotMode.RUNNING
            mode_change = True
        elif robot_mode == RobotMode.RUNNING:
            rospy.logerr('Robot already in running mode.')

    elif msg.data == 'mode':
        # Mode button pressed
        # Send robot to READY mode
        robot_mode = RobotMode.READY
        mode_change = True


def wait_for_node(node_name):
    # Wait for node to start
    while node_name not in rosnode.get_node_names():
        rospy.logwarn('Node is not running: {0}'.format(node_name))
        sleep(1)

    rospy.loginfo('Node is running: {0}'.format(node_name))


def check_area(red,yellow,blue, error=1e-2):
    total_area = red[0]+yellow[0]+blue[0]

    result = set()

    if (red[0]/total_area) >= red[1]-error and (red[0]/total_area) <= red[1]+error:
        result.add('red')
    if (yellow[0]/total_area) >= yellow[1]-error and (yellow[0]/total_area) <= yellow[1]+error:
        result.add('yellow')
    if (blue[0]/total_area) >= blue[1]-error and (blue[0]/total_area) <= blue[1]+error:
        result.add('blue')
    result.add('red')
    return result
def find_centre(colour_list):
    #are obstacles global?
    l = len(colour_list)
    if l == 0:
        return (-1.0,-1.0)
    x = 0.0
    y = 0.0

    if 'red' in colour_list:
        x += obstacles.red.x
        y += obstacles.red.y
    if 'yellow' in colour_list:
        x += obstacles.yellow.x
        y += obstacles.yellow.y
    if 'blue' in colour_list:
        x += obstacles.blue.x
        y += obstacles.blue.y
    
    x = x/l
    y = y/l

    return (x,y)
def get_circle_3pt(alpha, beta, kappa):
    # stationary target
    if alpha == beta or alpha == kappa:
        return alpha, 0
    if beta == kappa:
        return beta, 0
    # swap if y are same
    def swap (j,k):
        return k,j
    if alpha[1] == beta[1]:
        alpha, kappa = swap(alpha, kappa)
    elif alpha[1] == kappa[1]:
        alpha, kappa = swap(alpha, beta)
    # moving target
    mid_ab = ((alpha[0]+beta[0])/2, (alpha[1]+beta[1])/2)
    mid_ak = ((alpha[0]+kappa[0])/2, (alpha[1]+kappa[1])/2)

    slope_ab = (alpha[1]-beta[1])/(alpha[0]-beta[0]
                                   ) if (alpha[0]-beta[0]) != 0 else float('inf')
    slope_ak = (alpha[1]-kappa[1])/(alpha[0]-kappa[0]
                                    ) if (alpha[0]-kappa[0]) != 0 else float('inf')

    if slope_ab == 0 or slope_ak == 0:
        print(alpha, beta, kappa)
    if slope_ab == float('inf'):
        s_ab = 0
    elif slope_ab == 0:
        s_ab = float('inf')
    else:
        s_ab = -1./slope_ab

    if slope_ak == float('inf'):
        s_ak = 0
    elif slope_ak == 0:
        s_ak = float('inf')
    else:
        s_ak = -1./slope_ak


    # y - sx = y0 - sx0
    # let y0 - sx0 = C
    # y = C + sx
    # x = delta C_12 / del s_21

    C_ab = mid_ab[1]-s_ab*mid_ab[0]
    C_ak = mid_ak[1]-s_ak*mid_ak[0]


    x = (C_ak - C_ab)/(s_ab - s_ak)
    y = s_ab*x+C_ab

    center = (x, y)
    radius = distance(center, alpha)

    return center, radius
def reorient(c):
    c_x = c[0]

    # align_to_x = align robot so that c_x ~ x
    # turn_dist = amount robot turns per shift action
    turns = int((c_x - centre_x)//turn_dist)
    if robot_size == 'adult':
        s = 1.2
    elif robot_size == 'kid':
        s = 0.8
    for t in range(abs(turns)):
        # check sign convention
        if turns > 0:
            # turn right
            control_module.walking().walk(gait='forward')
            sleep(s)
            pass
        else:
            # turn left
            control_module.walking().walk(gait='backward')
            sleep(s)
            pass

    control_module.walking().stop()
    return


def distance(a, b):
    return math.sqrt((a[0]-b[0])**2+(a[1]-b[1])**2)


def start_archery():
    global mode_change, control_module, rate, fired, state

    print 'Start event!'
    # control_module.action().play_action('archery_sides') ##CHECK READY

    start = -1
    # total_time vs time to centre
    total_time = -1
    time_to_center = -1
    range_center = []


    # kid size target for testing
    area_ratio_error = 1e-2
    red_area_ratio = 0.04
    yellow_area_ratio = 0.32
    blue_area_ratio = 0.64

    use_colours = check_area((obstacles.red.area, red_area_ratio),
                (obstacles.yellow.area, yellow_area_ratio),
                (obstacles.blue.area, blue_area_ratio),
                area_ratio_error)
    while len(use_colours) == 0:
        print('Cannot resolve target')


    
    # sleep(5)

    timing = False
    start_position = None
    mid_position = None
    end_position = None
    time_list = []
    center_list = []
    avg_count = 6
    timing_position = None
    while not rospy.is_shutdown() and not mode_change:
        cur = find_centre(use_colours)
        if not timing and cur != (-1,-1):
            c = avg_count
            if robot_size == 'adult':
                control_module.action().play_action("archery_walk")
                sleep(1)
            while c > 0:
                cur = find_centre(use_colours)
                if cur != (-1,-1):
                    c-=1
                    if start_position == None:
                        start_position = cur
                    elif mid_position == None:
                        mid_position = cur
                    elif end_position == None:
                        end_position = cur
                    else:
                        center, radius = get_circle_3pt(
                            start_position,
                            mid_position,
                            end_position)
                        start_position = None
                        mid_position = None
                        end_position = None
                        center_list.append(center)
                sleep(0.5)
            timing = True
            center_avg = np.mean(center_list, axis=0)
            reorient(center_avg)
            if robot_size == 'adult':
                control_module.action().play_action("archery_ready")
            sleep(3)
        else:
            if state == State.WAIT_ANALYZE and cur != (-1,-1):
                
                print('STATE', state)
                state = State.ANALYZE
                start = time.time()
                timing_position = cur
                sleep(1)
            elif state == State.ANALYZE:
                dist = distance(cur, timing_position)
                #print('dist:', dist, 'fire_error:', fire_error)
                if dist <= timing_error and timing:
                    end = time.time()
                    # total_time = 
                    time_list.append(end - start)
                    print('STATE', state)
                    if avg_count > 0:
                        avg_count -= 1
                        state = State.WAIT_ANALYZE
                    else:
                        total_time = np.mean(time_list)

                        state = State.WAIT_FIRE
                        print('total_time:',total_time)
                else:
                    print('STATE', state,'Target centre: {:.2f}, {:.2f}, Distance squares: {}'.format(cur[0], cur[1], dist))
            elif state == State.WAIT_FIRE:
               ### remove after testing
                #fire_position = cur
               ###
                time_to_center = total_time - fire_delay
                if time_to_center <= 0:
                	time_to_center = (total_time * fire_delay) - fire_delay
                print('time_to_center:',time_to_center)

                #reorient(center_avg)
                control_module.action().play_action('archery_draw')
                sleep(4)
                #print('STATE', state, 'Entering fire loop\ttime to center:', time_to_center)
                while distance(find_centre(use_colours), fire_position) > fire_error:
                    pass
                sleep(time_to_center)
                control_module.action().play_action('archery_release')
                rospy.loginfo('Firing')
                state = State.FIRE
            elif state == State.FIRE:
                break
            #else:
                #print('STATE', state,'Target centre: {:.2f}, {:.2f}'.format(cur[0], cur[1]))
    return


def set_variables():
    global timing_error, fire_delay, fire_error, fire_position, area_ratio_error, red_area_ratio, yellow_are_ratio, blue_area_ratio, turn_dist, centre_x
    config_variables_file = os.path.join(rospkg.get_pkg_dir('events'), 'config', event,
                                         'variables_{0}.json'.format(robot_size))

    vars = None
    with open(config_variables_file, 'r') as fp:
        vars = json.load(fp)

    timing_error = vars['timing_error']
    fire_delay = vars['fire_delay']
    fire_error = vars['fire_error']
    # replace with coordinates where arrow is aimed
    fire_position = vars['fire_position']
    area_ratio_error = vars['area_ratio_error']
    red_area_ratio = vars['red_area_ratio']
    yellow_area_ratio = vars['yellow_area_ratio']
    blue_area_ratio = vars['blue_area_ratio']

    turn_dist = vars['turn_dist']
    centre_x = vars['centre_x']

if __name__ == '__main__':
    global robot_size
    # Initalize global variables
    robot_mode = RobotMode.READY
    mode_change = False
    obstacles = Archery_Target()
    SPIN_RATE = 0.5
    event = 'archery'
    fired = False
    state = State.WAIT_ANALYZE

    robot_size = rospy.get_param('archery_node/size')

    set_variables() # Important !

    # Wait for other nodes to launch
    wait_for_node('/op3_manager')
    wait_for_node('/vision_node')

    # Initialze ROS node
    rospy.init_node('archery_node')
    rate = rospy.Rate(SPIN_RATE)

    # Initialize OP3 ROS generic subscriber
    button_sub = rospy.Subscriber('/robotis/open_cr/button', String, button_callack)

    # Initialze vision subscribers
    blue_sub = rospy.Subscriber('/vision_node/blue', PositionArea,
                                lambda msg: obstacles.blue.update(msg.x, msg.y, msg.position, msg.area))
    red_sub = rospy.Subscriber('/vision_node/red', PositionArea,
                               lambda msg: obstacles.red.update(msg.x, msg.y, msg.position, msg.area))
    yellow_sub = rospy.Subscriber('/vision_node/yellow', PositionArea,
                                        lambda msg: obstacles.yellow.update(msg.x, msg.y, msg.position, msg.area))

     # Initializing modules
    control_module = ControlModule(event, robot_size)

    # Enabling walking module
    sleep(10)

    if robot_size == 'adult':
        control_module.action().play_action("archery_walk")
    elif robot_size == 'kid':
        control_module.action().play_action("archery_ready")
    sleep(2)


    if robot_size == 'adult':
        robot_mode = RobotMode.RUNNING
        mode_change = True
        raw_input('Tab Enter to start')
    elif robot_size == 'kid':
        rospy.loginfo('Press start button to begin.')

    while not rospy.is_shutdown():
        if mode_change:
            mode_change = False

            if robot_mode == RobotMode.READY:
                rospy.loginfo('Resetting OP3. Press start button to begin.')
                if robot_size == 'kid':
                    control_module.head().move_head(pan_position=0, tilt_position=-0.8)

            elif robot_mode == RobotMode.RUNNING:
                rospy.loginfo('Starting archery event.')

                start_archery()

        rate.sleep()



