#!/usr/bin/env python

from enum import Enum
from time import sleep

import rosnode
import rospy
import roslib.packages as rospkg
import os
import json
from std_msgs.msg import String

from control_modules.control_module import ControlModule, AdvancedControlModule
from basketball_objs import Obstacles
from vision.msg import PositionArea
from tools.follower import Follower
from tools.tracker import Tracker
from events.msg import Track


class RobotMode(Enum):
    READY = 0
    RUNNING = 1


class Movements(Enum):
    MARCH = 0
    FORWARD = 1
    BACKWARD = 2
    TURN_LEFT = 3
    TURN_RIGHT = 4
    ACTION = 5


def button_callack(msg):
    global robot_mode, mode_change

    if msg.data == 'start':
        # Start button pressed
        # Send robot to RUNNING mode
        if robot_mode == RobotMode.READY:
            robot_mode = RobotMode.RUNNING
            mode_change = True
        elif robot_mode == RobotMode.RUNNING:
            rospy.logerr('Robot already in running mode.')

    elif msg.data == 'mode':
        # Mode button pressed
        # Send robot to READY mode
        robot_mode = RobotMode.READY
        mode_change = True


def wait_for_node(node_name):
    # Wait for node to start
    while node_name not in rosnode.get_node_names():
        rospy.logwarn('Node is not running: {0}'.format(node_name))
        sleep(1)

    rospy.loginfo('Node is running: {0}'.format(node_name))


def play_basketball():
    global mode_change, start_following, stage, control_module, rate, track_pub, obstacles, obj_tracker

    control_module.action().play_action('walk_ready')
    if robot_size =='adult':
        control_module.head().move_head(pan_position=0, tilt_position=-0.6)
    sleep(0.5)

    while not rospy.is_shutdown() and not mode_change:
        print(stage)

        if stage == 0:
            stage = 1

        tracking_status = obj_tracker.process_tracking()
        if stage == 1:
            track_pub.publish(obstacles.mixed.x, obstacles.mixed.y, obstacles.mixed.area)
        elif stage == 3 or stage == 4:
            track_pub.publish(obstacles.red.x, obstacles.red.y, obstacles.red.area)

        mixed_x_pub, mixed_sub_y_pub = obstacles.mixed.x, obstacles.mixed.y

        red_x_pub, red_y_pub = obstacles.red.x, obstacles.red.y

        sleep(0.001)

        if stage == 1:
            obj_tracker.provide_marker(mixed_x_pub, mixed_sub_y_pub, obstacles.mixed.area)
        elif stage == 3 or stage == 4:
            obj_tracker.provide_marker(red_x_pub, red_y_pub, obstacles.red.area)

        # STAGE 1: Track and follow the basketball until its close enough.
        if stage == 1:
            if start_following:
                obj_tracker.start_tracking()
                obj_follower.start_following()
                start_following = False
                sleep(1)

            if obstacles.mixed.area > min_ball_size:
                obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0, "follow",False)
                if obstacles.mixed.area > max_ball_size:
                    stage = 2
                    # control_module.walking().stop()
                    obj_follower.stop_following()
                    obj_tracker.stop_tracking()
                    if robot_size == 'adult':
                        control_module.head().move_head(pan_position=0, tilt_position=-0.8)
                    else:
                        control_module.head().move_head(pan_position=0, tilt_position=-0.8)
                    y_good, x_good = False, False
        # STAGE 2: Adjust the position of the robot relative to the ball to pick up successfully
        if stage == 2:

            if obstacles.mixed.y > max_ball_y: # 290
                control_module.walking().walk('backward')
                sleep(0.3)
                control_module.walking().stop()
            elif obstacles.mixed.y < min_ball_y and obstacles.mixed.y > 0: #200
                control_module.walking().walk('forward')
                sleep(0.3)
                control_module.walking().stop()
            else:
                y_good = True

            
            if obstacles.mixed.x > 300:
                control_module.walking().walk('turn_left')
                sleep(0.3)
            elif obstacles.mixed.x < 125 and obstacles.mixed.x > 0:
                control_module.walking().walk('turn_right')
                sleep(0.3)
            else:
                x_good = True

            if x_good and y_good:
                stage = 3

        # STAGE 3: Play the pick up and throw ready motions
        if stage == 3:
            control_module.walking().stop()
            control_module.action().play_action('pickup')
            sleep(2)
            if robot_size == 'kid':
                control_module.action().play_action('throw_ready')
                sleep(1.5)

            stage = 4 if robot_size == 'kid' else 3.5
            if stage ==4:
                obj_tracker.start_tracking()
                obj_follower.start_following()

        if stage == 3.5:
            control_module.head().move_head(pan_position=0, tilt_position=-0.1)
            # control_module.walking().backward(step=5)
            control_module.walking().backward()
            sleep(6)
            control_module.walking().stop()
            sleep(1)
            obj_tracker.start_tracking()
            obj_follower.start_following()
            stage = 4

        # STAGE 4: Use vision to detect what side the net is on, back up in the opposite direction to straighten out
        if stage == 4:

            if (obstacles.red.area > min_basket_size):
                obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0,auto = False)
                if (obstacles.red.area > max_basket_size):
                    obj_tracker.stop_tracking()
                    obj_follower.stop_following()
                    stage = 5 if robot_size =='kid' else 4.5

        if stage == 4.5:
            control_module.head().move_head(pan_position=0, tilt_position=-0.1)
            if obstacles.red.x < 427*(0.5-0.05):
                control_module.walking().walk(gait='turn_left')
                sleep(0.5)
            elif obstacles.red.x > 427*(0.5+0.05):
                control_module.walking().walk(gait='turn_right')
                sleep(0.5)
            else:
                stage = 5

        # STAGE 5: Move forward, tracking and following the net, then throw
        if stage == 5:
            if robot_size == 'kid':
                control_module.walking(gait='prepare_to_throw').walk(gait='prepare_to_throw')
            sleep(time_prepare_throw)
            control_module.action().play_action('throw')
            stage = 6

    sleep(3)


def set_variables():
    global min_ball_size, max_ball_size, min_basket_size, max_basket_size, time_prepare_throw, min_ball_y,max_ball_y
    config_variables_file = os.path.join(rospkg.get_pkg_dir('events'), 'config', event,
                                         'variables_{0}.json'.format(robot_size))

    vars = None
    with open(config_variables_file, 'r') as fp:
        vars = json.load(fp)

    min_ball_size = vars['min_ball_size']
    max_ball_size = vars['max_ball_size']

    min_ball_y = vars['min_ball_y']
    max_ball_y = vars['max_ball_y']

    min_basket_size = vars['min_basket_size']
    max_basket_size = vars['max_basket_size']
    time_prepare_throw = vars['time_prepare_throw']


if __name__ == '__main__':

    # Initalize global variables
    robot_mode = RobotMode.READY
    mode_change = False
    obstacles = Obstacles()
    SPIN_RATE = 0.5
    event = 'basketball'
    stage = 0
    start_following = True

    robot_size = rospy.get_param('basketball_node/size')

    set_variables()  # Important !

    # Wait for other nodes to launch
    wait_for_node('/op3_manager')
    wait_for_node('/vision_node')

    # Initialze ROS node
    rospy.init_node('basketball_node')
    rate = rospy.Rate(SPIN_RATE)

    # Initialize OP3 ROS generic subscriber
    button_sub = rospy.Subscriber('/robotis/open_cr/button', String, button_callack)

    # Initialze vision subscribers
    ball1_sub = rospy.Subscriber('/vision_node/ball1', PositionArea,
                                lambda msg: obstacles.ball1.update(msg.x, msg.y, msg.position, msg.area))
    red_sub = rospy.Subscriber('/vision_node/red', PositionArea,
                               lambda msg: obstacles.red.update(msg.x, msg.y, msg.position, msg.area))
    ball2_sub = rospy.Subscriber('/vision_node/ball2', PositionArea,
                                  lambda msg: obstacles.ball2.update(msg.x, msg.y, msg.position, msg.area))

    mixed_sub = rospy.Subscriber('/vision_node/mixed', PositionArea,
                                  lambda msg: obstacles.mixed.update(msg.x, msg.y, msg.position, msg.area))

    circle_sub = rospy.Subscriber('/vision_node/circle', PositionArea,
                                  lambda msg: obstacles.mixed.update(msg.x, msg.y, msg.position, msg.area))

    track_pub = rospy.Publisher('/track', Track, queue_size=0)

    # Initializing modules
    control_module = ControlModule(event, robot_size)

    obj_tracker = Tracker(event, robot_size, control_module)
    obj_follower = Follower(event, robot_size, control_module)

    # if robot_size == 'kid':
    #     control_module = ControlModule(event, robot_size)
    # else:
    #     control_module = AdvancedControlModule(event, robot_size)

    # Enabling walking module
    sleep(10)

    if robot_size == 'adult':
        control_module.walking().stop()
        control_module.head().move_head(pan_position=0, tilt_position=-0.6)
        raw_input('Tab enter to check the basket')
        control_module.walking().stop()
        control_module.head().move_head(pan_position=0, tilt_position=-0.1)
        raw_input('Tab enter to start')
        sleep(3)
        robot_mode = RobotMode.RUNNING
        mode_change = True

    elif robot_size == 'kid':
        rospy.loginfo('Press start button to begin.')

    while not rospy.is_shutdown():
        if mode_change:
            mode_change = False

            if robot_mode == RobotMode.READY:
                rospy.loginfo('Resetting OP3. Press start button to begin.')

                control_module.walking().stop()
                control_module.head().move_head(pan_position=0, tilt_position=-0.8)

            elif robot_mode == RobotMode.RUNNING:
                rospy.loginfo('Starting basketball event.')
                play_basketball()

        rate.sleep()
