#! /usr/bin/env python

import json
import os
from time import sleep

import roslib.packages as rospkg
import rospy
from std_msgs.msg import Int32

from op3_action_module_msgs.srv import IsRunning

class ActionModule(object):

    name = 'action_module'
    action_enabled = False

    def __init__(self, event, robot_size):
        super(ActionModule, self).__init__(event, robot_size)

        self.__motion_index_pub = rospy.Publisher(
            '/robotis/action/page_num', Int32, queue_size=0)
        self.__is_running_client = rospy.ServiceProxy(
            '/robotis/action/is_running', IsRunning)

        self.actions = self.__load_configuration(event)

    def __load_configuration(self, event):
        config_file_path = os.path.join(rospkg.get_pkg_dir('events'), 'config', event, 'action_configurations.json')

        with open(config_file_path, 'r') as fp:
            return json.load(fp)


    def __action_is_running(self):
        service_name = '/robotis/action/is_running'

        rospy.wait_for_service(service_name)
        try:
            return self.__is_running_client().is_running
        except rospy.ServiceException:
            rospy.logerr('Failed to call service: {0}'.format(service_name))

    def play_action(self, action):
        if not ActionModule.action_enabled:
            rospy.logwarn('Action module is not enabled')
            return

        if action in self.actions.keys():
            rospy.loginfo('Playing action: {0}'.format(action))

            message = Int32()
            message.data = self.actions[action]['index']

            for _ in range(self.actions[action]['play_count']):
                self.__motion_index_pub.publish(message)
                while self.__action_is_running():
                    sleep(0.05)
        else:
            rospy.logwarn('Action ({0}) not found in action configuration file'.format(action))
