#! /usr/bin/env python

import rospy
import math
from enum import Enum
from time import sleep

from std_msgs.msg import Header, String
from sensor_msgs.msg import JointState
from robotis_controller_msgs.msg import ScanLimits


class HeadControlModule(object):

    name = 'head_control_module'
    head_enabled = False

    def __init__(self, event, robot_size):
        super(HeadControlModule, self).__init__(event, robot_size)

        self.__set_joint_state_pub = rospy.Publisher(
            '/robotis/head_control/set_joint_states', JointState, queue_size=0)

        # Added by Louis O'C to allow custom pan/tilt limits
        self.head_scan_limit_pub = rospy.Publisher('/scan_limits', ScanLimits, queue_size=1)


    def __move_actuators(self, actuator_names, positions, velocity=0):
        header = Header()
        header.seq = 0
        header.stamp = rospy.Time.now()
        header.frame_id = ''

        message = JointState()
        message.header = header
        message.name = actuator_names
        message.position = positions
        message.velocity = [velocity] * len(actuator_names)
        message.effort = [0] * len(actuator_names)

        self.__set_joint_state_pub.publish(message)


    def move_head(self, pan_position, tilt_position):
        if not HeadControlModule.head_enabled:
            rospy.logwarn('Head control module is not enabled')
            return

        self.__move_actuators(actuator_names=['head_pan', 'head_tilt'],
                              positions=[pan_position, tilt_position])
        
        sleep(0.05)

    # Call this to alter the limits of the pan/tilt of the tracking search
    def change_head_scan(self, min_pan=-85, max_pan=85, min_tilt=-75, max_tilt=30):
        # Default values are: min_pan=-85.0, max_pan=85.0, min_tilt=-75, max_tilt=30
        message = ScanLimits()
        message.minPan = min_pan
        message.maxPan = max_pan
        message.minTilt = min_tilt
        message.maxTilt = max_tilt

        self.head_scan_limit_pub.publish(message)
