#!/usr/bin/env python
from enum import Enum
from time import sleep
import rosnode
import rospy
import roslib.packages as rospkg
import os
import json
import math
from std_msgs.msg import String
from control_modules.control_module import AdvancedControlModule
from penalty_obs import Obstacles
from vision.msg import PositionArea, DetectedLines
from tools.follower import Follower
from tools.tracker import Tracker
from events.msg import Track
from robot import Robot, RobotMovement
from map_module.map_module import MapModule
from tools.follower import Follower
from tools.tracker import Tracker

class RobotMode(Enum):
    READY = 0
    RUNNING = 1
class RobotStart(Enum):
    LEFT = 0
    CENTER = 1
    RIGHT = 2
class Movements(Enum):
    MARCH = 0
    FORWARD = 1
    BACKWARD = 2
    TURN_LEFT = 3
    TURN_RIGHT = 4
    ACTION = 5
    UNDEFINE = 6
    FINISH = 7
    SHIFT_LEFT = 8
    SHIFT_RIGHT = 9
class StandState(Enum):
    STAND = 0
    FALLEN_FORWARD = 1
    FALLEN_BACKWARD = 2
class RoboStates(Enum):
    INITIAL = 0
    APPROACH_BALL = 1
    AVOID_OBSTACLES = 2
    APPROACH_SHOOT_AREA = 3
    APPROACH_SHOOT_AREA_2 = 4
    SHOOT = 5
    FINISH = 6
def button_callack(msg):
    global robot_mode, mode_change, stopped
    if stopped:
        if msg.data == 'mode':
            robot_start_pos = RobotStart.LEFT
        elif msg.data == 'start':
            robot_start_pos = RobotStart.CENTER
        elif msg.data == 'user':
            robot_start_pos = RobotStart.RIGHT
    else:
        if msg.data == 'start':
            # Start button pressed
            # Send robot to RUNNING mode
            if robot_mode == RobotMode.READY:
                robot_mode = RobotMode.RUNNING
                mode_change = True
            elif robot_mode == RobotMode.RUNNING:
                rospy.logerr('Robot already in running mode.')
        elif msg.data == 'mode':
            # Mode button pressed
            # Send robot to READY mode
            robot_mode = RobotMode.READY
            mode_change = True
            stopped = True
def wait_for_node(node_name):
    # Wait for node to start
    while node_name not in rosnode.get_node_names():
        rospy.logwarn('Node is not running: {0}'.format(node_name))
        sleep(1)
    rospy.loginfo('Node is running: {0}'.format(node_name))
def goalie_backward():
    control_module.walking().backward(step=1)
    return 'backward'
def goalie_forward():
    control_module.walking().forward(step=1)
    return 'forward'
def goalie_shift(override=None):
    if override == 'left' or (obstacles.circle.x <= screen_middle-center_err and override is None):
        control_module.walking().shift_left(step=1)
        return 'left'
    if override == 'right' or (obstacles.circle.x >= screen_middle+center_err and override is None):
        control_module.walking().shift_right(step=1)
        return 'right'
def goalie_rewind(move_history):
    switch = {
        'forward': goalie_backward(),
        'left': lambda: goalie_shift(override='right'),
        'right': lambda: goalie_shift(override='left')
    }
    while move_history:
        tmp = move_history.pop()
        # print(tmp)
        # print(switch[tmp])
        switch[tmp]
    return
def goalie():
    global obstacles, init_tilt_position
    stack = []
    track_stack = False
    control_module.action().play_action('arm_up')
    middle = lambda x: abs(screen_middle-x)<=center_err

    while not rospy.is_shutdown() and not mode_change:
        if obstacles.circle.get_average_area() > min_area and obstacles.circle.y > 200:
            #tilt head down
            print ('head down')
            init_tilt_position -= 0.1
            control_module.head().move_head(pan_position=0, tilt_position=init_tilt_position)
        elif obstacles.circle.get_average_area() > min_area and obstacles.circle.y < 70:
            #tilt head up
            print ('head up')
            init_tilt_position += 0.1
            control_module.head().move_head(pan_position=0, tilt_position=init_tilt_position)
        sleep(1)
        print ('average area: ' + str (obstacles.circle.get_average_area()))
        if obstacles.circle.get_average_area() > min_area and not middle(obstacles.circle.x):
            if track_stack:
                stack.append(goalie_shift())
            else:
                goalie_shift()
        if obstacles.circle.get_average_area() > med_area and obstacles.circle.get_average_area() < max_area:
            # track_stack = True
            # stack.append(goalie_forward())

            control_module.action().play_action('sitdown')
            while obstacles.circle.get_average_area() > med_area and obstacles.circle.get_average_area() < max_area:
                control_module.action().play_action('block')
            print('stand up')
            control_module.action().play_action('standup')
            sleep(3)
            control_module.head().move_head(pan_position=0, tilt_position=init_tilt_position)
            # control_module.action().play_action('arm_up')
        if obstacles.circle.get_average_area() > max_area:
            handle_kick(obstacles.circle.x)
            return
        # if obstacles.circle.get_average_area() < min_area and track_stack:
        #     goalie_rewind(stack)
        #     stack = []
        #     track_stack = False
def goalie_with_tracker():
    # control_module.head().change_head_scan(min_pan=-50.0, max_pan=50.0, min_tilt=-85, max_tilt=-5)
    tracking_status = obj_tracker.process_tracking()

    is_tracking = False
    is_following = False
    is_turned = False
    is_walk = False

    while not rospy.is_shutdown():
        tracking_status = obj_tracker.process_tracking()
        obj_tracker.provide_marker(
            obstacles.circle.x, 
            obstacles.circle.y, 
            obstacles.circle.area)
        print('average area: ' + str(obstacles.circle.get_average_area()))
        if not is_following:
                is_following = True
                obj_follower.start_following()
                rospy.loginfo('[states info] ' + str(states))
        if obstacles.circle.get_average_area() > min_area:
            obj_follower.process_following(
                obj_tracker.get_pan(), 
                0,
                0, 
                auto=False)
        if obstacles.circle.get_average_area() > med_area:
            obj_follower.process_following(
                obj_tracker.get_pan(),
                obj_tracker.get_tilt(),
                0,
                auto=False)
        if obstacles.circle.get_average_area() > max_area:
            handle_kick(obstacles.circle.x)
            return
        if obstacles.circle.get_average_area() < min_area:
            obj_follower.wait_following()
    return
def handle_kick(obj_position):
    # print('angle_robot', angle_robot, obj_position)
    control_module.head().move_head(pan_position=0, tilt_position=-0.9)
    sleep(0.25)
    angle_robot = control_module.plan_orientation['angle']
    print('turning right', angle_robot)
    
    if obj_position > 213:  # 213 is half of resized vision resolution
        control_module.action().play_action('right_kick')
    else:
        control_module.action().play_action('left_kick')
    control_module.action().play_action('walk_ready')
    sleep(1)
    control_module.head().move_head(pan_position=0, tilt_position=-1.0)
def set_variables():
    global min_area, med_area, max_area, head_pan, head_tilt, center_err, screen_middle, use_tracking
    config_variables_file = os.path.join(rospkg.get_pkg_dir('events'), 'config', event,
                                         'variables_{0}.json'.format(robot_size))
    vars = None
    with open(config_variables_file, 'r') as fp:
        vars = json.load(fp)

    min_area = int(vars['min_ball_area_goalie'])
    med_area = int(vars['med_ball_area_goalie'])
    max_area = int(vars['max_ball_area_goalie'])

    head_pan = vars['head_pan']
    head_tilt = vars['head_tilt']
    center_err = vars['middle_width']
    screen_middle = vars['middle_line']

    use_tracking = vars['use_tracking']


if __name__ == '__main__':
    # Initalize global variables
    global init_tilt_position
    robot_mode = RobotMode.READY
    mode_change = False
    robot = Robot(forward_rate=1.0, left_rate=10.0, right_rate=9.0)
    obstacles = Obstacles()
    SPIN_RATE = 0.5
    NUM_MAP_ROWS, NUM_MAP_COLUMNS = 19, 28
    event = 'goalie'
    start_following = True
    look_at_lines = False
    stopped = False
    robot_start_pos = RobotStart.CENTER
    lines = []
    states = RoboStates.INITIAL



    # Wait for other nodes to launch
    wait_for_node('/op3_manager')
    wait_for_node('/vision_node')
    # Initialze ROS node
    rospy.init_node('goalie_node')
    robot_size = rospy.get_param('goalie_node/size')
    field = MapModule(event, robot_size)
    rate = rospy.Rate(SPIN_RATE)
    set_variables()  # Important !
    # Initialize OP3 ROS generic subscriber
    button_sub = rospy.Subscriber('/robotis/open_cr/button', String, button_callack)
    # Initialze vision subscribers

    blue_sub = rospy.Subscriber('/vision_node/blue', PositionArea,
                                lambda msg: obstacles.blue.update(msg.x, msg.y, msg.position, msg.area))
    red_sub = rospy.Subscriber('/vision_node/red', PositionArea,
                               lambda msg: obstacles.red.update(msg.x, msg.y, msg.position, msg.area))
    yellow_sub = rospy.Subscriber('/vision_node/yellow', PositionArea,
                                  lambda msg: obstacles.yellow.update(msg.x, msg.y, msg.position, msg.area))
    mixed_sub = rospy.Subscriber('/vision_node/mixed', PositionArea,
                                 lambda msg: obstacles.mixed.update(msg.x, msg.y, msg.position, msg.area))
    circle_sub = rospy.Subscriber('/vision_node/circle', PositionArea,
                                  lambda msg: obstacles.circle.update(msg.x, msg.y, msg.position, msg.area))
    track_pub = rospy.Publisher('/track', Track, queue_size=0)
    
    # Initializing modules
    control_module = AdvancedControlModule(event, robot_size)
    control_module.plan_orientation['x'] = field.map['initial_pos']['x']
    control_module.plan_orientation['y'] = field.map['initial_pos']['y']
    control_module.plan_orientation['angle'] = math.degrees(field.map['initial_pos']['angle'])
    print (control_module.plan_orientation)
    obj_tracker = Tracker(event, robot_size, control_module)
    obj_follower = Follower(event, robot_size, control_module, True)
    # Enabling walking module
    sleep(10)
    control_module.action().play_action('walk_ready')
    sleep(0.5)
    init_tilt_position = -0.522
    control_module.head().move_head(pan_position=0, tilt_position=-1.1)
    # init_tilt_position = -1.0
    if robot_size == 'adult':
        robot_mode = RobotMode.RUNNING
        mode_change = True
        control_module.head().move_head(pan_position=0, tilt_position=-0.261)
        control_module.head().move_head(pan_position=0, tilt_position=init_tilt_position)
        raw_input('tab Enter to start')
    elif robot_size == 'kid':
        rospy.loginfo('Press start button to begin.')
    while not rospy.is_shutdown():
        if mode_change:
            mode_change = False
            if robot_mode == RobotMode.READY:
                rospy.loginfo('Resetting OP3. Press start button to begin.')
                control_module.walking().stop()
                control_module.head().move_head(pan_position=0, tilt_position=-0.261)
                robot_mode = RobotMode.RUNNING
            elif robot_mode == RobotMode.RUNNING:
                rospy.loginfo('Starting goalie event.')
                if use_tracking:
                    goalie_with_tracker()
                else:
                    goalie()

        rate.sleep()
