#!/usr/bin/env python
import std_msgs
from iros_vision.msg import DrumAction
from control_modules.control_module import ControlModule
import rospy
import rosnode
from robot_drummer import play
from time import sleep

def wait_for_node(node_name):
    # Wait for node to start
    while node_name not in rosnode.get_node_names():
        rospy.logwarn('Node is not running: {0}'.format(node_name))
        sleep(1)
    rospy.loginfo('Node is running: {0}'.format(node_name))



def drum_vision_cb(msg):
    global controlModule
    state = [msg.ride, msg.ignore, msg.snare, msg.hihat, msg.kick]
    play(state, control_module) #send the state to play to the robot drummer

def calibrate_drums():
    global control_module
    control_module.action().play_action("drums_ready")
    #test_time() #############################################just for testing of time in motions
    sleep_time = 0.5
    caliration_iterations = 6
    raw_input("******************Press ENTER to begin\n")

    calibrate_motion('HH-Kick', sleep_time, caliration_iterations)
    calibrate_motion('Crash-Kick', sleep_time, caliration_iterations)
    calibrate_motion('HH-Snare', sleep_time, caliration_iterations)
    calibrate_motion('Snare-Ride', sleep_time, caliration_iterations)
    


    print("******************Press enter to FINISH calibration")
    print("******************Enter 'r' to REPEAT calibration\n")
    repeat = raw_input()

    if repeat == 'r':
        calibrate_drums()
    
#calibrates one motion (for example kick)
def calibrate_motion(motion, sleep_time, calibration_iterations):
    print("CALIBRATING ", motion)
    while True:
        for  i in range(0, calibration_iterations):
            control_module.action().play_action(motion)
            sleep(sleep_time)
        repeat = raw_input("******************Press 'r' to repeat. Press anything else to continue\n")
        if repeat != 'r':
            break


if __name__ == '__main__':

    SPIN_RATE = 0.5

    # Initialze ROS node
    rospy.init_node('drum_player_node')
    rate = rospy.Rate(SPIN_RATE)

    #wait for the manager
    wait_for_node('/op3_manager')
    control_module = ControlModule('iros2019', 'adult')
    sleep(10)
    

     #Create ready topic to start the vision
    drums_ready_pub = rospy.Publisher('/iros_drums/ready', std_msgs.msg.Bool, queue_size=1)

    skip = raw_input('Press Enter to wave (s to skip)')

    if skip != 's':
        control_module.action().play_action("polaris_wave")        

    control_module.action().play_action("drums_ready") #go to ready position
    
    wait_for_node('/drums_vision')

    # Initialize OP3 ROS generic subscriber
    drums_vision_sub = rospy.Subscriber('/iros_drums/drums', DrumAction, drum_vision_cb)

    #check if drum calibration is enabled
    drum_calibration = rospy.get_param('drum_player_node/drum_calibration') #check if we are gonna do sound and position check before running the program
    
    if drum_calibration == "1":
        print("Drum calibration enabled")
        calibrate_drums()


    control_module.action().play_action("drums_ready")
    raw_input("Press enter to start")
    msg = std_msgs.msg.Bool()
    msg.data = True
    drums_ready_pub.publish(msg)

    while not rospy.is_shutdown():
        print("-")
        rate.sleep()

