#!/usr/bin/env python

from enum import Enum
from time import sleep

import rosnode
import rospy
import roslib.packages as rospkg
import os
import json
from std_msgs.msg import String
from weight_objs import WeightObjects
from tools.follower import Follower
from tools.tracker import Tracker

from control_modules.control_module import ControlModule
from vision.msg import PositionArea


class RobotMode(Enum):
    READY = 0
    RUNNING = 1


class Movements(Enum):
    MARCH = 0
    FORWARD = 1
    BACKWARD = 2
    TURN_LEFT = 3
    TURN_RIGHT = 4
    ACTION = 5

def button_callback(msg):
    global robot_mode, mode_change

    if msg.data == 'start':
        # Start button pressed
        # Send robot to RUNNING mode
        if robot_mode == RobotMode.READY:
            robot_mode = RobotMode.RUNNING
            mode_change = True
        elif robot_mode == RobotMode.RUNNING:
            rospy.logerr('Robot already in running mode.')

    elif msg.data == 'mode':
        # Mode button pressed
        # Send robot to READY mode
        change_weight(current_weight)
        rospy.logwarn(current_weight.value)
        robot_mode = RobotMode.READY
        mode_change = True


def wait_for_node(node_name):
    # Wait for node to start
    while node_name not in rosnode.get_node_names():
        rospy.logwarn('Node is not running: {0}'.format(node_name))
        sleep(1)

    rospy.loginfo('Node is running: {0}'.format(node_name))


def start_weight_lifting():
    

if __name__ == '__main__':

    # Initalize global variables
    robot_mode = RobotMode.READY
    mode_change = False
    obstacles = WeightObjects()
    SPIN_RATE = 0.5
    event = 'weight_lifting'
    stage = 0

    start_following = True

    robot_size = rospy.get_param('weight_lifting_node/size')

    # Wait for other nodes to launch
    wait_for_node('/op3_manager')
    wait_for_node('/louis_vision')

    if robot_size == 'adult':
        stage = 0

    # Initialze ROS node
    rospy.init_node('weight_lifting_node')
    rate = rospy.Rate(SPIN_RATE)

    # Initialize OP3 ROS generic subscriber
    button_sub = rospy.Subscriber('/robotis/open_cr/button', String, button_callback)

    # Initialze vision subscribers
    blue_sub = rospy.Subscriber('/vision_node/blue', PositionArea,
                                lambda msg: obstacles.blue.update(msg.x, msg.y, msg.position, msg.angle, msg.area))
    red_sub = rospy.Subscriber('/vision_node/red', PositionArea,
                               lambda msg: obstacles.red.update(msg.x, msg.y, msg.position, msg.angle, msg.area))
    yellow_sub = rospy.Subscriber('/vision_node/yellow', PositionArea,
                                        lambda msg: obstacles.yellow.update(msg.x, msg.y, msg.position, msg.angle, msg.area))

    # Initializing modules
    control_module = ControlModule(event, robot_size)

    obj_tracker = Tracker(event, robot_size, control_module)
    obj_follower = Follower(event, robot_size, control_module)

    # Enabling walking module
    sleep(10)

    if robot_size == 'adult':
        control_module.walking().stop()
        control_module.head().move_head(pan_position=0, tilt_position=-0.8)
        # raw_input('Tab Enter to start')

        control_module.head().move_head(pan_position=0, tilt_position=-1.2)
        raw_input('Tab Enter to start')
        sleep(3)
        robot_mode = RobotMode.RUNNING
        mode_change = True
    elif robot_size == 'kid':
        control_module.action().play_action('walk_ready')
        control_module.head().move_head(pan_position=0, tilt_position=tilt_head)
        rospy.loginfo('Press start button to begin.')

    while not rospy.is_shutdown():
        if mode_change:
            mode_change = False

            if robot_mode == RobotMode.READY:
                rospy.loginfo('Resetting OP3. Press start button to begin.')

                control_module.walking().stop()
                control_module.head().move_head(pan_position=0, tilt_position=-0.8)
            
            elif robot_mode == RobotMode.RUNNING:
                rospy.loginfo('Starting weightlifting event.')

                start_weight_lifting()

        rate.sleep()



