import json
import math
import os
import sys
from time import sleep
from events.srv import get_map, get_mapResponse
from events.msg import new_pos
import roslib.packages as rospkg
import rospy
from std_msgs.msg import String, Bool
import numpy as np
import json
import math


class MapModule:

    def __init__(self, event, robot_size, map_name=None):
        self.robot_size = robot_size
        self.event = event
        if map_name is None:
            self.name = 'default'
        else:
            self.name = map_name
        self.map = self.load_map(self.name)
        self.get_map_service = rospy.Service('/events/get_map', get_map, self.get_map_handler)
        self.save_map_sub = rospy.Subscriber('/events/save_map', String, self.save_map_handler)

    def load_map(self, name):
        calibration_file_path = os.path.join(rospkg.get_pkg_dir('events'), 'config', self.event,
                                             name + "_" + self.robot_size + '.json')

        try:
            with open(calibration_file_path, 'r') as fp:
                config = json.load(fp)
            config['initial_pos']['angle']= math.radians(config['initial_pos']['angle'])
        except:

            fp = open(calibration_file_path, 'w+')
            config = self.new_map(name)

            json.dump(config, fp)
            fp.close()
            config['initial_pos']['angle'] = math.radians(config['initial_pos']['angle'])
        return config

    def save_map(self, name):
        calibration_file_path = os.path.join(rospkg.get_pkg_dir('events'), 'config', self.event,
                                             name + "_" + self.robot_size + '.json')
        self.map['initial_pos']['angle'] = math.degrees(self.map['initial_pos']['angle'])
        fp = open(calibration_file_path, 'w')

        json.dump(self.map, fp, sort_keys=True, indent=4)
        fp.close()

    '''
    REGL, WALL, STRIP format: [ [ [x1,y1],[x2,y2]....  ],.... ]
    CIRC format: [ [x,y, radius].... ]
    '''

    def new_map(self, name):
        return {
            'name': name,
            'height': 300,
            'width': 300,
            'initial_pos': {
                'x': 10,
                'y': 10,
                'angle': 90.0
            },
            'obstacles': {
                'REGL': [],
                'WALL': [],
                'CIRC': []

            },
            'MARKER': []
        }

    def get_map_handler(self, req):
        self.map = self.load_map(req.name)
        print json.dumps(self.map)
        return get_mapResponse(json.dumps(self.map))

    def save_map_handler(self, req):
        self.map = json.loads(req.data)
        self.save_map(self.map['name'])
