#!/usr/bin/env python

from enum import Enum
from time import sleep

import rosnode
import rospy
from std_msgs.msg import String
import roslib.packages as rospkg
import os
import json

from control_modules.control_module import AdvancedControlModule
from obstacles import Obstacles
from vision.msg import PositionArea
from tools.follower import Follower
from tools.tracker import Tracker
from events.msg import Track


class RobotMode(Enum):
    READY = 0
    RUNNING = 1


class Movements(Enum):
    MARCH = 0
    FORWARD = 1
    BACKWARD = 2
    TURN_LEFT = 3
    TURN_RIGHT = 4
    ACTION = 5


def button_callack(msg):
    global robot_mode, mode_change

    if msg.data == 'start':
        # Start button pressed
        # Send robot to RUNNING mode
        if robot_mode == RobotMode.READY:
            robot_mode = RobotMode.RUNNING
            mode_change = True
        elif robot_mode == RobotMode.RUNNING:
            rospy.logerr('Robot already in running mode.')

    elif msg.data == 'mode':
        # Mode button pressed
        # Send robot to READY mode
        robot_mode = RobotMode.READY
        mode_change = True


def wait_for_node(node_name):
    # Wait for node to start
    while node_name not in rosnode.get_node_names():
        rospy.logwarn('Node is not running: {0}'.format(node_name))
        sleep(1)

    rospy.loginfo('Node is running: {0}'.format(node_name))


def start_marathon():
    global mode_change, control_module, rate, start_following, stage, obj_tracker, obj_follower, forward_count, right_count, left_count

    forward_count, right_count, left_count = 0, 0, 0

    sleep(0.1)

    while not rospy.is_shutdown() and not mode_change:
        tracking_status = obj_tracker.process_tracking()

        red_x_pub, red_y_pub = obstacles.red.x, obstacles.red.y

        obj_tracker.provide_marker(red_x_pub, red_y_pub, obstacles.red.area)

        if stage == 0:
            stage = 1

        # Find red line and follow it
        if stage == 1:
            if start_following:
                obj_tracker.start_tracking()
                obj_follower.start_following()
                start_following = False
            
            if obstacles.red.area > 300:
                obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0, 'follow', False)

            if arrow_type is not 'None':
                stage = 2
                

        # Handle arrows
        if stage == 2:
            if arrow_type == 'forward':
                forward_count += 1
            elif arrow_type == 'left':
                left_count += 1
            elif arrow_type == 'right':
                right_count += 1

            if right_count > 2:
                sleep(2)
                #control_module.walking().walk('turn_right')
                print('turning right')
                control_module.walking().turn_right(angle = 90)
                #sleep(turning_time)
                stage = 3
            elif left_count > 2:
                sleep(2)
                #control_module.walking().walk('turn_left')
                print('turning left')
                control_module.walking().turn_left(angle = 90)
                #sleep(turning_time)
                stage = 3
            elif forward_count > 2:
                sleep(2)
                control_module.walking().walk('forward')
                print('going forwards')
                sleep(forward_time)
                stage = 3


        # Blind walk forwards to get to next red segment
        if stage == 3:
            print('walking blindly ahead')
            control_module.walking().stop()
            forward_count, right_count, left_count = 0,0,0
            control_module.head().move_head(pan_position=0, tilt_position=head_tilt_arrow)
            control_module.walking().walk('forward')
            sleep(forward_time)
            stage = 1


def arrow_callback(msg):
    global arrow_type
    arrow_type = msg.data


def set_variables():
    global min_line_size, turning_time, forward_time, head_tilt_init, head_tilt_arrow
    config_variables_file = os.path.join(rospkg.get_pkg_dir('events'), 'config', event, 'variables_{0}.json'.format(robot_size))

    vars = None
    with open(config_variables_file, 'r') as fp:
            vars = json.load(fp)


    min_line_size = vars['min_line_size']
    turning_time = vars['turning_time']
    forward_time = vars['forward_time']
    head_tilt_init = vars['head_tilt_init']
    head_tilt_arrow = vars['head_tilt_arrow']


if __name__ == '__main__':

    # Initalize global variables
    robot_mode = RobotMode.READY
    mode_change = False
    obstacles = Obstacles()
    SPIN_RATE = 0.5
    event = 'marathon'
    start_following = True
    stage = 0
    arrow_type = "None"

    robot_size = rospy.get_param('marathon_node/size')

    set_variables() # Important!

    # Wait for other nodes to launch
    wait_for_node('/op3_manager')
    wait_for_node('/vision_node')

    # Initialze ROS node
    rospy.init_node('marathon_node')
    rate = rospy.Rate(SPIN_RATE)

    # Initialize OP3 ROS generic subscriber
    button_sub = rospy.Subscriber('/robotis/open_cr/button', String, button_callack)

    # Initialze vision subscribers
    blue_sub = rospy.Subscriber('/vision_node/blue', PositionArea,
                                lambda msg: obstacles.blue.update(msg.x, msg.y, msg.position, msg.area))
    red_sub = rospy.Subscriber('/vision_node/red', PositionArea,
                               lambda msg: obstacles.red.update(msg.x, msg.y, msg.position, msg.area))
    yellow_sub = rospy.Subscriber('/vision_node/yellow', PositionArea,
                                        lambda msg: obstacles.yellow.update(msg.x, msg.y, msg.position, msg.area))

    arrow_sub = rospy.Subscriber('vision_node/arrow', String, arrow_callback)

    track_pub = rospy.Publisher('/track', Track, queue_size=0)

    # marker_detector = rospy.Subscriber('/vision_node/arrow', String, marker_callback)

    # Initializing modules
    control_module = AdvancedControlModule(event, robot_size)

    obj_tracker = Tracker(event, robot_size, control_module)
    obj_follower = Follower(event, robot_size, control_module)

    # Enabling walking module
    sleep(10)

    control_module.head().change_head_scan(min_pan=-50.0, max_pan=50.0, min_tilt=-50, max_tilt=20)
    sleep(0.5)

    control_module.action().play_action('walk_ready')
    sleep(1)
    control_module.head().move_head(pan_position=0, tilt_position=head_tilt_init)
    sleep(1)

    if robot_size == 'adult':
        robot_mode = RobotMode.RUNNING
        raw_input('tab Enter to start')
        mode_change = True
    elif robot_size == 'kid':
        rospy.loginfo('Press start button to begin.')

    while not rospy.is_shutdown():
        if mode_change:
            mode_change = False

            if robot_mode == RobotMode.READY:

                rospy.loginfo('Resetting OP3. Press start button to begin.')

                control_module.walking().stop()
            
            elif robot_mode == RobotMode.RUNNING:
                rospy.loginfo('Starting marathon run event.')

                start_marathon()

        rate.sleep()



