#!/usr/bin/env python

from enum import Enum
from time import sleep

import rosnode
import rospy
from std_msgs.msg import String

from control_modules.control_module import ControlModule, AdvancedControlModule
from obstacles import Obstacles
from vision.msg import PositionArea
from map_module.map_module import MapModule

import roslib.packages as rospkg
import os
import json


class RobotMode(Enum):
    READY = 0
    RUNNING = 1


class Movements(Enum):
    MARCH = 0
    FORWARD = 1
    BACKWARD = 2
    TURN_LEFT = 3
    TURN_RIGHT = 4
    ACTION = 5
    UNDEFINE = 6


def button_callack(msg):
    global robot_mode, mode_change

    if msg.data == 'start':
        # Start button pressed
        # Send robot to RUNNING mode
        if robot_mode == RobotMode.READY:
            robot_mode = RobotMode.RUNNING
            mode_change = True
        elif robot_mode == RobotMode.RUNNING:
            rospy.logerr('Robot already in running mode.')

    elif msg.data == 'mode':
        # Mode button pressed
        # Send robot to READY mode
        robot_mode = RobotMode.READY
        mode_change = True


def wait_for_node(node_name):
    # Wait for node to start
    while node_name not in rosnode.get_node_names():
        rospy.logwarn('Node is not running: {0}'.format(node_name))
        sleep(1)

    rospy.loginfo('Node is running: {0}'.format(node_name))


def avoid_obstacles():
    global mode_change, control_module, obstacles, rate, head_pan, head_tilt

    control_module.action().play_action('walk_ready')
    sleep(3)
    control_module.head().move_head(pan_position=head_pan, tilt_position=head_tilt)

    previous_movement = Movements.MARCH
    current_movement = None
    yellow_prefer_left = True
    num_steps_since_yellow = 0
    num_steps_yellow_turn = 0
    blue_prefer_left = True
    angle_turn = 0

    while not rospy.is_shutdown() and not mode_change:

        sleep_walk = 1
        print(angle_turn)

        if obstacles.yellow.is_center and obstacles.yellow.y > yellow_min_top:
            # YELLOW is ahead
            num_steps_since_yellow = 0
            obstacles.blue.is_last_seen = False
            sleep_walk = 2

            if angle_turn > 30:
                sleep_walk = angle_turn / 14.7
                current_movement = Movements.TURN_LEFT
            elif angle_turn < -30:
                sleep_walk = -angle_turn / 14.7
                current_movement = Movements.TURN_RIGHT
            elif num_steps_yellow_turn == 3:
                # Keeps seeing YELLOW, most likely a sideline
                yellow_prefer_left = not yellow_prefer_left
                obstacles.yellow.is_last_seen = True
                current_movement = Movements.TURN_LEFT if yellow_prefer_left else Movements.TURN_RIGHT
            if not obstacles.yellow.is_last_seen and \
                    obstacles.yellow.is_on_left and not obstacles.yellow.is_on_right:
                # YELLOW on left
                current_movement = Movements.TURN_RIGHT
                yellow_prefer_left = False
            elif not obstacles.yellow.is_last_seen and \
                    not obstacles.yellow.is_on_left and obstacles.yellow.is_on_right:
                # YELLOW on right
                current_movement = Movements.TURN_LEFT
                yellow_prefer_left = True
            else:
                num_steps_yellow_turn += 1

                # Turn based on opposite of preference
                if not obstacles.yellow.is_last_seen:
                    yellow_prefer_left = not yellow_prefer_left
                    obstacles.yellow.is_last_seen = True
                current_movement = Movements.TURN_LEFT if yellow_prefer_left else Movements.TURN_RIGHT


        elif robot_size=='kid' and (obstacles.red.is_on_left or obstacles.red.is_center or obstacles.red.is_on_right):
            # RED is ahead
            num_steps_since_yellow += 1
            num_steps_yellow_turn = 0
            obstacles.yellow.is_last_seen = False
            obstacles.blue.is_last_seen = False

            if obstacles.blue.is_on_left or (obstacles.red.is_on_right and obstacles.red.area < 3000):
                # RED is to the right side
                current_movement = Movements.TURN_RIGHT
            elif obstacles.blue.is_on_right or (obstacles.red.is_on_left and obstacles.red.area < 3000):
                # RED is to the left side
                current_movement = Movements.TURN_LEFT
            else:
                # Red obstacle --> get down and start crawling
                current_movement = Movements.ACTION

        elif obstacles.blue.is_center and obstacles.blue.y > blue_min_top:
            # BLUE is ahead
            num_steps_since_yellow += 1
            num_steps_yellow_turn = 0
            obstacles.yellow.is_last_seen = False

            if angle_turn > 30:
                sleep_walk = angle_turn / 14.7
                current_movement = Movements.TURN_LEFT
            elif angle_turn < -30:
                sleep_walk = -angle_turn / 14.7
                current_movement = Movements.TURN_RIGHT
            elif num_steps_since_yellow <= 10:
                # Based on yellow preference
                current_movement = Movements.TURN_LEFT if yellow_prefer_left else Movements.TURN_RIGHT
            else:
                if not obstacles.blue.is_last_seen and obstacles.blue.is_on_left and not obstacles.blue.is_on_right:
                    # BLUE on left
                    current_movement = Movements.TURN_RIGHT
                    blue_prefer_left = False
                elif not obstacles.blue.is_last_seen and not obstacles.blue.is_on_left and obstacles.blue.is_on_right:
                    # BLUE on right
                    current_movement = Movements.TURN_LEFT
                    blue_prefer_left = True
                else:
                    # Turn based on preference derived from old pattern
                    obstacles.blue.is_last_seen = True
                    current_movement = Movements.TURN_LEFT if blue_prefer_left else Movements.TURN_RIGHT

        else:
            # Path is clear --> proceed
            num_steps_since_yellow += 1
            num_steps_yellow_turn = 0
            obstacles.yellow.is_last_seen = False
            obstacles.blue.is_last_seen = False

            current_movement = Movements.FORWARD

        # Execute movement
        if previous_movement != current_movement:
            if current_movement == Movements.MARCH:
                control_module.walking().march()
            elif current_movement == Movements.FORWARD:
                control_module.walking().forward()
            elif current_movement == Movements.BACKWARD:
                control_module.walking().backward()
            elif current_movement == Movements.TURN_LEFT:
                angle_turn -= (sleep_walk * 14.7)
                control_module.walking().turn_left()
                sleep(sleep_walk)
            elif current_movement == Movements.TURN_RIGHT:
                angle_turn += (sleep_walk * 14.7)
                control_module.walking().turn_right()
                sleep(sleep_walk)
            elif current_movement == Movements.ACTION:
                control_module.walking().backward()
                sleep(2.5)
                control_module.action().play_action('crawl_ready')
                control_module.action().play_action('crawl')
                control_module.action().play_action('stand_up')
                control_module.action().play_action('walk_ready')
                control_module.walking().march()
                control_module.head().move_head(pan_position=head_pan, tilt_position=head_tilt)
                sleep(3)

            previous_movement = current_movement

        rate.sleep()


# adult size one don't delete
#
# def avoid_obstacles():
#     global mode_change, control_module, obstacles, rate
#
#     control_module.action().play_action('walk_ready')
#     sleep(3)
#     if robot_size =="kid":
#         control_module.head().move_head(pan_position=0, tilt_position=-0.80)
#     else:
#         control_module.head().move_head(pan_position=0, tilt_position=-1.0)
#     previous_movement = Movements.MARCH
#     current_movement = None
#     yellow_prefer_left = True
#     num_steps_since_yellow = 0
#     num_steps_yellow_turn = 0
#     blue_prefer_left = True
#     angle_turn = 0
#
#     while not rospy.is_shutdown() and not mode_change:
#
#         sleep_walk = 1
#         print(angle_turn)
#
#         if obstacles.yellow.is_center and obstacles.yellow.y > 115:
#             # YELLOW is ahead
#             num_steps_since_yellow = 0
#             obstacles.blue.is_last_seen = False
#             sleep_walk = 2
#
#             if control_module.get_offset_degree() > 30:
#                 control_module.walking().turn_left(angle=control_module.get_offset_degree())
#                 print 'angle correction left'
#                 # current_movement = Movements.TURN_LEFT
#                 current_movement = Movements.UNDEFINE
#
#             elif control_module.get_offset_degree() < -30:
#
#                 control_module.walking().turn_right(angle=control_module.get_offset_degree())
#                 current_movement = Movements.UNDEFINE
#                 print 'angle correction right'
#             # elif angle_turn < -30:
#             #     sleep_walk = -angle_turn / 14.7
#             #     current_movement = Movements.TURN_RIGHT
#             elif num_steps_yellow_turn == 3:
#                 # Keeps seeing YELLOW, most likely a sideline
#                 yellow_prefer_left = not yellow_prefer_left
#                 obstacles.yellow.is_last_seen = True
#                 current_movement = Movements.TURN_LEFT if yellow_prefer_left else Movements.TURN_RIGHT
#             if not obstacles.yellow.is_last_seen and \
#                     obstacles.yellow.is_on_left and not obstacles.yellow.is_on_right:
#                 # YELLOW on left
#                 current_movement = Movements.TURN_RIGHT
#                 yellow_prefer_left = False
#             elif not obstacles.yellow.is_last_seen and \
#                     not obstacles.yellow.is_on_left and obstacles.yellow.is_on_right:
#                 # YELLOW on right
#                 current_movement = Movements.TURN_LEFT
#                 yellow_prefer_left = True
#             else:
#                 num_steps_yellow_turn += 1
#
#                 # Turn based on opposite of preference
#                 if not obstacles.yellow.is_last_seen:
#                     yellow_prefer_left = not yellow_prefer_left
#                     obstacles.yellow.is_last_seen = True
#                 current_movement = Movements.TURN_LEFT if yellow_prefer_left else Movements.TURN_RIGHT
#
#
#         elif robot_size=='kid' and obstacles.red.is_on_left or obstacles.red.is_center or obstacles.red.is_on_right:
#             # RED is ahead
#             num_steps_since_yellow += 1
#             num_steps_yellow_turn = 0
#             obstacles.yellow.is_last_seen = False
#             obstacles.blue.is_last_seen = False
#
#             if obstacles.blue.is_on_left or (obstacles.red.is_on_right and obstacles.red.area < 3000):
#                 # RED is to the right side
#                 current_movement = Movements.TURN_RIGHT
#             elif obstacles.blue.is_on_right or (obstacles.red.is_on_left and obstacles.red.area < 3000):
#                 # RED is to the left side
#                 current_movement = Movements.TURN_LEFT
#             else:
#                 # Red obstacle --> get down and start crawling
#                 current_movement = Movements.ACTION
#
#         elif obstacles.blue.is_center and obstacles.blue.y > 50:
#             # BLUE is ahead
#             num_steps_since_yellow += 1
#             num_steps_yellow_turn = 0
#             obstacles.yellow.is_last_seen = False
#
#             if control_module.get_offset_degree() > 30:
#                 control_module.walking().turn_left(angle=control_module.get_offset_degree())
#                 current_movement = Movements.UNDEFINE
#             elif control_module.get_offset_degree() < -30:
#                 control_module.walking().turn_left(angle=control_module.get_offset_degree())
#                 current_movement = Movements.UNDEFINE
#             elif num_steps_since_yellow <= 10:
#                 # Based on yellow preference
#                 current_movement = Movements.TURN_LEFT if yellow_prefer_left else Movements.TURN_RIGHT
#             else:
#                 if not obstacles.blue.is_last_seen and obstacles.blue.is_on_left and not obstacles.blue.is_on_right:
#                     # BLUE on left
#                     current_movement = Movements.TURN_RIGHT
#                     blue_prefer_left = False
#                 elif not obstacles.blue.is_last_seen and not obstacles.blue.is_on_left and obstacles.blue.is_on_right:
#                     # BLUE on right
#                     current_movement = Movements.TURN_LEFT
#                     blue_prefer_left = True
#                 else:
#                     # Turn based on preference derived from old pattern
#                     obstacles.blue.is_last_seen = True
#                     current_movement = Movements.TURN_LEFT if blue_prefer_left else Movements.TURN_RIGHT
#
#         else:
#             # Path is clear --> proceed
#             num_steps_since_yellow += 1
#             num_steps_yellow_turn = 0
#             obstacles.yellow.is_last_seen = False
#             obstacles.blue.is_last_seen = False
#
#             current_movement = Movements.FORWARD
#
#         # Execute movement
#         if previous_movement != current_movement:
#             if current_movement == Movements.MARCH:
#                 control_module.walking().march()
#             elif current_movement == Movements.FORWARD:
#                 control_module.walking().forward()
#             elif current_movement == Movements.BACKWARD:
#                 control_module.walking().backward()
#             elif current_movement == Movements.TURN_LEFT:
#                 angle_turn -= (sleep_walk * 14.7)
#                 control_module.walking().turn_left()
#                 # sleep(sleep_walk)
#             elif current_movement == Movements.TURN_RIGHT:
#                 angle_turn += (sleep_walk * 14.7)
#                 control_module.walking().turn_right()
#                 # sleep(sleep_walk)
#             elif current_movement == Movements.ACTION:
#                 control_module.walking().backward()
#                 sleep(2.5)
#                 control_module.action().play_action('crawl_ready')
#                 control_module.action().play_action('crawl')
#                 control_module.action().play_action('stand_up')
#                 control_module.action().play_action('walk_ready')
#                 control_module.walking().march()
#                 control_module.head().move_head(pan_position=0, tilt_position=-0.9)
#                 sleep(3)
#
#             previous_movement = current_movement
#
#         rate.sleep()
def set_variables():
    global head_pan, head_tilt, yellow_min_top, blue_min_top
    config_variables_file = os.path.join(rospkg.get_pkg_dir('events'), 'config', event,
                                         'variables_{0}.json'.format(robot_size))

    vars = None
    with open(config_variables_file, 'r') as fp:
        vars = json.load(fp)

    head_pan = vars['head_pan']
    head_tilt = vars['head_tilt']
    # replace with coordinates where arrow is aimed
    yellow_min_top = vars['yellow_min_top']
    blue_min_top = vars['blue_min_top']


if __name__ == '__main__':

    # Initalize global variables
    robot_mode = RobotMode.READY
    mode_change = False
    obstacles = Obstacles()
    SPIN_RATE = 0.5
    event = 'obstacle_run'

    robot_size = rospy.get_param('obstacle_run_node/size')
    launch_type = rospy.get_param('obstacle_run_node/launch_type')

    # Wait for other nodes to launch
    wait_for_node('/op3_manager')
    wait_for_node('/vision_node')
    if launch_type == "web":
        wait_for_node('/rosbridge_websocket')

    # Initialze ROS node
    rospy.init_node('obstacle_run_node')
    rate = rospy.Rate(SPIN_RATE)
    set_variables()

    # Initialize OP3 ROS generic subscriber
    button_sub = rospy.Subscriber('/robotis/open_cr/button', String, button_callack)

    # Initialze vision subscribers
    blue_sub = rospy.Subscriber('/vision_node/blue', PositionArea,
                                lambda msg: obstacles.blue.update(msg.x, msg.y, msg.position, msg.area))
    red_sub = rospy.Subscriber('/vision_node/red', PositionArea,
                               lambda msg: obstacles.red.update(msg.x, msg.y, msg.position, msg.area))
    yellow_sub = rospy.Subscriber('/vision_node/yellow', PositionArea,
                                  lambda msg: obstacles.yellow.update(msg.x, msg.y, msg.position, msg.area))

    # Initializing modules
    # control_module = ControlModule(event, robot_size) if launch_type == "terminal" else AdvancedControlModule(event,
    #                                                                                                           robot_size)

    control_module = AdvancedControlModule(event,robot_size)
    # control_module = ControlModule(event, robot_size)
    map_module = MapModule(event, robot_size)

    # Enabling walking module
    sleep(10)
    if robot_size == 'adult':
        robot_mode = RobotMode.RUNNING
        control_module.head().move_head(pan_position=head_pan, tilt_position=head_tilt)

        raw_input('enter tab to start')
        mode_change = True
    elif robot_size == 'kid':
        rospy.loginfo('Press start button to begin.')

    while not rospy.is_shutdown():
        if mode_change:
            mode_change = False

            if robot_mode == RobotMode.READY:
                rospy.loginfo('Resetting OP3. Press start button to begin.')

                control_module.walking().stop()
                control_module.head().move_head(pan_position=head_pan, tilt_position=head_tilt)

            elif robot_mode == RobotMode.RUNNING:
                rospy.loginfo('Starting obstacle run event.')

                avoid_obstacles()

        rate.sleep()


