#!/usr/bin/env python
from enum import Enum
from time import sleep
import rosnode
import rospy
import roslib.packages as rospkg
import os
import json
import math
from std_msgs.msg import String
from control_modules.control_module import AdvancedControlModule
from penalty_obs import Obstacles
from vision.msg import PositionArea, DetectedLines
from tools.follower import Follower
from tools.tracker import Tracker
from events.msg import Track
from slam.srv import ToggleLocalization
from slam.msg import RobotPosition
from robot import Robot, RobotMovement
from map_module.map_module import MapModule


class RobotMode(Enum):
    READY = 0
    RUNNING = 1


class RobotStart(Enum):
    LEFT = 0
    CENTER = 1
    RIGHT = 2


class Movements(Enum):
    MARCH = 0
    FORWARD = 1
    BACKWARD = 2
    TURN_LEFT = 3
    TURN_RIGHT = 4
    ACTION = 5
    UNDEFINE = 6
    FINISH = 7

    SHIFT_LEFT = 8
    SHIFT_RIGHT = 9


class StandState(Enum):
    STAND = 0
    FALLEN_FORWARD = 1
    FALLEN_BACKWARD = 2


class RoboStates(Enum):
    INITIAL = 0
    FIND_BALL = 1
    APPROACH_BALL = 2
    AVOID_OBSTACLES = 3
    APPROACH_SHOOT_AREA = 4
    APPROACH_SHOOT_AREA_2 = 5
    SHOOT = 6
    FINISH = 7
    EXIT = 99


def button_callack(msg):
    global robot_mode, mode_change, stopped
    if stopped:
        if msg.data == 'mode':
            robot_start_pos = RobotStart.LEFT
        elif msg.data == 'start':
            robot_start_pos = RobotStart.CENTER
        elif msg.data == 'user':
            robot_start_pos = RobotStart.RIGHT
    else:
        if msg.data == 'start':
            # Start button pressed
            # Send robot to RUNNING mode
            if robot_mode == RobotMode.READY:
                robot_mode = RobotMode.RUNNING
                mode_change = True
            elif robot_mode == RobotMode.RUNNING:
                rospy.logerr('Robot already in running mode.')
        elif msg.data == 'mode':
            # Mode button pressed
            # Send robot to READY mode
            robot_mode = RobotMode.READY
            mode_change = True
            stopped = True


def wait_for_node(node_name):
    # Wait for node to start
    while node_name not in rosnode.get_node_names():
        rospy.logwarn('Node is not running: {0}'.format(node_name))
        sleep(1)
    rospy.loginfo('Node is running: {0}'.format(node_name))


#
# def goalie_backward():
#     backward_sleep = robot.forward_rate * 8
#     control_module.walking().backward()
#     sleep(backward_sleep)
#     return 'backward'
#
#
# def goalie_forward():
#     forward_sleep = robot.forward_rate * 8
#     control_module.walking().forward()
#     sleep(forward_sleep)
#     return 'forward'


# def goalie_shift(override=None):
#     sidestep_sleep = 2
#     if override == 'left' or (obstacles.circle.is_left and override is None):
#         control_module.walking().walk(gait='gait_step_left')
#         sleep(sidestep_sleep)
#         return 'left'
#     if override == 'right' or (obstacles.circle.is_right and override is None):
#         control_module.walking().walk(gait='gait_step_right')
#         sleep(sidestep_sleep)
#         return 'right'
#
#
# def goalie_rewind(move_history):
#     switch = {
#         'forward': goalie_backward(),
#         'left': lambda: goalie_shift(override='right'),
#         'right': lambda: goalie_shift(override='left')
#     }
#     while move_history:
#         switch[move_history]()
#         sleep(2)
#     return
#
#
# def goalie():
#     global obstacles
#     stack = []
#     track_stack = False
#     while not rospy.is_shutdown() and not mode_change:
#         if obstacles.circle.get_average_area() > min_area and not obstacles.circle.is_center:
#             if track_stack:
#                 stack.append(goalie_shift())
#             else:
#                 goalie_shift()
#         if obstacles.circle.get_average_area() > med_area:
#             track_stack = True
#             stack.append(goalie_forward())
#         if obstacles.circle.get_average_area() > max_area:
#             handle_kick(obstacles.circle.x)
#             return
#         if obstacles.circle.get_average_area() < min_area and track_stack:
#             goalie_rewind(stack)
#             stack = []
#             track_stack = False


# def memorize_lines(msg):
#     global lines, look_at_lines
#
#     if look_at_lines:
#         if 'T' in msg.boundary_line and 'L' in msg.goal_line:
#             lines += ['TL']
#         elif 'L' in msg.goal_line:
#             lines += ['gL']
#         elif 'center' in msg.center_line:
#             lines += ['center']
#         else:
#             lines += ['']
#
#
# def split_list(alist, wanted_parts=1):
#     length = len(alist)
#     return [alist[i * length // wanted_parts: (i + 1) * length // wanted_parts] for i in range(wanted_parts)]


# def play_soccer_slam():
#     global start_following, following_ball, object_found, obj_tracker, track_pub, obstacles, with_slam, rl_angle, robot_size
#
#     movement = None
#     previous_movement = None
#
#     while not rospy.is_shutdown() and not mode_change:
#         tracking_status = obj_tracker.process_tracking()
#
#         # track_pub.publish(obstacles.red.x, obstacles.red.y, obstacles.red.area)
#         obj_tracker.provide_marker(obstacles.circle.x, obstacles.circle.y, obstacles.circle.area)
#
#         if with_slam:
#             movement = robot.next_movement(avoid_obstacle=obstacles.blue.is_center, num_map_rows=NUM_MAP_ROWS,
#                                            num_map_columns=NUM_MAP_COLUMNS)
#
#             if movement == RobotMovement.OBSTACLE:
#                 # Avoid obstacle right in front of robot with localization turned off
#                 rospy.loginfo('[Navigation] avoid obstacle')
#
#                 toggle_localization_srv(pause=True, obstacle=True, start=False, turn_left=False, turn_right=False,
#                                         place_left=False, place_right=False, place_center=False)
#                 control_module.walking().turn_right()
#                 sleep(robot.turn_right_rate / 4.0)
#                 control_module.walking().forward()
#                 sleep(robot.forward_rate * 8)
#                 control_module.walking().turn_left()
#                 sleep(robot.turn_left_rate / 4.0)
#                 control_module.walking().stop()
#                 toggle_localization_srv(pause=False)
#
#             if movement == RobotMovement.TURN_LEFT:
#                 obj_tracker.stop_tracking()
#                 obj_follower.stop_following()
#                 # Turn left
#                 rospy.loginfo('[Navigation] turn left')
#
#                 toggle_localization_srv(pause=True, turn_left=True, start=False, obstacle=False, turn_right=False,
#                                         place_left=False, place_right=False, place_center=False)
#                 control_module.walking().turn_left()
#                 sleep(robot.turn_left_rate)
#                 control_module.walking().stop()
#                 toggle_localization_srv(pause=False)
#                 start_following = True
#
#             elif movement == RobotMovement.TURN_RIGHT:
#                 obj_tracker.stop_tracking()
#                 obj_follower.stop_following()
#                 # Turn right
#                 rospy.loginfo('[Navigation] turn right')
#
#                 toggle_localization_srv(pause=True, turn_right=True, start=False, turn_left=False, obstacle=False,
#                                         place_left=False, place_right=False, place_center=False)
#                 control_module.walking().turn_right()
#                 sleep(robot.turn_right_rate)
#                 control_module.walking().stop()
#                 toggle_localization_srv(pause=False)
#                 start_following = True
#
#             previous_movement = movement
#
#         if start_following:
#             obj_tracker.start_tracking()
#             obj_follower.start_following()
#             start_following = False
#             sleep(1)
#
#         if obstacles.circle.area > min_ball_size:
#             rospy.logerr('before area {}'.format(obstacles.circle.area))
#             if obstacles.circle.area < max_ball_size:
#                 obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0, auto=False)
#             else:
#                 obj_tracker.stop_tracking()
#                 obj_follower.stop_following()
#                 sleep(1)
#                 rospy.logerr('min_ball_size {0} max_ball_size {1}'.format(min_ball_size, max_ball_size))
#                 rospy.logerr('area {}'.format(obstacles.circle.area))
#                 handle_kick(obstacles.circle.x)
#                 start_following = True
#         else:
#             obj_follower.wait_following()


# def play_soccer(end_pos_x, do_shoot=False):
#     global start_following, following_ball, object_found, obj_tracker, track_pub, obstacles, with_slam, angle_robot, robot_size, states
#     movement = None
#     previous_movement = None
#     count = 0
#     angle_robot = 0
#     done = False
#     while not done:
#         current_movement = None
#         tracking_status = obj_tracker.process_tracking()
#         # track_pub.publish(obstacles.red.x, obstacles.red.y, obstacles.red.area)
#         obj_tracker.provide_marker(obstacles.circle.x, obstacles.circle.y, obstacles.circle.area)
#
#         if start_following:
#             obj_tracker.start_tracking()
#             obj_follower.start_following()
#             start_following = False
#             sleep(1)
#
#         if obstacles.circle.get_average_area() > min_ball_size:
#             print(obstacles.circle.area)
#             if obstacles.circle.get_average_area() < max_ball_size:
#                 rl_angle = obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0, auto=False)
#                 print('rl_angle', rl_angle)
#                 rl_angle = int(round(math.degrees(rl_angle)))
#                 rl_angle = (angle_robot + rl_angle) % 360
#                 # control_module.plan_orientation['angle'] = rl_angle
#                 angle_robot = rl_angle
#                 count = 0
#                 current_movement = None
#             else:
#                 count += 1
#                 if count > 3:
#                     obj_tracker.stop_tracking()
#                     obj_follower.stop_following()
#                     sleep(1)
#                     # handle_kick(obstacles.circle.x)
#                     if do_shoot:
#                         handle_kick(obstacles.circle.x)
#                         done = True
#                         current_movement = Movements.FINISH
#                     else:
#                         current_movement = Movements.FORWARD
#                     obj_follower.stop_following()
#                     obj_tracker.stop_tracking()
#                     control_module.head().move_head(pan_position=0, tilt_position=-0.9)
#                     start_following = True
#                     count = 0
#         else:
#             count = 0
#             obj_follower.wait_following()
#             current_movement = None
#
#         if control_module.plan_orientation['x'] < end_pos_x :
#             done = True
#
#
#         # handle walk
#         if previous_movement != current_movement and current_movement is not None:
#             if current_movement == Movements.FORWARD:
#                 control_module.walking().forward(step= 5)
#         previous_movement = current_movement

def handle_kick(obj_position):
    # print('angle_robot', angle_robot, obj_position)
    control_module.head().move_head(pan_position=0, tilt_position=-0.9)
    sleep(0.25)
    angle_robot = control_module.plan_orientation['angle']
    print('turning right', angle_robot)
    if angle_robot < 0:
        control_module.walking().turn_left(angle=float(angle_robot))
    else:
        control_module.walking().turn_right(angle=float(angle_robot))
    if obj_position > 213:  # 213 is half of resized vision resolution
        control_module.action().play_action('right_kick')
    else:
        control_module.action().play_action('left_kick')
    control_module.action().play_action('walk_ready')
    sleep(1)
    control_module.head().move_head(pan_position=0, tilt_position=-1.0)


# def avoid_obstacles(pan, tilt,state = 1  , mode='turn'):
#     global mode_change, control_module, obstacles, rate, states
#     sleep(3)
#     control_module.head().move_head(pan_position=pan, tilt_position=tilt)
#     obstacles.blue.is_last_seen = False
#     previous_movement = Movements.MARCH
#
#     blue_prefer_left = True
#     angle_turn = 0
#     done = False
#
#     while not done:
#
#         if obstacles.blue.is_center:
#             # BLUE is ahead
#
#             if not obstacles.blue.is_last_seen and obstacles.blue.is_on_left and not obstacles.blue.is_on_right:
#                 # BLUE on left
#                 current_movement = Movements.TURN_RIGHT if mode == 'turn' else Movements.SHIFT_RIGHT
#                 blue_prefer_left = False
#             elif not obstacles.blue.is_last_seen and not obstacles.blue.is_on_left and obstacles.blue.is_on_right:
#                 # BLUE on right
#                 current_movement = Movements.TURN_LEFT if mode == 'turn' else Movements.SHIFT_LEFT
#                 blue_prefer_left = True
#             else:
#                 # Turn based on preference derived from old pattern
#                 obstacles.blue.is_last_seen = True
#                 if mode == 'turn':
#                     current_movement = Movements.TURN_LEFT if blue_prefer_left else Movements.TURN_RIGHT
#                 else:
#                     current_movement = Movements.SHIFT_LEFT if blue_prefer_left else Movements.SHIFT_RIGHT
#
#         elif not obstacles.blue.is_center and obstacles.circle.is_center:
#             # Path is clear --> proceed
#
#             obstacles.blue.is_last_seen = False
#
#             current_movement = Movements.FINISH
#
#         else:
#             if obstacles.circle.is_left:
#                 current_movement = Movements.TURN_RIGHT if mode == 'turn' else Movements.SHIFT_RIGHT
#             else:
#                 current_movement = Movements.TURN_LEFT if mode == 'turn' else Movements.SHIFT_LEFT
#
#         if obstacles.circle.get_average_area == -1 and state == 1:
#             current_movement = Movements.FINISH
#
#         # Execute movement
#         if previous_movement != current_movement:
#             if current_movement == Movements.SHIFT_LEFT:
#                 control_module.walking().stright_walk_helper('shift_left')
#             elif current_movement == Movements.SHIFT_RIGHTS:
#                 control_module.walking().stright_walk_helper('shift_right')
#             elif current_movement == Movements.TURN_LEFT:
#                 control_module.walking().turning_helper('turn_left')
#             elif current_movement == Movements.TURN_RIGHT:
#
#                 control_module.walking().turning_helper('turn_right')
#
#
#             elif current_movement == Movements.FINISH:
#                 done = True
#
#                 control_module.stop()
#
#             previous_movement = current_movement
#         rate.sleep()


def set_variables():
    global min_ball_size, max_ball_size, min_area, med_area, max_area, head_pan, head_tilt, yellow_min_top, blue_min_top
    config_variables_file = os.path.join(rospkg.get_pkg_dir('events'), 'config', event,
                                         'variables_{0}.json'.format(robot_size))
    vars = None
    with open(config_variables_file, 'r') as fp:
        vars = json.load(fp)
    min_ball_size = int(vars['min_ball_size'])
    max_ball_size = int(vars['max_ball_size'])

    head_pan = vars['head_pan']
    head_tilt = vars['head_tilt']
    # replace with coordinates where arrow is aimed
    yellow_min_top = vars['yellow_min_top']
    blue_min_top = vars['blue_min_top']


def cacl_goal_angle():
    goal_pos = [0, field.map['height'] / 2]
    to_goal = math.atan2(goal_pos[1] - control_module.plan_orientation['y'],
                         goal_pos[0] - control_module.plan_orientation['x']) - math.radians(
        control_module.plan_orientation['angle'])
    if to_goal < 0:
        to_goal += math.pi
    temp = math.radians(control_module.plan_orientation['angle']) if math.radians(
        control_module.plan_orientation['angle']) > 0 else math.pi + math.radians(
        control_module.plan_orientation['angle'])
    return 'turn_left_with_ball' if to_goal > temp else 'turn_right_with_ball', abs(to_goal - temp)


def is_obstacles_front( obj_tracker, obj_follower,current_tilt):
    tilt = [-70, -50, -30, -10]
    obj_tracker.stop_tracking()
    obj_follower.stop_following()
    for i in tilt:
        control_module.head().move_head(pan_position=-math.radians(control_module.plan_orientation['offset']),
                                        tilt_position=math.radians(i))
        rospy.loginfo('[states info] scan: ' + str(tilt) + ' degree')
        sleep(0.5)
        if obstacles.blue.is_center:
            control_module.head().move_head(pan_position=math.radians(control_module.plan_orientation['offset']),
                                            tilt_position=current_tilt)
            sleep(1)
            obj_tracker.start_tracking()
            obj_follower.start_following()
            return True
    control_module.head().move_head(pan_position=math.radians(control_module.plan_orientation['offset']),
                                    tilt_position=current_tilt)
    sleep(1)
    obj_tracker.start_tracking()
    obj_follower.start_following()
    return False


def process_penalty_kick():
    global states
    # setting up event
    control_module.action().play_action('arm_up')
    control_module.head().change_head_scan(min_pan=-50.0, max_pan=50.0, min_tilt=-85, max_tilt=-5)
    tracking_status = obj_tracker.process_tracking()

    is_tracking = False
    is_following = False
    is_turned = False
    is_walk = False
    while not rospy.is_shutdown():
        tracking_status = obj_tracker.process_tracking()

        if states == RoboStates.INITIAL:
            obj_tracker.provide_marker(obstacles.circle.x, obstacles.circle.y, obstacles.circle.area)
            # find the ball
            if not is_tracking:
                obj_tracker.start_tracking()
                is_tracking = not is_tracking
                rospy.loginfo('[states info] ' + str(states))
            if 0.4 < obstacles.circle.x / 427 < 0.6 and 0.4 < obstacles.circle.y / 320 < 0.6:
                states = RoboStates.APPROACH_BALL
        if states == RoboStates.APPROACH_BALL:
            obj_tracker.provide_marker(obstacles.circle.x, obstacles.circle.y, obstacles.circle.area)
            if not is_following:
                is_following = not is_following
                obj_follower.start_following()
                rospy.loginfo('[states info] ' + str(states))
            if obstacles.circle.get_average_area() > min_ball_size:
                obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0, auto=False)
                # if ball is near robot foot
                # please test which approach is better:
                # approach 1
                # if obj_tracker.get_tilt()>-80 :
                # approach 2:
                # if obstacles.circle.get_average_area()>max_ball_size:
                # approach the ball
                if 0.4 < obstacles.circle.x / 427 < 0.5 and 0.65 < obstacles.circle.y / 320 < 0.9:
                    rospy.loginfo('[states info] ball approached')
                    states = RoboStates.AVOID_OBSTACLES
            else:
                obj_follower.wait_following()
        if states == RoboStates.AVOID_OBSTACLES:
            if not is_turned:
                # path is clear
                if not obstacles.blue.is_center:
                    # control_module.walking().stright_walk_helper('forward_with_ball')
                    obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0,
                                                   'forward_with_ball', auto=False)
                else:
                    # the first obstacle is always in front of the ball, turn left
                    # is ensrue the robot will only turn 90 degree offset to the field

                    # control_module.walking().turning_helper('turn_left_with_ball', angle= 90 - control_module.plan_orientation['offset'])
                    # control_module.walking().stright_walk_helper('forward_with_ball', step=2)
                    required_step = control_module.get_walk_step('turn_left_with_ball',
                                                                 90 - control_module.plan_orientation['offset'])
                    obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0,
                                                   'forward_with_ball', auto=False)
                    control_module.wait_step_finish(required_step)
                    obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0,
                                                   'forward_with_ball', auto=False)

                    rospy.loginfo('[states info] ' + str(states))
                    is_turned = not is_turned
            else:
                if is_obstacles_front( obj_tracker,obj_follower ,obj_tracker.get_tilt()):
                    # turn has been made, if obstacle still in the center, walk 2 more step
                    # control_module.walking().stright_walk_helper('forward_with_ball', step=2)
                    obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0,
                                                   'forward_with_ball', auto=False)
                    sleep(2)


                else:
                    required_step = control_module.get_walk_step('turn_right_with_ball',
                                                                  - control_module.plan_orientation['offset'])
                    obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0,
                                                   'turn_right_with_ball', auto=False)
                    control_module.wait_step_finish(required_step)

                    # control_module.walking().turning_helper('turn_right_with_ball',
                    #                                         angle=control_module.plan_orientation['offset'])
                    # control_module.walking().stright_walk_helper('forward_with_ball')
                    obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0,
                                                   'forward_with_ball', auto=False)
                    rospy.loginfo('[states info] obstacles cleared')
                    is_turned = False
                    states = RoboStates.APPROACH_SHOOT_AREA
            if control_module.plan_orientation['x'] < 150:
                states = RoboStates.APPROACH_SHOOT_AREA

        if states == RoboStates.APPROACH_SHOOT_AREA:
            if not is_turned:
                mode, angle = cacl_goal_angle()
                # control_module.walking().turning_helper(mode, angle=angle)

                required_step = control_module.get_walk_step(mode, angle)
                obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0, mode, auto=False)
                control_module.wait_step_finish(required_step)


                # control_module.walking().stright_walk_helper('forward_with_ball')

                obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0,
                                               'forward_with_ball', auto=False)
                if control_module.plan_orientation['x'] < 110:
                    rospy.loginfo('[states info] shooting area approached')
                    states = RoboStates.SHOOT
        if states == RoboStates.SHOOT:
            control_module.action().play_action('right_kick')
            rospy.loginfo('[states info] SHOOT!')
            states = RoboStates.EXIT


if __name__ == '__main__':
    # Initalize global variables
    robot_mode = RobotMode.READY
    mode_change = False
    robot = Robot(forward_rate=1.0, left_rate=10.0, right_rate=9.0)
    obstacles = Obstacles()
    SPIN_RATE = 0.5
    NUM_MAP_ROWS, NUM_MAP_COLUMNS = 19, 28
    event = 'penalty_kick'
    start_following = True
    look_at_lines = False
    stopped = False
    robot_start_pos = RobotStart.CENTER
    lines = []
    states = RoboStates.INITIAL
    with_slam = rospy.get_param('penalty_kick_node/with_slam')
    robot_size = rospy.get_param('penalty_kick_node/size')
    field = MapModule(event, robot_size)

    set_variables()  # Important !
    # Wait for other nodes to launch
    wait_for_node('/op3_manager')
    wait_for_node('/vision_node')
    if with_slam:
        wait_for_node('/slam_node')
    # Initialze ROS node
    rospy.init_node('penalty_kick_node')
    rate = rospy.Rate(SPIN_RATE)
    # Initialize OP3 ROS generic subscriber
    button_sub = rospy.Subscriber('/robotis/open_cr/button', String, button_callack)
    # Initialze vision subscribers
    # line_vision_sub = rospy.Subscriber('/vision_node/lines', DetectedLines, memorize_lines)
    blue_sub = rospy.Subscriber('/vision_node/blue', PositionArea,
                                lambda msg: obstacles.blue.update(msg.x, msg.y, msg.position, msg.area))
    red_sub = rospy.Subscriber('/vision_node/red', PositionArea,
                               lambda msg: obstacles.red.update(msg.x, msg.y, msg.position, msg.area))
    yellow_sub = rospy.Subscriber('/vision_node/yellow', PositionArea,
                                  lambda msg: obstacles.yellow.update(msg.x, msg.y, msg.position, msg.area))
    mixed_sub = rospy.Subscriber('/vision_node/mixed', PositionArea,
                                 lambda msg: obstacles.mixed.update(msg.x, msg.y, msg.position, msg.area))
    circle_sub = rospy.Subscriber('/vision_node/circle', PositionArea,
                                  lambda msg: obstacles.circle.update(msg.x, msg.y, msg.position, msg.area))
    track_pub = rospy.Publisher('/track', Track, queue_size=0)
    # SLAM:
    if with_slam:
        toggle_localization_srv = rospy.ServiceProxy('/slam/toggle_localization', ToggleLocalization)
        robot_position_sub = rospy.Subscriber('/slam/robot_position', RobotPosition, robot.update_pose)
    # Initializing modules
    control_module = AdvancedControlModule(event, robot_size)
    control_module.plan_orientation['x'] = field.map['initial_pos']['x']
    control_module.plan_orientation['y'] = field.map['initial_pos']['y']
    control_module.plan_orientation['angle'] = math.degrees(field.map['initial_pos']['angle'])

    print control_module.plan_orientation
    obj_tracker = Tracker(event, robot_size, control_module)
    obj_follower = Follower(event, robot_size, control_module, True)
    # Enabling walking module
    sleep(10)
    control_module.action().play_action('walk_ready')
    sleep(0.5)
    control_module.head().move_head(pan_position=0, tilt_position=-1.1)
    if robot_size == 'adult':
        robot_mode = RobotMode.RUNNING
        mode_change = True
        raw_input('tab Enter to start')
    elif robot_size == 'kid':
        rospy.loginfo('Press start button to begin.')
    while not rospy.is_shutdown():
        if mode_change:
            mode_change = False
            if robot_mode == RobotMode.READY:
                rospy.loginfo('Resetting OP3. Press start button to begin.')
                control_module.walking().stop()
                control_module.head().move_head(pan_position=0, tilt_position=-0.85)
                robot_mode = RobotMode.RUNNING
            elif robot_mode == RobotMode.RUNNING:
                rospy.loginfo('Starting united soccer event.')
                process_penalty_kick()

        rate.sleep()
