#!/usr/bin/env python

from enum import Enum
from time import sleep
from math import pi
import sys
import rosnode
import rospy
from std_msgs.msg import String
from events.msg import remote_control_action, remote_control_walking, remote_control_head
from os import system

from control_modules.control_module import AdvancedControlModule
from map_module.map_module import MapModule

class RobotMode(Enum):
    READY = 0
    RUNNING = 1


def button_callack(msg):
    global robot_mode, mode_change

    if msg.data == 'start':
        # Start button pressed
        # Send robot to RUNNING mode
        if robot_mode == RobotMode.READY:
            robot_mode = RobotMode.RUNNING
            mode_change = True
            control_module.action().play_action('walk_ready')
        elif robot_mode == RobotMode.RUNNING:
            rospy.logerr('Robot already in running mode.')

    elif msg.data == 'mode':
        # Mode button pressed
        # Send robot to READY mode
        robot_mode = RobotMode.READY
        mode_change = True


def process_walking_call_back(msg):
    if msg.movement == 'march':
        control_module.walking().march()

    elif msg.movement == 'forward':
        control_module.walking().forward()
        process_new_msg_publish('forward')

    elif msg.movement == 'backward':
        control_module.walking('backward').backward()
        process_new_msg_publish('backward')

    elif msg.movement == 'turn_left':
        control_module.walking('turn_left').turn_left()
        process_new_msg_publish('turn_left')

    elif msg.movement == 'turn_right':
        control_module.walking('turn_right').turn_right()
        process_new_msg_publish('turn_right')

    elif msg.movement == 'shift_left':
        control_module.walking('shift_left').shift_left()
        process_new_msg_publish('shift_left')

    elif msg.movement == 'shift_right':
        control_module.walking('shift_right').shift_right()
        process_new_msg_publish('shift_right')

    elif msg.movement == 'stop':
        control_module.walking().stop()
        process_new_msg_publish('stop')
    print(msg.movement)


def process_head_movement_call_back(msg):
    newAngle = msg.data * pi / 180
    if msg.jointName == 'h':
        camera_angle['pan_position'] = newAngle
    elif msg.jointName == 'v':
        camera_angle['tilt_position'] = newAngle
    control_module.head().move_head(camera_angle['pan_position'], camera_angle['tilt_position'])


def process_action_call_back(msg):
    robot_action = None
    if msg.data == "1":
        robot_action = "push_door"
    elif msg.data == "2":
        robot_action = "turn_pipe"
    elif msg.data == "0":
        robot_action = "reset"
    # elif msg.data == "4":
    #     robot_action = "right_kick"
    # elif msg.data == "5":
    #     robot_action = "left_kick"
    if robot_action is not None:
        control_module.action().play_action(robot_action)


def wait_for_node(node_name):
    # Wait for node to start
    while node_name not in rosnode.get_node_names():
        rospy.logwarn('Node is not running: {0}'.format(node_name))
        sleep(1)

    rospy.loginfo('Node is running: {0}'.format(node_name))
    rospy.loginfo('{0} started'.format(node_name))


def copy_html():
    # require sudo permission

    system("sudo cp -r ./html /var/www")


def process_new_msg_publish(msg):

    new_msg = String()
    new_msg.data = msg
    ros_message_pub.publish(new_msg)


if __name__ == '__main__':

    # Initalize global variables
    print "hello"
    robot_mode = RobotMode.READY
    mode_change = False
    SPIN_RATE = 0.5
    event = 'remote_control_server'
    camera_angle = {'pan_position': 0, 'tilt_position': -0.8}
    # copy_html()

    # Wait for other nodes to launch
    wait_for_node('/op3_manager')
    wait_for_node('/rosbridge_websocket')
    wait_for_node('/usb_cam_node')
    rospy.loginfo('Start Teleop')
    # Initialze ROS node
    rospy.init_node('remote_control_server_node')
    rate = rospy.Rate(SPIN_RATE)

    # Initialize OP3 ROS generic subscriber
    button_sub = rospy.Subscriber('/robotis/open_cr/button', String, button_callack)

    # Initialze vision subscribers
    walking_sub = rospy.Subscriber('/telop_control/remote_control_walking', remote_control_walking,
                                   process_walking_call_back)
    head_moving_sub = rospy.Subscriber('/telop_control/remote_control_head', remote_control_head,
                                       process_head_movement_call_back)
    action_sub = rospy.Subscriber('/telop_control/remote_control_action', String, process_action_call_back)

    ros_message_pub = rospy.Publisher(
        '/events/remote_controller/ros_msg', String, queue_size=1)


    # Initializing modules
    robot_size = rospy.get_param('remote_control_server_node/size')
    control_module = AdvancedControlModule(event, robot_size)
    map_module = MapModule(event,robot_size,"default")

    # Enabling walking module
    sleep(10)
    rospy.loginfo('Press start button to begin.')
    process_new_msg_publish('Press start button to begin.')
    control_module.head().move_head(pan_position=camera_angle['pan_position'],
                                    tilt_position=camera_angle['tilt_position'])

    while not rospy.is_shutdown():
        if mode_change:
            mode_change = False

            if robot_mode == RobotMode.READY:
                rospy.loginfo('Resetting OP3. Press start button to begin.')
                process_new_msg_publish('Resetting OP3. Press start button to begin.')

                control_module.walking().stop()
                control_module.head().move_head(pan_position=camera_angle['pan_position'],
                                                tilt_position=camera_angle['tilt_position'])
                print("robot ready")

            elif robot_mode == RobotMode.RUNNING:
                rospy.loginfo('Starting teleop event.')

        rate.sleep()
