#!/usr/bin/env python

from enum import Enum
from time import sleep

import rosnode
import rospy
from std_msgs.msg import String
import roslib.packages as rospkg
import os
import time
import json

from control_modules.control_module import ControlModule
# from marker import Markers
from obstacles import Obstacles
from vision.msg import PositionArea
from tools.follower import Follower
from tools.tracker import Tracker
from events.msg import Track

class RobotMode(Enum):
    READY = 0
    RUNNING = 1

class SubEvent(Enum):
    TERRAIN = 0
    LADDER = 1

class Movements(Enum):
    MARCH = 0
    FORWARD = 1
    BACKWARD = 2
    TURN_LEFT = 3
    TURN_RIGHT = 4
    ACTION = 5


def button_callack(msg):
    global robot_mode, mode_change, sub_event

    if msg.data == 'start':
        # Start button pressed
        # Send robot to RUNNING mode
        if robot_mode == RobotMode.READY:
            robot_mode = RobotMode.RUNNING
            mode_change = True
        elif robot_mode == RobotMode.RUNNING:
            rospy.logerr('Robot already in running mode.')

    elif msg.data == 'mode':
        # Mode button pressed
        # Send robot to READY mode
        robot_mode = RobotMode.READY
        mode_change = True

        if sub_event == SubEvent.TERRAIN:
            sub_event = SubEvent.LADDER
            print('Ladder Event Selected')
        elif sub_event == SubEvent.LADDER:
            sub_event = SubEvent.TERRAIN
            print('Terrain Event Selected')


def wait_for_node(node_name):
    # Wait for node to start
    while node_name not in rosnode.get_node_names():
        rospy.logwarn('Node is not running: {0}'.format(node_name))
        sleep(1)

    rospy.loginfo('Node is running: {0}'.format(node_name))


def uneven_terrain():
    global mode_change, control_module, rate, start_following, layer, times, stage

    stage = 0

    while not rospy.is_shutdown() and not mode_change:
        track_pub.publish(obstacles.green.x, obstacles.green.y, obstacles.green.area)

        #print(times)
        if stage == 1:
            track_pub.publish(obstacles.blue.x, obstacles.blue.y, obstacles.blue.area)
            obj_tracker.provide_marker(obstacles.blue.x, obstacles.blue.y, obstacles.blue.area)
        elif stage == 2:
            track_pub.publish(obstacles.yellow.x, obstacles.yellow.y, obstacles.yellow.area)
            obj_tracker.provide_marker(obstacles.yellow.x, obstacles.yellow.y, obstacles.yellow.area)
        elif stage == 3:
            track_pub.publish(obstacles.red.x, obstacles.red.y, obstacles.red.area)
            obj_tracker.provide_marker(obstacles.red.x, obstacles.red.y, obstacles.red.area)

        tracking_status = obj_tracker.process_tracking()

        if stage == 0:
            # print('blue:', obstacles.blue.area)
            # print('red:', obstacles.red.area)
            # print('yellow:', obstacles.yellow.area)
            control_module.head().move_head(pan_position=0, tilt_position=-1.4)
            # print('front bar:', obstacles.red.y)
            # print('blue:', obstacles.blue.y)
            stage = 1

        if stage == 1:
            if start_following:
                obj_tracker.start_tracking()
                sleep(1)
                obj_follower.start_following()
                start_following = False
                sleep(1)

            # Find and follow the largest marker, or first layer
            if obstacles.blue.area > 75000: # Numbers are arbitrary until tuning/testing
                start = time.time()
                obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0, 'follow')
                if obstacles.blue.area > 80000:
                    print(obstacles.blue.y)
                    obj_follower.stop_following()
                    obj_tracker.stop_tracking()
                    control_module.head().move_head(pan_position=0, tilt_position=-1.4)
                    end = time.time()
                    # sleep(3)
                    if obstacles.blue.y >= 155 and obstacles.green.y > 200:
                        control_module.walking().stop()
                        control_module.action().play_action('step_up') 
                        layer += 1
                        stage = 2
                        times.append(end - start)
                        start_following = True
                        control_module.action().play_action('walk_ready')

                    elif obstacles.blue.y < 155 and obstacles.green.y > 200:
                        control_module.walking().walk('short_steps')
        if stage == 2:
            if start_following:
                obj_tracker.start_tracking()
                sleep(1)
                obj_follower.start_following()
                start_following = False
                sleep(1)

            if obstacles.yellow.area > 40000:
                start = time.time()
                obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0, 'follow')
                if obstacles.yellow.area > 50000: 
                    obj_follower.stop_following()
                    obj_tracker.stop_tracking()
                    control_module.head().move_head(pan_position=0, tilt_position=-1.4)
                    end = time.time()

                    if obstacles.yellow.y < 150 and obstacles.green.y > 200:
                        control_module.walking().walk('short_steps') 
                    #sleep(2)
                    elif obstacles.yellow.y >= 150 and obstacles.green.y > 200:
                        control_module.walking().stop()
                        control_module.action().play_action('step_up') 
                        layer += 1
                        stage = 3
                        times.append(end - start)
                        print(times[1])
                        start_following = True

        if stage == 3:
            if start_following:
                obj_tracker.start_tracking()
                sleep(1)
                obj_follower.start_following()
                start_following = False
                sleep(1)
                
            if obstacles.red.area > 2000:
                start = time.time()
                obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0, 'follow')
                if obstacles.red.area > 2500: 
                    obj_follower.stop_following()
                    obj_tracker.stop_tracking()
                    control_module.head().move_head(pan_position=0, tilt_position=-1.4)
                    if obstacles.red.y < 150 and obstacles.green.y > 200:
                        control_module.walking().walk('short_steps') #Define a gait with a low z amplitude so the robot doesn't trip
                        # sleep(2)
                    elif obstacles.red.y >= 150:
                        end = time.time()
                        control_module.walking().stop()
                        control_module.action().play_action('step_up') #Define an action to advance to the next level
                        layer += 1
                        stage = 4
                        times.append(end - start)
                        print(times[2])
        if stage == 4:
            control_module.walking().walk('turn_right')
            sleep(5)
            # Turn 180 degrees
            control_module.walking().walk('forward')
            sleep(1)
            control_module.action().play_action('step_down')

            control_module.walking().walk('forward')
            print(times[2])
            sleep(times[2])
            control_module.action().play_action('step_down')
            
            control_module.walking().walk('forward')
            print(times[1])
            sleep(times[1])
            control_module.action().play_action('step_down')

            control_module.walking().walk('forward')

def ladder_climb():
    global mode_change, control_module, rate, start_following

    stage = 0

    while not rospy.is_shutdown() and not mode_change:
        tracking_status = obj_tracker.process_tracking()
        red_x_pub, red_y_pub = obstacles.red.x, obstacles.red.y

        obj_tracker.provide_marker(red_x_pub, red_y_pub, obstacles.red.area)

        if stage == 0:
            stage = 1

        # Approach the ladder based on follower
        if stage == 1: 
            if start_following:
                obj_tracker.start_tracking()
                obj_follower.start_following()
                start_following = False

            if obstacles.red.area > 300 :
                obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0, 'follow', True)

                if obstacles.red.area > 3400: # Tune this number
                    stage = 2
            
        # Do the actions
        if stage == 2:
            control_module.walking().stop()
            control_module.action().play_action('ladder_arms')
            sleep(1)
            control_module.action().play_action('ladder_legs')
            stage = 3

        if stage == 3:
            print('Done')


def set_variables():
    global min_marker_size, max_marker_size
    config_variables_file = os.path.join(rospkg.get_pkg_dir('events'), 'config', event, 'variables_{0}.json'.format(robot_size))

    vars = None
    with open(config_variables_file, 'r') as fp:
            vars = json.load(fp)
    
    min_marker_size = vars['min_marker_size']
    max_marker_size = vars['max_marker_size']


if __name__ == '__main__':

    # Initalize global variables
    robot_mode = RobotMode.READY
    sub_event = SubEvent.TERRAIN
    mode_change = False
    obstacles = Obstacles()
    SPIN_RATE = 0.5
    event = 'spartan_race'
    start_following = True
    stage = 0
    layer = 0
    times = []

    robot_size = rospy.get_param('spartan_race_node/size')

    set_variables() # Important !

    # Wait for other nodes to launch
    wait_for_node('/op3_manager')
    wait_for_node('/vision_node')

    # Initialze ROS node
    rospy.init_node('spartan_race_node')
    rate = rospy.Rate(SPIN_RATE)

    # Initialize OP3 ROS generic subscriber
    button_sub = rospy.Subscriber('/robotis/open_cr/button', String, button_callack)

    # Initialze vision subscribers
    red_sub = rospy.Subscriber('/vision_node/red', PositionArea,
                                lambda msg: obstacles.red.update(msg.x, msg.y, msg.position, msg.angle, msg.area))
    blue_sub = rospy.Subscriber('/vision_node/blue', PositionArea,
                               lambda msg: obstacles.blue.update(msg.x, msg.y, msg.position, msg.angle, msg.area))
    yellow_sub = rospy.Subscriber('/vision_node/yellow', PositionArea,
                               lambda msg: obstacles.yellow.update(msg.x, msg.y, msg.position, msg.angle, msg.area))
    green_sub = rospy.Subscriber('/vision_node/green', PositionArea,
                               lambda msg: obstacles.green.update(msg.x, msg.y, msg.position, msg.angle, msg.area))

    track_pub = rospy.Publisher('/track', Track, queue_size=0)

    # Initializing modules
    control_module = ControlModule(event, robot_size)

    obj_tracker = Tracker(event, robot_size, control_module)
    obj_follower = Follower(event, robot_size, control_module)

    # Enabling walking module
    sleep(10)
    control_module.action().play_action('walk_ready')
    if robot_size == 'adult':
        robot_mode = RobotMode.RUNNING
        mode_change = True
    elif robot_size == 'kid':
        rospy.loginfo('Press start button to begin.')

    while not rospy.is_shutdown():
        if mode_change:
            mode_change = False

            if robot_mode == RobotMode.READY:
                rospy.loginfo('Resetting OP3. Press start button to begin.')

                control_module.walking().stop()
                control_module.head().move_head(pan_position=0, tilt_position=-0.8)
                #begin_sprint()
            elif robot_mode == RobotMode.RUNNING:
                if sub_event == SubEvent.LADDER:
                    print('Starting ladder climb event.')
                    ladder_climb()
                elif sub_event == SubEvent.TERRAIN:
                    print('Starting uneven terrain event')
                    uneven_terrain()


        rate.sleep()



