#!/usr/bin/env python

from enum import Enum
from time import sleep

import rosnode
import rospy
from std_msgs.msg import String
import roslib.packages as rospkg
import os
import json

from control_modules.control_module import ControlModule
# from marker import Markers
from obstacles import Obstacles
from vision.msg import PositionArea
from tools.follower import Follower
from tools.tracker import Tracker
from events.msg import Track

class RobotMode(Enum):
    READY = 0
    RUNNING = 1


class Movements(Enum):
    MARCH = 0
    FORWARD = 1
    BACKWARD = 2
    TURN_LEFT = 3
    TURN_RIGHT = 4
    ACTION = 5


def button_callack(msg):
    global robot_mode, mode_change

    if msg.data == 'start':
        # Start button pressed
        # Send robot to RUNNING mode
        if robot_mode == RobotMode.READY:
            robot_mode = RobotMode.RUNNING
            mode_change = True
        elif robot_mode == RobotMode.RUNNING:
            rospy.logerr('Robot already in running mode.')

    elif msg.data == 'mode':
        # Mode button pressed
        # Send robot to READY mode
        robot_mode = RobotMode.READY
        mode_change = True


def wait_for_node(node_name):
    # Wait for node to start
    while node_name not in rosnode.get_node_names():
        rospy.logwarn('Node is not running: {0}'.format(node_name))
        sleep(1)

    rospy.loginfo('Node is running: {0}'.format(node_name))


def begin_sprint():
    global mode_change, control_module, rate, start_following, stage, blind_backwards

    control_module.walking().stop()
    sleep(0.1)

    while not rospy.is_shutdown() and not mode_change:
        tracking_status = obj_tracker.process_tracking()

        blue_x_pub, blue_y_pub = obstacles.blue.x, obstacles.blue.y 

        obj_tracker.provide_marker(blue_x_pub, blue_y_pub, obstacles.blue.area)

        if stage == 0:
            if robot_size == 'adult':
                control_module.walking().walk('initial')
                sleep(1)
            stage = 1

        elif stage == 1:
            if start_following:
                obj_tracker.start_tracking()
                sleep(2)
                obj_follower.start_following()
                start_following = False
                sleep(1)
            
            if obstacles.blue.area > min_marker_size:
                obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0, 'follow', auto=False)
                if obstacles.blue.area > max_marker_size:
                    stage = 2
                    start_following = True

        elif stage == 2:
            # Go backwards
            if start_following:
                control_module.walking().walk('march')
                sleep(1)
                obj_tracker.start_tracking()
                obj_follower.start_following()
                start_following = False
                
                # obj_tracker.stop_tracking()
                # control_module.head().move_head(pan_position=0, tilt_position=-0.8)                
                count = 0
                

            obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(),0 , 'backward', auto=False)
            # if blind_backwards:
            #     control_module.walking().walk('backward')
            #     sleep(30)
            # else:                
            #     while count < 5:
            #         if obstacles.red.angle >= 75 and obstacles.red.angle <= 125:
            #             control_module.walking().walk('backward')
            #             count+=1
            #             sleep(5)
            #         elif obstacles.red.angle < 75 and obstacles.red.angle > 0:
            #             control_module.walking().walk('turn_left')
            #             count+=1
            #             sleep(1)
            #         elif obstacles.red.angle > 125:
            #             count+=1
            #             control_module.walking().walk('turn_right')
            #             sleep(1)



def set_variables():
    global min_marker_size, max_marker_size
    config_variables_file = os.path.join(rospkg.get_pkg_dir('events'), 'config', event, 'variables_{0}.json'.format(robot_size))

    vars = None
    with open(config_variables_file, 'r') as fp:
            vars = json.load(fp)
    
    min_marker_size = vars['min_marker_size']
    max_marker_size = vars['max_marker_size']


if __name__ == '__main__':

    # Initalize global variables
    robot_mode = RobotMode.READY
    mode_change = False
    obstacles = Obstacles()
    SPIN_RATE = 0.5
    event = 'sprint'
    start_following = True
    stage = 0
    blind_backwards = False

    robot_size = rospy.get_param('sprint_node/size')

    set_variables() # Important !

    # Wait for other nodes to launch
    wait_for_node('/op3_manager')
    wait_for_node('/vision_node')

    # Initialze ROS node
    rospy.init_node('sprint_node')
    rate = rospy.Rate(SPIN_RATE)

    # Initialize OP3 ROS generic subscriber
    button_sub = rospy.Subscriber('/robotis/open_cr/button', String, button_callack)

    # Initialze vision subscribers
    blue_sub = rospy.Subscriber('/vision_node/pink', PositionArea,
                                lambda msg: obstacles.blue.update(msg.x, msg.y, msg.position, msg.angle, msg.area))
    red_sub = rospy.Subscriber('/vision_node/green', PositionArea,
                               lambda msg: obstacles.red.update(msg.x, msg.y, msg.position, msg.angle, msg.area))

    track_pub = rospy.Publisher('/track', Track, queue_size=0)

    # Initializing modules
    control_module = ControlModule(event, robot_size)

    # control_module.head().change_head_scan(min_pan=-50.0, max_pan=50.0, min_tilt=-50, max_tilt=20)
    # sleep(0.5)

    obj_tracker = Tracker(event, robot_size, control_module)
    obj_follower = Follower(event, robot_size, control_module)

    # Enabling walking module
    sleep(10)
    control_module.action().play_action('walk_ready')
    if robot_size == 'adult':
        robot_mode = RobotMode.RUNNING
        mode_change = True
        raw_input('tab Enter to start')
    elif robot_size == 'kid':
        rospy.loginfo('Press start button to begin.')

    while not rospy.is_shutdown():
        if mode_change:
            mode_change = False

            if robot_mode == RobotMode.READY:
                rospy.loginfo('Resetting OP3. Press start button to begin.')

                control_module.walking().stop()
                control_module.head().move_head(pan_position=0, tilt_position=-0.8)
                #begin_sprint()
            elif robot_mode == RobotMode.RUNNING:
                rospy.loginfo('Starting sprint event.')
                begin_sprint()

        rate.sleep()



