#!/usr/bin/env python

from enum import Enum
from time import sleep

import rosnode
import rospy
from std_msgs.msg import String

from control_modules.control_module import ControlModule
from line import Line
from vision.msg import PositionArea

import roslib.packages as rospkg
import os
import json


class RobotMode(Enum):
    READY = 0
    RUNNING = 1


def button_callack(msg):
    global robot_mode, mode_change

    if msg.data == 'start':
        # Start button pressed
        # Send robot to RUNNING mode
        if robot_mode == RobotMode.READY:
            robot_mode = RobotMode.RUNNING
            mode_change = True
        elif robot_mode == RobotMode.RUNNING:
            rospy.logerr('Robot already in running mode.')

    elif msg.data == 'mode':
        # Mode button pressed
        # Send robot to READY mode
        robot_mode = RobotMode.READY
        mode_change = True


def wait_for_node(node_name):
    # Wait for node to start
    while node_name not in rosnode.get_node_names():
        rospy.logwarn('Node is not running: {0}'.format(node_name))
        sleep(1)

    rospy.loginfo('Node is running: {0}'.format(node_name))

     
def do_jump():
    global mode_change, control_module, line, rate

    stage = 0

    # Go to ready position
    control_module.action().play_action('walk_ready')
    sleep(1)
    control_module.head().move_head(pan_position=0, tilt_position=head_tilt)
    sleep(1)

    while not rospy.is_shutdown() and not mode_change:
        if stage == 0:
            if blind_jump:
                # Take steps until reaching line
                control_module.walking().forward()
                sleep(time_to_line)
                control_module.walking().stop()
                stage = 2
            else:
                print('line.y',line.y)
                if line.y < max_marker_y:
                    control_module.walking().forward()
                    sleep(0.5)
                else:
                    if robot_size == 'kid':
                        control_module.walking().forward()
                        sleep(0.5)
                    control_module.walking().stop()
                    stage = 2
                
        elif stage == 2:
            # Jump
            sleep(1)
            control_module.action().play_action('jump')
            sleep(1)
            control_module.action().play_action('jump')
            sleep(1)
            control_module.action().play_action('jump')
            sleep(1)
            stage = 3

        rate.sleep()


def set_variables():
    global time_to_line, max_marker_size, head_tilt, blind_jump, max_marker_y
    config_variables_file = os.path.join(rospkg.get_pkg_dir('events'), 'config', event, 'variables_{0}.json'.format(robot_size))

    vars = None
    with open(config_variables_file, 'r') as fp:
            vars = json.load(fp)
    
    time_to_line = vars['time_to_line']
    max_marker_size = vars['max_marker_size']
    head_tilt = vars['head_tilt']
    blind_jump = vars['blind_jump']
    max_marker_y = vars['max_marker_y']


if __name__ == '__main__':

    # Initalize global variables
    robot_mode = RobotMode.READY
    mode_change = False
    SPIN_RATE = 0.5
    event = 'triple_jump'
    line = Line()

    robot_size = rospy.get_param('triple_jump_node/size')

    set_variables() # Important !

    # Wait for other nodes to launch
    wait_for_node('/op3_manager')
    # wait_for_node('/vision_node')

    # Initialze ROS node
    rospy.init_node('triple_jump_node')
    rate = rospy.Rate(SPIN_RATE)

    # Initialize OP3 ROS generic subscriber
    button_sub = rospy.Subscriber('/robotis/open_cr/button', String, button_callack)

    red_sub = rospy.Subscriber('/vision_node/line', PositionArea,
                               lambda msg: line.update(msg.x, msg.y, msg.position, msg.angle, msg.area))

    # Initializing modules
    control_module = ControlModule(event, robot_size)

    # Enabling walking module
    sleep(10)
    if robot_size == 'adult':
        robot_mode = RobotMode.RUNNING
        mode_change = True
        control_module.head().move_head(pan_position=0, tilt_position=head_tilt)
        raw_input('Tab Enter to start')
    elif robot_size == 'kid':
        rospy.loginfo('Press start button to begin.')

    while not rospy.is_shutdown():
        if mode_change:
            mode_change = False

            if robot_mode == RobotMode.READY:
                rospy.loginfo('Resetting OP3. Press start button to begin.')

                control_module.walking().stop()
                control_module.head().move_head(pan_position=0, tilt_position=-0.8)
            
            elif robot_mode == RobotMode.RUNNING:
                rospy.loginfo('Starting triple jump event.')

                do_jump()

        rate.sleep()


