#!/usr/bin/env python
from control_modules.control_module import AdvancedControlModule
from enum import Enum
from time import sleep

import rosnode
import rospy
from std_msgs.msg import String

import os


import roslib.packages as rospkg


class RobotMode(Enum):
    READY = 0
    RUNNING = 1


def button_callack(msg):
    global robot_mode, mode_change

    if msg.data == 'start':
        # Start button pressed
        # Send robot to RUNNING mode
        if robot_mode == RobotMode.READY:
            robot_mode = RobotMode.RUNNING
            mode_change = True
        elif robot_mode == RobotMode.RUNNING:
            rospy.logerr('Robot already in running mode.')

    elif msg.data == 'mode':
        # Mode button pressed
        # Send robot to READY mode
        robot_mode = RobotMode.READY
        mode_change = True


# def process_walking_call_back(msg):
#     global is_walked
#     # if keyboard.press('w') == 'march':
#     #     control_module.walking().march()
#     if keyboard.press('w'):
#         control_module.walking().forward()
#     elif keyboard.press('s'):
#         control_module.walking().backward()
#     elif keyboard.press('a'):
#         control_module.walking().turn_left()
#     elif keyboard.press('d'):
#         control_module.walking().turn_right()
#     elif keyboard.release('w') or keyboard.release('s') \
#             or keyboard.release('a') or keyboard.release('d'):
#         control_module.walking().stop()
#         is_walked = True

#
# def process_walking_press(msg):
#     if msg == 'march':
#         control_module.walking().march()
#     elif msg == 'forward':
#         control_module.walking().forward()
#     elif msg == 'backward':
#         control_module.walking().backward()
#     elif msg == 'turn_left':
#         control_module.walking().turn_left()
#     elif msg == 'turn_right':
#         control_module.walking().turn_right()
#
#     print msg


def process_walking_release(msg):
    global is_walked
    control_module.walking().stop()
    is_walked = True


def wait_for_node(node_name):
    # Wait for node to start
    while node_name not in rosnode.get_node_names():
        rospy.logwarn('Node is not running: {0}'.format(node_name))
        sleep(1)

    rospy.loginfo('Node is running: {0}'.format(node_name))


def calibration_question(is_rotation=False):
    if is_rotation:
        return float(input("how much did I rotate? (degree) (left(+),right(-)) "))
    else:
        return float(input("how far did I go in shift? (cm) (forward(+),backwrad(-)"))


def calibration(mode):
    control_module.action().play_action('walk_ready')
    if mode == "forward":
        control_module.walking().forward()

    elif mode == "backward":
        control_module.walking().backward()
    elif mode == "shift_left":
        control_module.walking().shift_left()
    elif mode == "shift_right":
        control_module.walking().shift_right()
    elif mode == "turn_left":
        control_module.walking().turn_left()
    elif mode == "turn_right":
        control_module.walking().turn_right()
    raw_input('tab ENTER to stop')
    control_module.walking().stop()



def calibration_launcher():
    print "calibration launched"

    calibration_type = control_module.calibration_factors.keys()

    for i in range (len(calibration_type)):
        print "%d.\t%s" %(i,calibration_type[i])

    selected_type = int(raw_input())


    control_module.calibration_prepare()
    if 'turn' not in calibration_type[selected_type]:
        calibration(calibration_type[selected_type])
        x = calibration_question()
        control_module.calibration_walking(x, calibration_type[selected_type])

    else:
        calibration(calibration_type[selected_type])
        rotation_angle = calibration_question(True)
        control_module.calibration_walking(rotation_angle, calibration_type[selected_type])



def test():

    control_module.action().play_action('walk_ready')
    calibration_type = control_module.calibration_factors.keys()

    for i in range(len(calibration_type)):
        print "%d.\t%s" % (i, calibration_type[i])

    selected_type = int(raw_input())
    calibration(calibration_type[selected_type])



if __name__ == '__main__':

    # Initalize global variables

    robot_mode = RobotMode.READY
    mode_change = False
    SPIN_RATE = 0.5

    current_file_path = os.path.join(rospkg.get_pkg_dir('events'), 'src')

    files = [f for f in sorted(os.listdir(current_file_path))]

    for i in range(len(files)):
        print ('%d\t%s' % (i, files[i]))

    event = files[int(input('please select an event No: '))]
    print 'current event: ' +event
    # Wait for other nodes to launch
    wait_for_node('/op3_manager')
    print "op3 manager started "

    print "start walking calibration..."
    # Initialze ROS node
    rospy.init_node('walking_calibration_node')
    rate = rospy.Rate(SPIN_RATE)
    is_walked = False
    robot_size = rospy.get_param('walking_calibration_node/size')
    print 'current event: ' + event + ' with size: '+robot_size
    # Initialize OP3 ROS generic subscriber
    button_sub = rospy.Subscriber('/robotis/open_cr/button', String, button_callack)



    # Initializing modules
    control_module = AdvancedControlModule(event, robot_size)

    sleep(10)
    rospy.loginfo('start')

    while not rospy.is_shutdown():

        type = int(input("""
        0: walk
        1: calibration
        2: tuning
        """))
        print()
        if type == 0:
            is_done = False
            control_module.action().play_action('walk_ready')
            while not is_done:
                command = raw_input("command: ").strip().split()

                if command[0] == "forward" and len(command) == 3:
                    if command[1] == "step":
                        control_module.walking().forward(step=float(command[2]))

                    else:
                        control_module.walking().forward(distance=float(command[2]))

                elif command[0] == "backward" and len(command) == 3:
                    if command[1] == "step":
                        control_module.walking().backward(step=float(command[2]))
                    else:
                        control_module.walking().backward(distance=float(command[2]))

                if command[0] == "shift_left" and len(command) == 3:
                    if command[1] == "step":
                        control_module.walking().shift_left(step=float(command[2]))

                    else:
                        control_module.walking().shift_left(distance=float(command[2]))

                elif command[0] == "shift_right" and len(command) == 3:
                    if command[1] == "step":
                        control_module.walking().shift_right(step=float(command[2]))
                    else:
                        control_module.walking().shift_right(distance=float(command[2]))



                elif command[0] == "turn_left":
                    control_module.walking().turn_left(angle=float(command[1]))
                elif command[0] == "turn_right":
                    control_module.walking().turn_right(angle=float(command[1]))
                elif command[0] == "done":
                    is_done = True
                elif len(command) == 0:
                    break
        elif type == 1:

            calibration_launcher()
        elif type == 2:
            test()
        rate.sleep()
