#!/usr/bin/env python

from enum import Enum
from time import sleep

import rosnode
import rospy
import roslib.packages as rospkg
import os
import json
from std_msgs.msg import String
from weight_objs import WeightObjects
from tools.follower import Follower
from tools.tracker import Tracker

from control_modules.control_module import ControlModule
from vision.msg import PositionArea


class RobotMode(Enum):
    READY = 0
    RUNNING = 1


class Movements(Enum):
    MARCH = 0
    FORWARD = 1
    BACKWARD = 2
    TURN_LEFT = 3
    TURN_RIGHT = 4
    ACTION = 5

class Weight(Enum):
    FIFTY = 0
    SIXTY = 1
    SEVENTY = 2
    EIGHTY = 3

current_weight = Weight.FIFTY

def button_callback(msg):
    global robot_mode, mode_change

    if msg.data == 'start':
        # Start button pressed
        # Send robot to RUNNING mode
        if robot_mode == RobotMode.READY:
            robot_mode = RobotMode.RUNNING
            mode_change = True
        elif robot_mode == RobotMode.RUNNING:
            rospy.logerr('Robot already in running mode.')

    elif msg.data == 'mode':
        # Mode button pressed
        # Send robot to READY mode
        change_weight(current_weight)
        rospy.logwarn(current_weight.value)
        robot_mode = RobotMode.READY
        mode_change = True


def wait_for_node(node_name):
    # Wait for node to start
    while node_name not in rosnode.get_node_names():
        rospy.logwarn('Node is not running: {0}'.format(node_name))
        sleep(1)

    rospy.loginfo('Node is running: {0}'.format(node_name))


def start_weight_lifting():
    global mode_change, control_module, rate, start_following, stage, obj_tracker, obj_follower

    count_missed = 0

    if robot_size =='adult':
        control_module.action().play_action('walk_ready')
        control_module.head().move_head(pan_position=0, tilt_position=-0.9)

    while not rospy.is_shutdown() and not mode_change:
        tracking_status = obj_tracker.process_tracking()
        
        obj_tracker.provide_marker(obstacles.red.x, obstacles.red.y, obstacles.red.area)
        
        if stage == 0: # Look for bar
            if start_following:
                obj_tracker.start_tracking()
                obj_follower.start_following()
                start_following = False

            # if obstacles.red.area > min_bar_size:
            average_area = obstacles.red.get_average_area()
            print "average area:" + str (average_area)
            if average_area > min_bar_size:
                # if obstacles.red.area > max_bar_size and obstacles.red.y > follower_y:
                # if obstacles.red.area > max_bar_size:
                if average_area > max_bar_size:
                    stage = 1
                    control_module.walking().walk(gait='initial')
                    print('STAGE ', stage)
                else:
                    obj_follower.process_following(obj_tracker.get_pan(), obj_tracker.get_tilt(), 0, auto= False)
            else:
                obj_follower.wait_following()
        elif stage == 1: # Ready to center when arrived to bar
            obj_tracker.stop_tracking()
            obj_follower.stop_following()
            sleep(0.5)
            control_module.head().move_head(pan_position=0, tilt_position=tilt_head)
            stage = 2
            print('STAGE ', stage)
        elif stage == 2: # center robot to center of bar
            print(obstacles.red.x, obstacles.red.y, obstacles.red.angle)

            angle_left = 90 - min_angle
            angle_right = 90 + min_angle
            average_y = obstacles.red.get_average_y()
            angle_ok = obstacles.red.angle > angle_left and obstacles.red.angle < angle_right

            if robot_size == 'adult':
                print "average y: "+ str(average_y)

                if average_y< 0:
                    control_module.walking().backward()

                elif obstacles.red.x < min_bar_x and angle_ok:
                    control_module.walking().walk(gait='shift_left')
                elif obstacles.red.x > max_bar_x and angle_ok:
                    control_module.walking().walk(gait='shift_right')
                elif obstacles.red.angle > angle_right:
                    control_module.walking().walk(gait='turn_right')
                elif obstacles.red.angle < angle_left:
                    control_module.walking().walk(gait='turn_left')
                elif average_y < pickup_y:
                    control_module.walking().walk(gait='initial')
            else:
                if obstacles.red.y < 0:
                    count_missed += 1
                    if count_missed > 5:
                        control_module.walking().backward()
                        count_missed = 0
                        sleep(0.5)
                else:
                    if obstacles.red.y < pickup_y:
                        control_module.walking().walk(gait='initial')
                    elif obstacles.red.x < min_bar_x and angle_ok:
                        control_module.walking().walk(gait='shift_left')
                    elif obstacles.red.x > max_bar_x and angle_ok:
                        control_module.walking().walk(gait='shift_right')
                    elif obstacles.red.angle > angle_right:
                        control_module.walking().walk(gait='turn_right')
                    elif obstacles.red.angle < angle_left:
                        control_module.walking().walk(gait='turn_left')
                    count_missed = 0
                
                    sleep(1)
            if obstacles.red.y > pickup_y and obstacles.red.x > min_bar_x and obstacles.red.x < max_bar_x and angle_ok:
                stage = 3
                print('STAGE ', stage)
        elif stage == 3: # do all the rest of event
            control_module.walking().stop()
            print(obstacles.red.x, obstacles.red.y, obstacles.red.angle)

            sleep(3)
            control_module.action().play_action('pick_up'+weight_flag)
            sleep(3)

            control_module.walking(gait='weight_down').walk(gait='weight_down')
            sleep(time_weight_down)
            control_module.action().play_action('above_head'+weight_flag)
            sleep(3)
            control_module.walking(gait='weight_up').walk(gait='weight_up')
            sleep(time_above_head)
            stage = 4


def change_weight(weight):
    if weight.value < 3:
        weight.value += 1
    else:
        weight = Weight.FIFTY


def set_variables():
    global weight_flag, min_bar_size, max_bar_size, follower_y, pickup_y, min_bar_x, max_bar_x, min_angle, time_weight_down, time_above_head, tilt_head
    config_variables_file = os.path.join(rospkg.get_pkg_dir('events'), 'config', event, 'variables_{0}.json'.format(robot_size))

    vars = None
    with open(config_variables_file, 'r') as fp:
            vars = json.load(fp)

    weight_flag = '_'+str(vars['weight_flag'])
    min_bar_size = vars['min_bar_size']
    max_bar_size = vars['max_bar_size']
    follower_y = vars['follower_y']

    pickup_y = vars['pickup_y']
    min_bar_x = vars['min_bar_x']
    max_bar_x = vars['max_bar_x']
    min_angle = vars['min_angle']

    time_weight_down = vars['time_weight_down']
    time_above_head = vars['time_above_head']

    tilt_head = vars['tilt_head']



if __name__ == '__main__':

    # Initalize global variables
    robot_mode = RobotMode.READY
    mode_change = False
    obstacles = WeightObjects()
    SPIN_RATE = 0.5
    event = 'weight_lifting'
    stage = 0

    start_following = True

    robot_size = rospy.get_param('weight_lifting_node/size')

    set_variables() # Important !

    # Wait for other nodes to launch
    wait_for_node('/op3_manager')
    wait_for_node('/vision_node')

    if robot_size == 'adult':
        stage = 0

    # Initialze ROS node
    rospy.init_node('weight_lifting_node')
    rate = rospy.Rate(SPIN_RATE)

    # Initialize OP3 ROS generic subscriber
    button_sub = rospy.Subscriber('/robotis/open_cr/button', String, button_callback)

    # Initialze vision subscribers
    blue_sub = rospy.Subscriber('/vision_node/blue', PositionArea,
                                lambda msg: obstacles.blue.update(msg.x, msg.y, msg.position, msg.angle, msg.area))
    red_sub = rospy.Subscriber('/vision_node/red', PositionArea,
                               lambda msg: obstacles.red.update(msg.x, msg.y, msg.position, msg.angle, msg.area))
    yellow_sub = rospy.Subscriber('/vision_node/yellow', PositionArea,
                                        lambda msg: obstacles.yellow.update(msg.x, msg.y, msg.position, msg.angle, msg.area))

    # Initializing modules
    control_module = ControlModule(event, robot_size)

    obj_tracker = Tracker(event, robot_size, control_module)
    obj_follower = Follower(event, robot_size, control_module)

    # Enabling walking module
    sleep(10)

    if robot_size == 'adult':
        control_module.walking().stop()
        control_module.head().move_head(pan_position=0, tilt_position=-0.8)
        # raw_input('Tab Enter to start')

        control_module.head().move_head(pan_position=0, tilt_position=-1.2)
        raw_input('Tab Enter to start')
        sleep(3)
        robot_mode = RobotMode.RUNNING
        mode_change = True
    elif robot_size == 'kid':
        control_module.action().play_action('walk_ready')
        control_module.head().move_head(pan_position=0, tilt_position=tilt_head)
        rospy.loginfo('Press start button to begin.')

    while not rospy.is_shutdown():
        if mode_change:
            mode_change = False

            if robot_mode == RobotMode.READY:
                rospy.loginfo('Resetting OP3. Press start button to begin.')

                control_module.walking().stop()
                control_module.head().move_head(pan_position=0, tilt_position=-0.8)
            
            elif robot_mode == RobotMode.RUNNING:
                rospy.loginfo('Starting weightlifting event.')

                start_weight_lifting()

        rate.sleep()



