import cv2 as cv
import copy
import numpy as np
# author: Borui Li

class Frame(object):
    def __init__(self, frame, width=-1, height=-1, border=0):
        self.frame = frame
        if width == -1 or height == -1:
            self.height, self.width = frame.shape[:2]

        else:
            self.width = width
            self.height = height
            self.frame = self.resize_frame(self.width, self.height)

        self.frame = cv.copyMakeBorder(self.frame, borderType=cv.BORDER_CONSTANT, value=[0, 0, 0],
                                       top=border, bottom=border, left=border, right=border)

        self.counter_line = None
        self.box_list = []

    def resize_frame(self, new_width, new_height):
        return cv.resize(self.frame, (new_width, new_height))

    def clone(self):
        return copy.deepcopy(self)

    def threshold(self, min, max):
        _, self.frame = cv.threshold(self.frame, min, max, cv.THRESH_BINARY)

    def convert_hsv(self):
        self.frame = cv.cvtColor(self.frame, cv.COLOR_BGR2HSV)

    def convert_gray(self):
        self.frame = cv.cvtColor(self.frame, cv.COLOR_BGR2GRAY)

    def convert_rgb(self):
        self.frame = cv.cvtColor(self.frame, cv.COLOR_GRAY2BGR)

    def cut_frame(self, x, y, width, height):
        self.frame = self.frame[y:y + height, x:x + width]
        self.height, self.width = self.frame.shape[:2]

    def is_hit(self):
        return self.frame[self.height / 2, self.width / 2] != 0

    def morphology(self, size_e,size_d):
        if size_e > 0:
            element = cv.getStructuringElement(cv.MORPH_RECT, (size_e, size_e))
            self.frame = cv.morphologyEx(self.frame, cv.MORPH_OPEN, element)
        if size_d>0:
            element = cv.getStructuringElement(cv.MORPH_RECT, (size_d, size_d))
            self.frame = cv.morphologyEx(self.frame, cv.MORPH_CLOSE, element)

    def get_counter_line(self):
        _, contours, hierarchy = cv.findContours(
            self.frame, mode=cv.RETR_EXTERNAL, method=cv.CHAIN_APPROX_SIMPLE)
        self.counter_line = contours
        return contours

    def object_detection(self):
        self.get_counter_line()
        self.box_list = []
        for i in self.counter_line:
            rotrect = cv.minAreaRect(i)
            box = cv.boxPoints(rotrect)
            if rotrect[2]<-85 or rotrect[2]>-5:
                self.box_list.append(np.int0(box))

    def cacl_box_area(self, box):
        return abs((box[0, 0] - box[1, 0]) * (box[1, 1] - box[2, 1]))

    def draw_box(self, output_frame):
        for i in self.box_list:
            cv.drawContours(output_frame, [i], 0, (0, 0, 255), 2)

    def is_hit_object(self):
        for i in self.box_list:
            if cv.pointPolygonTest(i,(self.height / 2, self.width / 2),False) == 1:
                return i
        else:
            return None
    def object_height(self, box):
        temp = np.asarray(box)
        length_1 = np.linalg.norm(temp[:1,]- temp[1:2,])
        length_2 = np.linalg.norm(temp[1:2,]-temp[2:3,])
        return min(length_1, length_2)


        # return np.max(temp[:,0]) - np.min(temp[:,0])

