# author: Borui Li
import cv2 as cv
import json

class gui:
    def __init__(self, config, config_path):
        self.init_x = 300
        self.init_y = 100
        self.output_window_name = 'output'
        self.debug_window_name = 'debug'
        self.default_hsv_min_value  = [0,10,10]
        self.default_track_bar_name =['H','S','V']
        self.default_hsv_max_value = [255,255,255]
        self.new_window(self.output_window_name, self.init_x,self.init_y)
        self.config_path = config_path
        self.config = config

    def new_window(self,new_window_name, init_x, init_y):
        cv.namedWindow(new_window_name)
        cv.moveWindow(new_window_name,init_x,init_y)

    def is_window_exist(self, window_name):
        return cv.getWindowProperty(window_name, 0) != -1

    def create_track_bar(self, window_name, track_bar_name, min, max, window_callback):
        if not self.is_window_exist(window_name):
            cv.namedWindow(window_name)
        cv.createTrackbar(track_bar_name,window_name,min,max,window_callback)

    def set_tackbar_pos(self, window_name, track_bar_name, new_pos):
        cv.setTrackbarPos(track_bar_name,window_name,new_pos)

    def get_track_bar_pos(self,window_name, track_bar_name):
        cv.getTrackbarPos(track_bar_name,window_name)

    def set_mouse_call_back(self,window_name, mouse_callback):
        if not self.is_window_exist(window_name):
            cv.namedWindow(window_name)
        cv.setMouseCallback(window_name,mouse_callback)


    def show(self, window_name, mat):
        if not self.is_window_exist(window_name):
            cv.namedWindow(window_name)
        cv.imshow(window_name,mat)

    def show_debug(self,window_name, debug_name, messages, frame):
        if not self.is_window_exist(window_name):
            cv.namedWindow(window_name)
            cv.moveWindow(window_name, self.init_x+700, self.init_y+300)

        cv.putText(frame, debug_name,org= (10,30), fontFace=cv.FONT_HERSHEY_DUPLEX, fontScale=1, color=(0, 0, 255))
        x = 15
        y = 40
        for i in messages:
            cv.putText(frame,i,org= (x,y), fontFace=cv.FONT_HERSHEY_DUPLEX, fontScale=1, color=(0, 0, 255))
            y += 10

        cv.imshow(window_name,frame)

    def create_hsv_track_bar_set(self, hsv_call_back):
        for hsv_min,hsv_max, name in zip(self.default_hsv_min_value,self.default_hsv_max_value,self.default_track_bar_name):
            self.create_track_bar(self.debug_window_name,name,hsv_min,hsv_max,lambda value: hsv_call_back(name,value))


    def save_json(self, config):
        with open(self.config_path, 'w') as fp:
            c = json.dumps(
                config, sort_keys=True, indent=2)
            c = c.replace('"##<', '').replace('>##"', '')
            fp.write(c)

