from frame import Frame
from vedio_control import Video
import cv2 as cv
import numpy as np
# author: Borui Li
class processing_frame:
    def __init__(self, config, name, video_control):
        self.frame = None
        self.config = config
        self.name = name
        self.video_control = video_control
        # self.current_status = '_ _ _ _'
        self.current_status = [False,False,False,False,False]
        self.prev_status = [False,False,False,False,False]
        self.object_height_count = []
        self.start_detect = True
        self.froze = False


    def send_new_frame(self, new_frame):
        self.frame = new_frame

    def cut_frame(self):

        frame_list = []
        single_length = self.config['markers'][self.name]['selected_region']['width'] / 5
        single_height = self.config['markers'][self.name]['selected_region']['height']
        start_x = self.config['markers'][self.name]['selected_region']['x']
        start_y = self.config['markers'][self.name]['selected_region']['y']
        selected_region = self.frame.clone()
        selected_region.cut_frame(start_x, start_y, single_length*5, single_height)

        # self.video_control.opencv_gui.show_debug(self.video_control.opencv_gui.debug_window_name,
        #                                          '', '', selected_region.frame)
        for i in range(0, 5):
            temp = selected_region.clone()
            temp.cut_frame(single_length * i, 0, single_length, single_height)
            frame_list.append(temp)

            if i != 0:
                self.video_control.draw_line(self.video_control.input_frame, (start_x + single_length * i, start_y),
                                             (start_x + single_length * i, start_y + single_height), (255, 0, 0))
        return frame_list,selected_region

    def draw_gui(self):

        single_height = self.config['markers'][self.name]['selected_region']['height']
        start_x = self.config['markers'][self.name]['selected_region']['x']
        start_y = self.config['markers'][self.name]['selected_region']['y']
        self.video_control.draw_rect(self.video_control.input_frame, (start_x, start_y), (
            start_x+self.config['markers'][self.name]['selected_region']['width'], start_y+single_height), (255, 0, 0))

        self.video_control.draw_line(self.video_control.input_frame, (start_x, start_y + single_height / 2), (
            start_x + self.config['markers'][self.name]['selected_region']['width'], start_y + single_height / 2),
                                     (255, 0, 0))


    def pre_process(self):
        self.frame.convert_gray()
        self.frame.threshold(self.config['markers'][self.name]['bright'], 255)

        self.frame.morphology(self.config['markers'][self.name]['erode'],self.config['markers'][self.name]['dilate'])
        # self.video_control.opencv_gui.show('gray', self.frame.frame)


    def show_debug(self):
        self.video_control.opencv_gui.show_debug(self.video_control.opencv_gui.debug_window_name,
                                                 self.name, '', self.frame.frame)
    def gather_output(self):
        pass

    def process_counter_line_detect(self, processing_list):
        output_mat_list = []
        for i in processing_list:
            # counter_line = i.get_counter_line()
            i.object_detection()
            output_frame = cv.cvtColor(i.frame, cv.COLOR_GRAY2BGR)
            # for i in counter_line:
            #     rotrect = cv.minAreaRect(i)
            #     box = cv.boxPoints(rotrect)
            #     box = np.int0(box)
            i.draw_box(output_frame)
            cv.circle(output_frame, (i.width / 2, i.height / 2), 1, (0, 255, 255),
                      thickness=1, lineType=8, shift=0)
                # cv.drawContours(output_frame, [box], 0, (0, 0, 255), 2)
                # cv.drawContours(output_frame, i, -1, (0, 255, 0), 3)
            output_mat_list.append(output_frame)
        return np.concatenate(output_mat_list,axis=1)

    def object_count_average(self):
        return sum(self.object_height_count)/self.object_height_count.__len__()

    # assumption, pattle will never appear at beginning

    def determine_object_type(self, obj_height, j):

        if self.object_height_count.__len__()==0:
            # print "added"
            self.object_height_count.append(obj_height)
            return 'drum'
        else:
            average = self.object_count_average()
            # if not self.froze:

                # print ('{0}: {1}: {3}: {4} =>{2}'.format(j,obj_height, abs(obj_height-average)/average, average,self.object_height_count.__len__()))
            if abs(obj_height-average)/average>0.2:
                return 'paddle'
            else:
                if  self.prev_status[j] != True:
                    # print "added"
                    self.object_height_count.append(obj_height)
                return 'drum'


    def processing_hit(self):

        if self.config['markers'][self.name]['selected_region']['width'] == -1 or not self.start_detect:
            return
        else:
            self.draw_gui()
            self.prev_status = self.current_status
            self.current_status = [False,False,False,False,False]
            processing_list,selected_region = self.cut_frame()
            counter_frame = self.process_counter_line_detect(processing_list)
            pattle = 0
            j = 0
            for i in processing_list:
                test = i.is_hit_object()

                if test is not None:
                    result = self.determine_object_type(i.object_height(test),j)

                    if result == 'drum':
                        self.current_status[j] = True
                    elif result == 'paddle':
                        self.current_status [j]= False
                        pattle += 1
                        pass
                    else:
                        self.current_status [j]= False
                else:
                    self.current_status [j] = False
                j+=1
            # if pattle >1:
            #     self.current_status [4] = True


            selected_region_frame = cv.cvtColor(selected_region.frame,cv.COLOR_GRAY2BGR)
            debug_frame = np.concatenate((selected_region_frame, counter_frame))

            cv.line(debug_frame, (0, selected_region.height), (
                selected_region.width,selected_region.height),
                                         (255, 0, 0))

            cv.line(debug_frame, (0, int(selected_region.height*1.5)), (
                selected_region.width,int(selected_region.height*1.5)),
                                         (255, 0, 0))
            self.video_control.opencv_gui.show_debug(self.video_control.opencv_gui.debug_window_name,
                                                     '', '', debug_frame)

        output_string = ''
        for i in range(0,5):
            if self.current_status[i]:
                output_string+='X '
            else:
                output_string+='_ '
        # if self.current_status[4]:
        #     output_string+='#'
        cv.putText(self.video_control.input_frame.frame, output_string, org=(10, 30),
                   fontFace=cv.FONT_HERSHEY_DUPLEX, fontScale=1, color=(0, 0, 255))

        # if not self.froze:
            # print (output_string)