#!/usr/bin/env python
from time import sleep, time
import roslib.packages as rospkg
import rosnode
import rospy
import os
import json
from control_modules.control_module import ControlModule
from vedio_control import Video
from processing_frame import processing_frame
from gui_control import gui_control as marker
import cv2 as cv
import robot_drummer
from iros_vision.msg import ObjectCoords
from iros_vision.msg import DrumAction
import std_msgs
# author: Borui Li


def drum_ready_cb(ready):
    global READY
    print("cb for ready !")
    READY = True


def wait_for_node(node_name):
    # Wait for node to start
    while node_name not in rosnode.get_node_names():
        rospy.logwarn('Node is not running: {0}'.format(node_name))
        sleep(1)
    rospy.loginfo('Node is running: {0}'.format(node_name))

#publish a msg with new drum motions
def publish_drums(publisher, state):
    msg = DrumAction()
    if len(state) == 5:
        msg.ride = state[0]
        msg.ignore = state[1]
        msg.snare = state[2]
        msg.hihat = state[3]
        msg.kick = state[4]
        publisher.publish(msg)




if __name__ == '__main__':
    READY = False
    SPIN_RATE = 10
    rospy.init_node('drums_vision')
    rate = rospy.Rate(SPIN_RATE)

    drums_vision_pub = rospy.Publisher(
        '/iros_drums/drums', DrumAction, queue_size=1)
    drums_ready_subs = rospy.Subscriber(
        '/iros_drums/ready', std_msgs.msg.Bool, drum_ready_cb)
   

    control_module = None

    # configuration path

    configuration_directory = rospy.get_param(
        'drums_vision/configuration_directory')

    camera_input = rospy.get_param('drums_vision/camera_input')

    file_path = os.path.join(configuration_directory, 'rockband_drums.json')
    config = None

    with open(file_path, 'r') as fp:
        config = json.load(fp)


    # set vision input for node

    video_path = None
    if camera_input:  # check if video input is from camera or from video played in memory
        video_path = 0
    else:
        video_path = os.path.join(rospkg.get_pkg_dir(
            'iros_vision'), 'static', '003.mp4')

    # initialze vision player

    vedio_control = Video(file_path, video_path)

    vedio_control.initial()
    processing_video = processing_frame(
        config, config['markers'].keys()[0], vedio_control)
    wait_time = 27
    counter = 0  # used to skip frames to avoid errors


    count_frame = 1
    SLEEP_BETWEEN_FRAMES = 17 #25 CONTROLS SPEED OF VIDEO!
    while not rospy.is_shutdown():
        if READY is True:

            if vedio_control.next_frame():
                processing_video.config = vedio_control.opencv_gui.config
                processing_video.send_new_frame(vedio_control.output_frame)
                processing_video.pre_process()
                processing_video.processing_hit()
                key_board_event = cv.waitKey(25)
                # only get the state every 3 frames
                if counter % 3 == 0:
                    state = processing_video.current_status  # get the current state
                    publish_drums(drums_vision_pub, state)
                counter += 1

                # processing_video.show_debug()
                # vedio_control.opencv_gui.show(vedio_control.opencv_gui.output_window_name,vedio_control.input_frame.frame)
                vedio_control.show_output()
                #key_board_event = cv.waitKey(11)
                if key_board_event != -1:
                    # print key_board_event
                    if key_board_event == ord('p'):
                        vedio_control.froze_frame()
                        processing_video.froze = not processing_video.froze
                    elif key_board_event == ord('s'):
                        processing_video.start_detect = not processing_video.start_detect
                    elif key_board_event == ord('f'):  # make video faster
                        if wait_time > 14:
                            wait_time -= 10
                            print('change to {0} fps'.format(1000/wait_time))
                        else:
                            print('reach maximum speed')

                    elif key_board_event == ord('g'):  # make video slower
                        wait_time += 10
                        print('change to {0} fps'.format(1000 / wait_time))

            else:
                break
    
