import numpy as np


class BaseObject(object):

    def __init__(self, configuration, module):
        self.module = module

        self.name = configuration['name']
        self.threshold = np.array(configuration['colourspace_threshold']) 
        self.value = np.array(configuration['colourspace_value']) 
        self.lower_bound, self.upper_bound = self.get_colour_space_bounds()
        self.opening_closing_ksize = configuration['opening_closing_ksize']
        self.blur_ksize = configuration['blur_ksize']
        self.mat = None

    @staticmethod
    def round_int(num):
        return max(0, int(round(num)))

    def get_colour_space_bounds(self):
        threshold_lower_bound = []
        threshold_upper_bound = []

        for index in range(0, len(self.value)):
            threshold_lower_bound += [self.round_int(
                self.value[index] - self.threshold[index])]
            threshold_upper_bound += [self.round_int(
                self.value[index] + self.threshold[index])]

        return np.array(threshold_lower_bound), np.array(threshold_upper_bound)


    def set_colour_space_value(self, values):
        self.value = np.array([self.round_int(value) for value in values])
        self.lower_bound, self.upper_bound = self.get_colour_space_bounds()

    def set_colour_space_threshold(self, threshold):
        self.threshold = np.array([self.round_int(value) if value >= 0 else self.threshold[index]
                                   for index, value in enumerate(threshold)])
        self.lower_bound, self.upper_bound = self.get_colour_space_bounds()

    def set_opening_closing_ksize(self, kernel_size):
        self.opening_closing_ksize = kernel_size

    def set_blur_ksize(self, kernel_size):
        self.blur_ksize = kernel_size

    def export_configuration(self):
        return {
            'colourspace_threshold': self.threshold,
            'colourspace_value': self.value,
            'name': self.name,
            'opening_closing_ksize': self.opening_closing_ksize,
            'blur_ksize': self.blur_ksize
        }