import json
import os
import shutil

import numpy as np

import rospy


class NumpyEncoder(json.JSONEncoder):
    # Encodes numpy arrays as lists since JSON does not support np.ndarray
    def default(self, obj):
        return '##<{0}>##'.format(obj.tolist()) if isinstance(obj, np.ndarray) else json.JSONEncoder.default(self, obj)


class Configuration(object):

    def __init__(self, configuration_directory, event, module, robot_size=''):
        if robot_size == '':
            self.file_path = os.path.join(
                configuration_directory, event, '{0}.json'.format(module))
        else:
            self.file_path = os.path.join(
                configuration_directory, event, '{0}_{1}.json'.format(module, robot_size))
        self.config = self.__load(module)

    def __load(self, module):
        if not os.path.isfile(self.file_path):
            rospy.logerr(
                'Missing vision configuration files for {0} module'.format(module))

        with open(self.file_path, 'r') as fp:
            return json.load(fp)

    def save(self):
        # Save the configurations in a JSON file
        with open(self.file_path, 'w') as fp:
            config = json.dumps(
                self.config, sort_keys=True, indent=2, cls=NumpyEncoder)
            config = config.replace('"##<', '').replace('>##"', '')
            fp.write(config)
