import copy

import cv2 as cv
import numpy as np


class Mat(object):

    BLACK = (0, 0, 0)
    WHITE = (255, 255, 255)

    def __init__(self, frame, width=0, height=0, border_size=0):
        self.frame, self.height, self.width = self.resize(
            frame, width=width, height=height)
        self.frame = self.add_border(border_size=border_size)
        self.original_frame = frame.copy()
        self.colour_space_frame = cv.cvtColor(
            self.frame, code=cv.COLOR_BGR2LAB)

    def clone(self):
        return copy.deepcopy(self)

    @staticmethod
    def resize(frame, width=0, height=0):
        # Resize an image based on the given width OR height
        # Will choose width if both width and height are given

        (_height, _width) = frame.shape[:2]
        ratio = _width / float(_height)

        if width > 0:
            height = int(round(width / ratio))
        elif height > 0:
            width = int(round(ratio * height))
        else:
            width, height = _width, _height

        if width != _width or height != _height:
            frame = cv.resize(frame, (width, height), interpolation=cv.INTER_AREA)

        return frame, height, width

    def add_border(self, border_size):
        # Add a black border around frame
        return cv.copyMakeBorder(self.frame, borderType=cv.BORDER_CONSTANT, value=[0, 0, 0],
                                 top=border_size, bottom=border_size, left=border_size, right=border_size)

    def extract_object(self, thresh_lb, thresh_ub, kernel_size=3, blur_ksize=3):
        # Blur and threshold the original image
        new_blur_ksize = blur_ksize*2+1             #blur_ksize have to be odd number
        blur = cv.GaussianBlur(self.colour_space_frame, ksize=(new_blur_ksize, new_blur_ksize), sigmaX=0)
        self.frame = cv.inRange(blur, lowerb=thresh_lb, upperb=thresh_ub)

        # Remove any noise

        new_kernel_size = kernel_size*2+1      #kernel_ksize have to be odd number
        kernel_ellipse = cv.getStructuringElement(
            cv.MORPH_ELLIPSE, (new_kernel_size, new_kernel_size))
        self.frame = cv.morphologyEx(
            self.frame, op=cv.MORPH_OPEN, kernel=kernel_ellipse)
        self.frame = cv.morphologyEx(
            self.frame, op=cv.MORPH_CLOSE, kernel=kernel_ellipse)
