import cv2 as cv


COLOUR_SPACE_CHANNELS = ('H', 'S', 'V')
COLOUR_SPACE_MAX_VALUES = (255, 255, 255)


DEBUG_WINDOW_NAME = 'debug'
COLOUR_SPACE_THRESHOLD_TRACKBAR_NAME_FORMAT = '{0} threshold'

opening_closing_ksize = 'opening_closing_ksize\n=value*2+1'
opening_closing_ksize_MAX_VALUE = 50
blur_ksize = 'blur_ksize=value*2+1'
blur_ksize_MAX_VALUE = 50

def create_window(name, x=0, y=0):
    # Create a window with name at position (x, y)
    cv.namedWindow(name)
    cv.moveWindow(name, x, y)


def create_trackbar(window_name, trackbar_name, default_value, max_value, callback):
    # Create a trackbar in window
    create_window(window_name)
    cv.createTrackbar(trackbar_name, window_name,
                      default_value, max_value, callback)


def set_trackbar_position(trackbar_name, window_name, position):
    # Set trackbar position
    cv.setTrackbarPos(trackbar_name, window_name, position)


def set_mouse_cb(window_name, callback):
    # Set mouse callback
    create_window(window_name)
    cv.setMouseCallback(window_name, callback)


def show(window_name, frame):
    # Show any window
    cv.imshow(window_name, frame)


def show_debug_window(frame, text):
    # Show the debug window with the selected object name
    frame = cv.cvtColor(frame, code=cv.COLOR_GRAY2BGR)
    cv.putText(frame, text=text, org=(10, 30),
               fontFace=cv.FONT_HERSHEY_DUPLEX, fontScale=1, color=(0, 0, 255))
    cv.imshow(DEBUG_WINDOW_NAME, frame)


def create_colour_space_threshold_trackbar(default_colour_space_threshold, callback):
    create_window(DEBUG_WINDOW_NAME)

    # Create the appropriate colour space threshold trackbars in the object window
    cv.createTrackbar(COLOUR_SPACE_THRESHOLD_TRACKBAR_NAME_FORMAT.format(COLOUR_SPACE_CHANNELS[0]),
                      DEBUG_WINDOW_NAME, default_colour_space_threshold[0], COLOUR_SPACE_MAX_VALUES[0],
                      lambda value: callback([value, -1, -1]))
    cv.createTrackbar(COLOUR_SPACE_THRESHOLD_TRACKBAR_NAME_FORMAT.format(COLOUR_SPACE_CHANNELS[1]),
                      DEBUG_WINDOW_NAME, default_colour_space_threshold[1], COLOUR_SPACE_MAX_VALUES[1],
                      lambda value: callback([-1, value, -1]))
    cv.createTrackbar(COLOUR_SPACE_THRESHOLD_TRACKBAR_NAME_FORMAT.format(COLOUR_SPACE_CHANNELS[2]),
                      DEBUG_WINDOW_NAME, default_colour_space_threshold[2], COLOUR_SPACE_MAX_VALUES[2],
                      lambda value: callback([-1, -1, value]))

def create_opening_closing_ksize_trackbar(default_kernel_size, callback):
    create_window(DEBUG_WINDOW_NAME)

    #create the opening and closing kernel size trackbar in the object window
    cv.createTrackbar(opening_closing_ksize, DEBUG_WINDOW_NAME,default_kernel_size, opening_closing_ksize_MAX_VALUE, callback)

def create_blur_ksize_trackbar(default_kernel_size, callback):
    create_window(DEBUG_WINDOW_NAME)
    # create the blur kernel size trackbar in the object window
    cv.createTrackbar(blur_ksize, DEBUG_WINDOW_NAME, default_kernel_size, blur_ksize_MAX_VALUE,
                      callback)


def set_colour_space_threshold_trackbar_position(colour_space_threshold_values):
    # Modify trackbar positions
    for index, channel in enumerate(COLOUR_SPACE_CHANNELS):
        set_trackbar_position(COLOUR_SPACE_THRESHOLD_TRACKBAR_NAME_FORMAT.format(channel),
                              DEBUG_WINDOW_NAME, colour_space_threshold_values[index])

def set_opening_closing_ksize_trackbar_position(kernel_size):
    set_trackbar_position(opening_closing_ksize, DEBUG_WINDOW_NAME,kernel_size)

def set_blur_ksize_trackbar(kernel_size):
    set_trackbar_position(blur_ksize, DEBUG_WINDOW_NAME, kernel_size)

def teardown():
    cv.destroyAllWindows()

def makeCenterCrossLine(frame):
    width, height = frame.shape[:2]
    ptr1 = (height / 2, 0)
    ptr2 = (height / 2, width)
    cv.line(frame, ptr1, ptr2, (0, 255, 255), 1)
    ptr1 = (0, width / 2)
    ptr2 = (height, width / 2)
    cv.line(frame, ptr1, ptr2, (0, 255, 255), 1)
