#! /usr/bin/env python

import cv2 as cv
import rospy
import numpy as np

from base_module import BaseModule
from configuration import Configuration
from mat import Mat
import opencv_gui as gui

from obstacles.obstacle_module import ObstacleModule
from line_segment_detector.line_segment_detector_module import LineSegmentDetectorModule
from marathon_marker_detector.marker_module import MarkerModule


def capture_hsv(event, x, y, flags, param):
    #print(base_module.original_img.frame[x,y])
    hsv = cv.cvtColor(base_module.original_img.frame, cv.COLOR_BGR2HSV)
    #print(hsv[y,x])
    #cv.imshow('hsv',hsv)
    #cv.namedWindow('color')
    #img = np.zeros((300,512,3), np.uint8)
    #img[:] = base_module.original_img.frame[y,x] #hsv[x,y]
    #hsv_img = cv.cvtColor(img, cv.COLOR_BGR2HSV)
    print(hsv[y,x])
    cv.putText(base_module.original_img.frame, ''.join(str(hsv[y,x])), (x,y), cv.FONT_HERSHEY_SIMPLEX, 2.0, (255,255,255))
    #cv.imshow('color', hsv_img)


if __name__ == '__main__':
    # Initialize ROS
    rospy.init_node('vision_node')
    rate = rospy.Rate(10)

    # Initialize base module
    configuration_directory = rospy.get_param(
        'vision_node/configuration_directory')
    event = rospy.get_param('vision_node/event')
    base_module = BaseModule(
        configuration_directory=configuration_directory, event=event)

    robot_size = rospy.get_param('vision_node/size')

    # Initialize each enabled module
    print(ObstacleModule.is_enabled(base_module))
    if ObstacleModule.is_enabled(base_module):
        obstacle_module = ObstacleModule(
            base_module=base_module, configuration_directory=configuration_directory, event=event, robot_size=robot_size
        )
    if LineSegmentDetectorModule.is_enabled(base_module):
        line_segment_detector_module = LineSegmentDetectorModule(
            base_module=base_module, configuration_directory=configuration_directory, event=event, robot_size=robot_size
        )
    if MarkerModule.is_enabled(base_module):
        marker_module = MarkerModule(
            base_module = base_module, configuration_directory = configuration_directory, event = event, robot_size=robot_size
        )

    base_module.create_trackbars()

    # Start vision processing
    try:
        while not rospy.is_shutdown():
            if base_module.read_from_camera():
                if ObstacleModule.is_enabled(base_module):
                    base_module.output_img = obstacle_module.detect(
                        input_mat=base_module.original_img.clone(), output_mat=base_module.output_img)

                if LineSegmentDetectorModule.is_enabled(base_module):
                    base_module.output_img = line_segment_detector_module.track_lines(
                        input_mat=base_module.original_img.clone(), output_mat=base_module.output_img)

                if MarkerModule.is_enabled(base_module):
                    base_module.output_img = marker_module.findArrow(
                        input_mat=base_module.original_img.clone(), output_mat=base_module.output_img)

                # Display GUI windows
                gui.show_debug_window(
                    frame=base_module.selected_debug.mat.frame, text=base_module.selected_debug.name)
                #cv.setMouseCallback('camera', capture_hsv)

                gui.makeCenterCrossLine(base_module.output_img.frame)
                cv.imshow('camera', base_module.output_img.frame)
                # gui.show('output', base_module.output_img.frame)

            cv.waitKey(1)

    except rospy.ROSInterruptException:
        # Clean up before exit
        gui.teardown()
        base_module.cap.release()