import numpy as np
import rospy

from vision.msg import DetectedLines

from base.base_object import BaseObject


class Field(BaseObject):

    def __init__(self, configuration, module):
        super(Field, self).__init__(configuration=configuration, module=module)

        self.min_area = configuration['min_area']

    def export_configuration(self):
        configuration = super(Field, self).export_configuration()
        configuration.update({
            'min_area': self.min_area
        })
        return configuration


class Lines(BaseObject):

    def __init__(self, configuration, module):
        super(Lines, self).__init__(configuration=configuration, module=module)

        self.max_distance_apart = configuration['max_distance_apart']
        self.min_length = configuration['min_length']
        self.max_width = configuration['max_width']
        self.corner_max_distance_apart = configuration['corner_max_distance_apart']
        self.output_center_colour = configuration["output_center_line_colour"]
        self.output_goal_area_colour = configuration["output_goal_area_line_colour"]
        self.output_boundary_colour = configuration["output_boundary_line_colour"]
        self.output_undefined_colour = configuration["output_undefined_line_colour"]

        self.pub = rospy.Publisher(
            '/vision_node/' + self.name, DetectedLines, queue_size=1)

    def export_configuration(self):
        configuration = super(Lines, self).export_configuration()
        configuration.update({
            'max_distance_apart': self.max_distance_apart,
            'min_length': self.min_length,
            'max_width': self.max_width,
            'corner_max_distance_apart': self.corner_max_distance_apart,
            "output_center_line_colour": np.array(self.output_center_colour),
            "output_goal_area_line_colour": np.array(self.output_goal_area_colour),
            "output_boundary_line_colour": np.array(self.output_boundary_colour),
            "output_undefined_line_colour": np.array(self.output_undefined_colour)
        })
        return configuration

    def publish_msg(self, lines):
        message = DetectedLines()

        message.boundary_line = '|'.join(
            [line_description[1] for line_description in lines['boundary']])
        message.goal_line = '|'.join(
            [line_description[1] for line_description in lines['goal_area']])
        message.center_line = '|'.join(
            [line_description[1] for line_description in lines['center']])
        message.undefined_lines = '|'.join(
            [line_description[1] for line_description in lines['undefined']])

        self.pub.publish(message)
