import random
from bisect import insort
import math
import sys

import cv2 as cv
import numpy as np

from line_segment import LineSegment, Corner, FieldLines
from comparingLines import ComparingLines
from base.mat import Mat


class LineSegmentDetectorMat(Mat):
    __OPENCV_LSD = cv.createLineSegmentDetector(_refine=cv.LSD_REFINE_STD)

    def __init__(self, frame):
        super(LineSegmentDetectorMat, self).__init__(frame=frame)
        self.line_segments = []
        self.horizontal = []
        self.vertical = []
        self.field_lines = []
        self.extra_lines = []
        self.srcFrame = frame.copy()



    def background_mask(self, thresh_lb, thresh_ub, min_area, line_width=40):
        # Blur and threshold the original image

        self.frame = cv.GaussianBlur(
            self.colour_space_frame, ksize=(5, 5), sigmaX=0)
        self.frame = cv.inRange(self.frame, lowerb=thresh_lb, upperb=thresh_ub)

        # Remove any noise
        kernel_ellipse = cv.getStructuringElement(cv.MORPH_ELLIPSE, (7, 7))
        self.frame = cv.morphologyEx(
            self.frame, op=cv.MORPH_OPEN, kernel=kernel_ellipse)
        self.frame = cv.morphologyEx(
            self.frame, op=cv.MORPH_CLOSE, kernel=kernel_ellipse)

        # Find contours that fulfill the min_area requirement
        _, contours, _ = cv.findContours(
            self.frame, mode=cv.RETR_EXTERNAL, method=cv.CHAIN_APPROX_SIMPLE)
        contours = [contour for contour in contours if cv.contourArea(
            contour) > min_area] if contours else None


        self.frame = np.zeros(self.frame.shape[:2], dtype=np.uint8)
        if contours:
            hull = self.__contoursConvexHull(contours)          #using ConvexHull to extract the field
            if hull is not None:
                for i, h in enumerate(hull):
                    cv.polylines(self.frame, [hull], True, (0, 255, 255), 2)
                    self.frame = cv.fillPoly(self.frame, [hull], (255, 255, 255))
                    self.frame = cv.morphologyEx(self.frame, cv.MORPH_ERODE,
                                                      cv.getStructuringElement(cv.MORPH_ELLIPSE, (5, 15)))

                # # Create a blank frame and fill in the contours
        # self.frame = np.zeros(self.frame.shape[:2], dtype=np.uint8)
        # if contours:
        #     cv.fillPoly(self.frame, pts=contours, color=self.WHITE)
        #
        #     # Morph close to fill in the lines
        #     self.frame = cv.morphologyEx(self.frame, cv.MORPH_CLOSE,
        #                                  cv.getStructuringElement(cv.MORPH_RECT, (line_width, line_width)))

    def extract_lines(self, thresh_lb, thresh_ub):
        super(LineSegmentDetectorMat, self).extract_object(thresh_lb=thresh_lb,
                                                           thresh_ub=thresh_ub, kernel_size=1)

    @staticmethod
    def __contoursConvexHull(contours):
        pts = []
        for i in range(0, len(contours)):
            if cv.contourArea(contours[i]) > 500:
                for j in range(0, len(contours[i])):
                    pts.append(contours[i][j])

        pts = np.array(pts)
        result = None
        if len(pts > 2):
            result = cv.convexHull(pts)
        return result

    @staticmethod
    def __filter_by_length(lines, min_length=-1):
        # Return only lines that are longer than min_length
        filtered_lines = []

        if lines is not []:
            for index, line in enumerate(lines):
                if line.length > min_length:
                    filtered_lines.append(line)

        return filtered_lines

    @staticmethod
    def __merge_line_by_distance_and_angle(lines_s, min_dist_to_connect, angle_offset):
        distance_offset = 3
        i = 0
        while i < len(lines_s):
            j = i + 1
            while j < len(lines_s):
                comparingLine = ComparingLines(lines_s[i], lines_s[j])
                pts_dist = comparingLine.end_point_dist()
                angleDifference = comparingLine.angleDifference
                if (angleDifference < angle_offset or angleDifference > 180 - angle_offset) and min(
                        pts_dist) < min_dist_to_connect and max(
                        pts_dist) + distance_offset > comparingLine.get_total_length():
                    lines_s.append(comparingLine.get_line_from_two_farest_points())
                    lines_s.pop(j)
                    lines_s.pop(i)
                    break
                else:
                    j += 1
            else:
                i += 1
        return lines_s

    @staticmethod
    def __merge_parallel_line(lines_s, max_dist_to_connect, angle_offset):
        distance_offset = 10
        i = 0
        while i < len(lines_s):
            j = i + 1
            while j < len(lines_s):
                comparingLine = ComparingLines(lines_s[i], lines_s[j])
                pts_dist = comparingLine.closest_point_to_line_dist()
                angleDifference = comparingLine.angleDifference
                if (angleDifference < angle_offset or angleDifference > 180 - angle_offset) and min(
                        pts_dist) < max_dist_to_connect and max(
                        pts_dist) + distance_offset < comparingLine.get_total_length():
                    lines_s.append(comparingLine.get_line_from_two_closest_end_points())
                    lines_s.pop(j)
                    lines_s.pop(i)
                    break
                else:
                    j += 1
            else:
                i += 1
        return lines_s

    # @staticmethod
    # def __categorize_by_angle(lines):
    #     horizontal = []
    #     vertical = []
    #
    #     if len(lines) > 0:
    #         # Find the average of the min_angle and max_angle, use that to offset the x-axis
    #         max_angle = max(line.angle for line in lines)
    #         min_angle = min(line.angle for line in lines)
    #         min_max_average = (math.fabs(max_angle) +
    #                            math.fabs(min_angle)) / 2.0
    #         x_axis_offset = math.pi / 4 if min_max_average > math.pi / 3 else 0
    #
    #         for line in lines:
    #             angle = line.angle if x_axis_offset == 0 else math.fabs(
    #                 line.angle)
    #             if angle > x_axis_offset:
    #                 vertical += [line]
    #             else:
    #                 horizontal += [line]
    #
    #     return horizontal, vertical
    #
    # @staticmethod
    # def __categorize_by_distance_apart(lines, max_distance_apart, frame_height):
    #     categorize_distance = []
    #
    #     for line in lines:
    #         closest_group_index = None
    #         shortest_distance_apart = sys.maxint
    #
    #         # Find a group for the line based on the shortest distance existing grouped lines
    #         for group_index, group in enumerate(categorize_distance):
    #             for reference_line in group:
    #                 distance_apart = min(reference_line.distance_to_point(pt_x=line.x1, pt_y=line.y1),
    #                                      reference_line.distance_to_point(
    #                                          pt_x=line.x2, pt_y=line.y2),
    #                                      line.distance_to_point(
    #                                          pt_x=reference_line.x1, pt_y=reference_line.y1),
    #                                      line.distance_to_point(pt_x=reference_line.x2, pt_y=reference_line.y2))
    #
    #                 # Calculate a new max_distance_apart based on the average value of y
    #                 # bottom of image --> larger y --> line segments further apart
    #                 average_y = (line.y1 + line.y2 +
    #                              reference_line.y1 + reference_line.y2) / 4.0
    #                 distance_apart_threshold = 1 + average_y / frame_height / 2
    #
    #                 if distance_apart < shortest_distance_apart and \
    #                         distance_apart < max_distance_apart * distance_apart_threshold:
    #                     closest_group_index = group_index
    #                     shortest_distance_apart = distance_apart
    #
    #         # Group two lines together if a grouping was found, otherwise, start a new group
    #         if closest_group_index is not None:
    #             categorize_distance[closest_group_index] += [line]
    #         else:
    #             categorize_distance += [[line]]
    #
    #     return categorize_distance
    #
    # @staticmethod
    # def __merge_grouped_lines__(groups):
    #     merged_lines = []
    #
    #     # Find the average of the lines in the group
    #     # Create a new line based on the average
    #     for group in groups:
    #         if len(group) > 1:
    #             x_coordinates = []
    #             y_coordinates = []
    #             average_angle = 0
    #
    #             for line in group:
    #                 x_coordinates += [line.x1, line.x2]
    #                 y_coordinates += [line.y1, line.y2]
    #                 average_angle += line.angle
    #
    #             min_x = min(x_coordinates)
    #             max_x = max(x_coordinates)
    #             min_y = min(y_coordinates)
    #             max_y = max(y_coordinates)
    #             angle = average_angle / len(group)
    #
    #             if angle > 0:
    #                 merged_lines += [LineSegment(x1=min_x,
    #                                              y1=min_y, x2=max_x, y2=max_y)]
    #             else:
    #                 merged_lines += [LineSegment(x1=min_x,
    #                                              y1=max_y, x2=max_x, y2=min_y)]
    #
    #     return merged_lines

    def lsd(self, max_distance_apart=10, min_length=15):
        # Find all line segment with OpenCV's LSD
        lines = self.__OPENCV_LSD.detect(self.frame)[0]

        lines_s=[]
        if lines is not None:
            for line in lines:
                self.line_segments.append(LineSegment(line[0][0], line[0][1], line[0][2], line[0][3]))
            lines_s.sort(key=lambda x: x.angle)

            self.line_segments = self.__merge_line_by_distance_and_angle(self.line_segments, 3, 10)
            self.line_segments = self.__merge_line_by_distance_and_angle(self.line_segments, 3, 20)

            self.line_segments = self.__filter_by_length(self.line_segments, 10)

            self.line_segments = self.__merge_line_by_distance_and_angle(self.line_segments, 35, 5)
            self.line_segments = self.__filter_by_length(self.line_segments, 40)
            self.line_segments = self.__merge_parallel_line(self.line_segments, 60, 15)


            # # Filter out lines that do not meet min_length condition
            # self.line_segments = self.__filter_by_length(
            #     lines, min_length=min_length)
            #
            # # Classify the lines by angle
            # horizontal, vertical = self.__categorize_by_angle(
            #     self.line_segments)
            #
            # # Classify the lines by distance apart
            # horizontal = self.__categorize_by_distance_apart(
            #     horizontal, max_distance_apart, self.frame.shape[0])
            # vertical = self.__categorize_by_distance_apart(
            #     vertical, max_distance_apart, self.frame.shape[0])
            #
            # # Find average line in classified groups
            # horizontal = self.__merge_grouped_lines__(horizontal)
            # vertical = self.__merge_grouped_lines__(vertical)
            #
            # self.horizontal = horizontal
            # self.vertical = vertical

    @staticmethod
    def classify_corner_shape(line, point, max_distance_apart):
        # Determine whether corner is L shaped or T shaped
        distance_apart = min(LineSegment.distance(x1=line.x1, y1=line.y1, x2=point[0], y2=point[1]),
                             LineSegment.distance(x1=line.x2, y1=line.y2, x2=point[0], y2=point[1]))

        return Corner.CornerType.l_shape if distance_apart <= max_distance_apart else Corner.CornerType.t_shape

    # @staticmethod
    # def __find_corners(lines, reference_lines, max_distance_apart):
    #     l_shape_corners = []
    #     t_shape_corners = []
    #
    #     remaining_reference_lines = list(reference_lines)
    #
    #     for reference_line in reference_lines:
    #         for line in lines:
    #             distance_pt1 = reference_line.distance_to_point(
    #                 pt_x=line.x1, pt_y=line.y1)
    #             distance_pt2 = reference_line.distance_to_point(
    #                 pt_x=line.x2, pt_y=line.y2)
    #             distance_apart = min(distance_pt1, distance_pt2)
    #
    #             # Find the shortest distance between the reference line and two endpoints of line
    #             # Classify as corner if distance_apart is shorter than max_distance_apart
    #             if distance_apart < max_distance_apart:
    #                 (pt_x, pt_y) = (line.x1, line.y1) if distance_apart == distance_pt1 else (
    #                     line.x2, line.y2)
    #                 corner_type = LineSegmentDetectorMat.classify_corner_shape(line=reference_line, point=(pt_x, pt_y),
    #                                                                            max_distance_apart=max_distance_apart)
    #                 if corner_type == Corner.CornerType.l_shape:
    #                     l_shape_corners += [Corner(x=pt_x, y=pt_y,
    #                                                line1=line, line2=reference_line)]
    #                     if reference_line in remaining_reference_lines:
    #                         remaining_reference_lines.remove(reference_line)
    #                 else:
    #                     t_shape_corners += [Corner(x=pt_x,
    #                                                y=pt_y, line1=reference_line)]
    #                     if reference_line in remaining_reference_lines:
    #                         remaining_reference_lines.remove(reference_line)
    #
    #     return l_shape_corners, t_shape_corners, remaining_reference_lines
    #



    # def find_corners(self, max_distance_apart=5):
    #     self.field_lines = []
    #
    #     # Find all possible T and L shape corners
    #     horizontal_l_shape, horizontal_t_shape, extra_vertical = self.__find_corners(lines=self.horizontal,
    #                                                                                  reference_lines=self.vertical,
    #                                                                                  max_distance_apart=max_distance_apart)
    #     vertical_l_shape, vertical_t_shape, extra_horizontal = self.__find_corners(lines=self.vertical,
    #                                                                                reference_lines=self.horizontal,
    #                                                                                max_distance_apart=max_distance_apart)
    #
    #     # Merge L shape corners
    #     l_shape_corners = []
    #     for horizontal in horizontal_l_shape:
    #         for vertical in vertical_l_shape:
    #             distance_apart = LineSegment.distance(
    #                 x1=horizontal.x, y1=horizontal.y, x2=vertical.x, y2=vertical.y)
    #             if distance_apart < max_distance_apart:
    #                 found_matching_line = False
    #
    #                 for existing_l_shape in l_shape_corners:
    #                     if existing_l_shape.has_line(horizontal.line1):
    #                         existing_l_shape.add_l_corner(horizontal.line2)
    #                         found_matching_line = True
    #                     elif existing_l_shape.has_line(horizontal.line2):
    #                         existing_l_shape.add_l_corner(horizontal.line1)
    #                         found_matching_line = True
    #
    #                 if not found_matching_line:
    #                     l_shape_corners += [FieldLines(
    #                         lines=[horizontal.line1, horizontal.line2], l_corners=1)]
    #
    #     # Merge T shape corners
    #     t_shape_corners = []
    #     for corner in horizontal_t_shape + vertical_t_shape:
    #         found_matching_line = False
    #
    #         for existing_t_shape in t_shape_corners:
    #             if existing_t_shape.has_line(corner.line1):
    #                 existing_t_shape.add_t_corner()
    #                 found_matching_line = True
    #
    #         if not found_matching_line:
    #             t_shape_corners += [FieldLines(
    #                 lines=[corner.line1], t_corners=1)]
    #
    #     # Merge T shape corners with L shape corners into lines
    #     for l_shape in l_shape_corners:
    #         for t_shape in list(t_shape_corners):
    #             if l_shape.has_line(t_shape.lines[0]):
    #                 l_shape.add_t_corner()
    #                 t_shape_corners.remove(t_shape)
    #
    #     self.field_lines = l_shape_corners + t_shape_corners
    #     self.extra_lines = extra_horizontal + extra_vertical

    def classify_lines(self):
        all_line = []
        boundary_line = []
        goal_area_line = []
        center_line = []
        curve_line = []
        undefined_lines = []
        centerOrGoalLines = []
        goalOrTouchLines = []
        l_shape_line = []


        all_line = list(self.line_segments)



        #find curve line: curve line if two angle between three are large and three line are connected
        max_dist_to_connect = 60
        for line in all_line:
            for reference_line in all_line:

                comparingLine1 = ComparingLines(line, reference_line)
                angleDifference1 = comparingLine1.angleDifference

                if angleDifference1 > 10 and angleDifference1 < 50 and \
                        min(
                            comparingLine1.end_point_dist()) < max_dist_to_connect and comparingLine1.length_difference() < 100:

                    for reference_line_2 in  all_line:

                        comparingLine2 = ComparingLines(reference_line_2, line)
                        angleDifference2 = comparingLine2.angleDifference
                        if angleDifference2 < 30 and angleDifference2 > 10 and \
                                min(
                                    comparingLine2.end_point_dist()) < max_dist_to_connect and comparingLine2.length_difference() < 100:
                            if reference_line_2 not in curve_line:
                                curve_line.append(reference_line_2)
                            if reference_line not in curve_line:
                                curve_line.append(reference_line)
                            if line not in curve_line:
                                curve_line.append(line)



        #find the T-shape and L-shape
        #the top of T-shape must be boundary line
        #the L-shape could be goal or boundary line
        max_distance_apart = 40
        for line in all_line:
            for reference_line in all_line:
                comparingLine = ComparingLines(line, reference_line)
                angleDifference = comparingLine.angleDifference

                distance_pt1 = reference_line.distance_to_point(
                    pt_x=line.x1, pt_y=line.y1)
                distance_pt2 = reference_line.distance_to_point(
                    pt_x=line.x2, pt_y=line.y2)
                distance_apart = min(distance_pt1, distance_pt2)
                if distance_apart < max_distance_apart and angleDifference > 50 and angleDifference < 140:
                    (pt_x, pt_y) = (line.x1, line.y1) if distance_apart == distance_pt1 else (
                        line.x2, line.y2)
                    corner_type = self.classify_corner_shape(line=reference_line, point=(pt_x, pt_y),
                                                        max_distance_apart=max_distance_apart)

                    if corner_type == Corner.CornerType.l_shape:          #if L-shape
                        if line not in l_shape_line:
                            l_shape_line.append(line)
                        if reference_line not in l_shape_line:
                            l_shape_line.append(reference_line)
                        if line not in goalOrTouchLines:
                            goalOrTouchLines.append(line)
                        if reference_line not in goalOrTouchLines:
                            goalOrTouchLines.append(reference_line)

                    else:                                                   #if T-shape

                        if line not in curve_line and line.length > 60:
                            if line not in centerOrGoalLines:
                                centerOrGoalLines.append(line)
                            if reference_line not in boundary_line:   #the top line of the T shape corner must be boundary_line
                                boundary_line.append(reference_line)

        #find goal line
        #if the line have both T-shape and L-shape feature
        for goalOrTouchLine in goalOrTouchLines:
            if goalOrTouchLine in centerOrGoalLines:
                goal_area_line.append(goalOrTouchLine)

        max_distance_apart = 50


        #find L corner is connected boundary_line
        for line in boundary_line:
            for reference_line in all_line:
                comparingLine = ComparingLines(line, reference_line)
                angleDifference = comparingLine.angleDifference

                distance_pt1 = reference_line.distance_to_point(
                    pt_x=line.x1, pt_y=line.y1)
                distance_pt2 = reference_line.distance_to_point(
                    pt_x=line.x2, pt_y=line.y2)
                distance_apart = min(distance_pt1, distance_pt2)
                cv_line = line.to_cv_line()
                cv_reference_line = reference_line.to_cv_line
                if distance_apart < max_distance_apart and angleDifference > 40 and angleDifference < 140:
                    (pt_x, pt_y) = (line.x1, line.y1) if distance_apart == distance_pt1 else (
                        line.x2, line.y2)
                    corner_type = self.classify_corner_shape(line=reference_line, point=(pt_x, pt_y),
                                                        max_distance_apart=max_distance_apart)

                    if corner_type == Corner.CornerType.l_shape:
                        if reference_line not in boundary_line:
                            boundary_line.append(reference_line)

                elif angleDifference < 15 and angleDifference > 165:
                    if cv_line[0][1] + cv_line[1][1] > cv_reference_line[0][1] + cv_reference_line[1][1]:
                        goal_area_line.append(reference_line)
                        boundary_line.remove(reference_line)
                    else:
                        goal_area_line.append(line)
                        boundary_line.remove(line)



        max_distance_apart = 20

        # find goal_line:
        # if the line is connect to L and T shape angle
        for line in goal_area_line:
            for reference_line in all_line:
                comparingLine = ComparingLines(line, reference_line)
                angleDifference = comparingLine.angleDifference

                distance_pt1 = reference_line.distance_to_point(
                    pt_x=line.x1, pt_y=line.y1)
                distance_pt2 = reference_line.distance_to_point(
                    pt_x=line.x2, pt_y=line.y2)
                distance_apart = min(distance_pt1, distance_pt2)
                if distance_apart < max_distance_apart and angleDifference > 40 and angleDifference < 140:
                    (pt_x, pt_y) = (line.x1, line.y1) if distance_apart == distance_pt1 else (
                        line.x2, line.y2)
                    corner_type = self.classify_corner_shape(line=reference_line, point=(pt_x, pt_y),
                                                        max_distance_apart=max_distance_apart)

                    if corner_type == Corner.CornerType.l_shape:

                        if reference_line not in goal_area_line:
                            goal_area_line.append(reference_line)

        # find parallel line if not boundary line was found:
        # the farther line is boundary_line
        #the closer line is goal line
        if (len(boundary_line) <2):
            for line in all_line:
                done = False
                for reference_line in all_line:
                    comparingLine = ComparingLines(line, reference_line)
                    angleDiff = comparingLine.angleDifference
                    end_point_dist = comparingLine.end_point_dist()

                    if (angleDiff < 30 or angleDiff > 150) and min(end_point_dist)>50 and\
                            math.fabs(line.length / reference_line.length) < 2 and not (line == reference_line):
                        line1 = line.to_cv_line()
                        line2 = reference_line.to_cv_line()

                        #if line y-position greatern than reference line
                        if (line1[0][1] + line1[1][1]) > (line2[0][1] + line2[1][1]):
                            if line not in goal_area_line:
                                goal_area_line.append(line)
                            if reference_line not in boundary_line:
                                boundary_line.append(reference_line)
                            done = True
                            break
                        else:
                            if line not in boundary_line:
                                boundary_line.append(line)
                            if reference_line not in goal_area_line:
                                goal_area_line.append(reference_line)
                                done = True
                            break
                if done:
                    break

        #make the return value for the msg
        return_boundary_line = []
        return_goal_area_line = []
        return_center_line = []
        return_undefined_lines = []
        return_curve_line = []
        return_l_shape_line = []


        for b_line in boundary_line:
            return_boundary_line += [([b_line], 'boundary_line')]

        for g_line in goal_area_line:
            return_goal_area_line += [([g_line], 'goal_area_line')]

        for c_line in center_line:
            return_center_line += [([c_line], 'center_line')]

        for l_line in l_shape_line:
            if l_line not in curve_line and  l_line not in center_line and  l_line not in goal_area_line and  l_line not in boundary_line:
                return_l_shape_line += [([l_line], 'l_shpae')]

        for line in all_line:
            if line not in boundary_line and line not in goal_area_line and line not in center_line and line not in curve_line and line not in l_shape_line:
                 return_undefined_lines += [([line], 'undefined_lines')]

        for curve_line in curve_line:
            return_curve_line += [([curve_line], 'undefined_lines')]


        # for line_set in l_shape:
        #     for line in line_set[0]:
        #         if line in boundary_line or line in goal_area_line or line in center_line or line in curve_line:
        #             l_shape.remove(line_set)
        return {
                'boundary': return_boundary_line,
                'goal_area': return_goal_area_line,
                'center': return_center_line,
                'undefined': return_undefined_lines,
                'curve': return_curve_line,
                'l_shape': return_l_shape_line,
            }
