import math
import cv2 as cv
import rospy
import numpy as np
from std_msgs.msg import String

from base.mat import Mat
from vision.msg import ArrowType

from base.base_object import BaseObject


class Marker(BaseObject):

    def __init__(self, configuration, module):
        super(Marker, self).__init__(
            configuration = configuration,
            module = module
        )

        self.min_area = configuration['min_area']
        self.max_area = configuration['max_area']
        self.output_colour = configuration['output_colour']

        self.pub = rospy.Publisher(
            '/vision_node/' + self.name, String, queue_size=0)

    def export_configuration(self):
        configuration = super(Marker, self).export_configuration()
        configuration.update({
            'min_area': self.min_area,
            'max_area': self.max_area,
            'output_colour': np.array(self.output_colour)
        })
        return configuration

    def publish_msg(self, arrow_type):
        self.pub.publish(arrow_type)