import cv2 as cv
import numpy as np

import rospy
from vision.msg import PositionArea

from base.base_object import BaseObject


class Obstacle(BaseObject):

    def __init__(self, configuration, module):
        super(Obstacle, self).__init__(
            configuration=configuration, module=module)

        self.min_area = configuration['min_area']
        self.max_area = configuration['max_area']
        self.output_colour = configuration['output_colour']

        self.pub = rospy.Publisher(
            '/vision_node/' + self.name, PositionArea, queue_size=1)

    def export_configuration(self):
        configuration = super(Obstacle, self).export_configuration()
        configuration.update({
            'min_area': self.min_area,
            'max_area': self.max_area,
            'output_colour': np.array(self.output_colour)
        })
        return configuration

    def publish_msg(self, x, y, position, area, angle):
        message = PositionArea()
        message.x = x
        message.y = y
        message.position = position
        message.area = area
        message.angle = angle
        self.pub.publish(message)
