import math

import cv2 as cv
import numpy as np

from base.mat import Mat


class ObstacleMat(Mat):

    def __init__(self, frame):
        super(ObstacleMat, self).__init__(frame=frame)

    def extract_obstacle(self, thresh_lb, thresh_ub, kernel_size=3, blur_ksize=3):
        self.extract_object(thresh_lb=thresh_lb,
                            thresh_ub=thresh_ub, kernel_size=kernel_size, blur_ksize=blur_ksize)

    def find_angle(self, contour):
        rect = cv.minAreaRect(contour)
        box = cv.boxPoints(rect)
        box = np.int0(box)
        angle = rect[2]
        # taken from https://stackoverflow.com/questions/15956124/minarearect-angles-unsure-about-the-angle-returned
        if rect[1][0] < rect[1][1]: # convert to negative and positive
            angle = 180 + angle
        else:
            angle = 90 + angle

        return angle, box


    def bounding_rect(self, min_area, max_area):
        # Find contours
        _, contours, hierarchy = cv.findContours(
            self.frame, mode=cv.RETR_EXTERNAL, method=cv.CHAIN_APPROX_SIMPLE)

        if contours:
            positions = []
            position_text = ''
            box = None
            angle = -100
            contours = sorted(contours, key=cv.contourArea, reverse=True)

            for contour in contours:
                if min_area <= cv.contourArea(contour) <= max_area:
                    x1, y1, w, h = cv.boundingRect(contour)
                    x2, y2 = x1+w, y1+h
                    mid_x, mid_y = (x2 - x1) / 2.0, (y2 - y1) / 2.0
                    area = cv.contourArea(contour)
                    
                    angle, box = self.find_angle(contour)

                    # Check for distance between self and obstacle
                    position_text += '_far_'

                    # Check for direction between self and obstacle
                    left_bound = self.width * (1.0/4.0)
                    right_bound = self.width * (3.0/4.0)

                    if x2 <= left_bound:
                        position_text += 'left'
                    elif x1 <= left_bound and x2 <= right_bound:
                        position_text += 'left_center'
                    elif x1 <= left_bound and x2 > right_bound:
                        position_text += 'left_center_right'
                    elif x1 > left_bound and x1 <= right_bound and x2 > left_bound and x2 <= right_bound:
                        position_text += 'center'
                    elif x1 > left_bound and x1 <= right_bound and x2 > right_bound:
                        position_text += 'center_right'
                    elif x1 > right_bound:
                        position_text += 'right'

                    positions += [((x1, y1), (x2, y2))]

        return (positions, position_text, cv.contourArea(contours[0]), angle, box) if contours else (None, 'none', -1, -100, None)
